# Jabs Mimir

**Jabs Mimir** is a lightweight, extensible UI micro-framework built on top of `tkinter` and `ttkbootstrap`, designed for rapid internal tool development and structured form workflows.

It provides:

- Reusable UI primitives with validation and tooltips
- Support for block-based form components with dynamic variable binding
- Integration with custom validation logic
- Modular architecture suitable for internal tooling and small boilerplate projects

---

## Installation

```bash
pip install jabs-mimir
```

---

## Quick Start

```python
from jabs_mimir import Mimir, DataBlockWrapper, UtilityModule
import tkinter as tk
import ttkbootstrap as tb

class App(tb.Window):
    def __init__(self):
        super().__init__(title="Mimir Demo")
        self.ui = Mimir(self)
        self.ui.setValidatorResolver(lambda name: {"not_empty": lambda val: bool(val.strip())}.get(name))

        self.ui.switchView(self.mainView)

    def mainView(self, ui, *args):
        frame = tb.Frame(self)
        fields = [
            {"type": "heading", "label": "Basic Info"},
            {"label": "Name", "key": "name", "variable": tk.StringVar(), "validation": "not_empty"},
            {"label": "Age", "key": "age", "variable": tk.IntVar()}
        ]

        meta = UtilityModule.buildBlockMeta(fields)
        block = DataBlockWrapper(meta)

        ui.renderFields(frame, fields)
        ui.addNextButton(frame, row=len(fields)+1, command=lambda: print(UtilityModule.getBlockValues(block)))

        return frame

if __name__ == "__main__":
    app = App()
    app.mainloop()
```

---

## Components

### `Mimir`
Manages UI views, tooltips, validation, field rendering, and form logic.

### `DataBlockWrapper`
A wrapper for block-level form metadata and values. Supports dot-access and `.get()`/`.set()` calls.

### `UtilityModule`
Helper methods for building field metadata, extracting values, validating blocks, etc.

---

## License

MIT License © 2025 William Lydahl
<details>
<summary><strong>📘 API Documentation</strong></summary>

<!doctype html>
<html lang="en">
<head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta name="generator" content="pdoc 15.0.1"/>
    <title>jabs_mimir API documentation</title>

    <style>/*! * Bootstrap Reboot v5.0.0 (https://getbootstrap.com/) * Copyright 2011-2021 The Bootstrap Authors * Copyright 2011-2021 Twitter, Inc. * Licensed under MIT (https://github.com/twbs/bootstrap/blob/main/LICENSE) * Forked from Normalize.css, licensed MIT (https://github.com/necolas/normalize.css/blob/master/LICENSE.md) */*,::after,::before{box-sizing:border-box}@media (prefers-reduced-motion:no-preference){:root{scroll-behavior:smooth}}body{margin:0;font-family:system-ui,-apple-system,"Segoe UI",Roboto,"Helvetica Neue",Arial,"Noto Sans","Liberation Sans",sans-serif,"Apple Color Emoji","Segoe UI Emoji","Segoe UI Symbol","Noto Color Emoji";font-size:1rem;font-weight:400;line-height:1.5;color:#212529;background-color:#fff;-webkit-text-size-adjust:100%;-webkit-tap-highlight-color:transparent}hr{margin:1rem 0;color:inherit;background-color:currentColor;border:0;opacity:.25}hr:not([size]){height:1px}h1,h2,h3,h4,h5,h6{margin-top:0;margin-bottom:.5rem;font-weight:500;line-height:1.2}h1{font-size:calc(1.375rem + 1.5vw)}@media (min-width:1200px){h1{font-size:2.5rem}}h2{font-size:calc(1.325rem + .9vw)}@media (min-width:1200px){h2{font-size:2rem}}h3{font-size:calc(1.3rem + .6vw)}@media (min-width:1200px){h3{font-size:1.75rem}}h4{font-size:calc(1.275rem + .3vw)}@media (min-width:1200px){h4{font-size:1.5rem}}h5{font-size:1.25rem}h6{font-size:1rem}p{margin-top:0;margin-bottom:1rem}abbr[data-bs-original-title],abbr[title]{-webkit-text-decoration:underline dotted;text-decoration:underline dotted;cursor:help;-webkit-text-decoration-skip-ink:none;text-decoration-skip-ink:none}address{margin-bottom:1rem;font-style:normal;line-height:inherit}ol,ul{padding-left:2rem}dl,ol,ul{margin-top:0;margin-bottom:1rem}ol ol,ol ul,ul ol,ul ul{margin-bottom:0}dt{font-weight:700}dd{margin-bottom:.5rem;margin-left:0}blockquote{margin:0 0 1rem}b,strong{font-weight:bolder}small{font-size:.875em}mark{padding:.2em;background-color:#fcf8e3}sub,sup{position:relative;font-size:.75em;line-height:0;vertical-align:baseline}sub{bottom:-.25em}sup{top:-.5em}a{color:#0d6efd;text-decoration:underline}a:hover{color:#0a58ca}a:not([href]):not([class]),a:not([href]):not([class]):hover{color:inherit;text-decoration:none}code,kbd,pre,samp{font-family:SFMono-Regular,Menlo,Monaco,Consolas,"Liberation Mono","Courier New",monospace;font-size:1em;direction:ltr;unicode-bidi:bidi-override}pre{display:block;margin-top:0;margin-bottom:1rem;overflow:auto;font-size:.875em}pre code{font-size:inherit;color:inherit;word-break:normal}code{font-size:.875em;color:#d63384;word-wrap:break-word}a>code{color:inherit}kbd{padding:.2rem .4rem;font-size:.875em;color:#fff;background-color:#212529;border-radius:.2rem}kbd kbd{padding:0;font-size:1em;font-weight:700}figure{margin:0 0 1rem}img,svg{vertical-align:middle}table{caption-side:bottom;border-collapse:collapse}caption{padding-top:.5rem;padding-bottom:.5rem;color:#6c757d;text-align:left}th{text-align:inherit;text-align:-webkit-match-parent}tbody,td,tfoot,th,thead,tr{border-color:inherit;border-style:solid;border-width:0}label{display:inline-block}button{border-radius:0}button:focus:not(:focus-visible){outline:0}button,input,optgroup,select,textarea{margin:0;font-family:inherit;font-size:inherit;line-height:inherit}button,select{text-transform:none}[role=button]{cursor:pointer}select{word-wrap:normal}select:disabled{opacity:1}[list]::-webkit-calendar-picker-indicator{display:none}[type=button],[type=reset],[type=submit],button{-webkit-appearance:button}[type=button]:not(:disabled),[type=reset]:not(:disabled),[type=submit]:not(:disabled),button:not(:disabled){cursor:pointer}::-moz-focus-inner{padding:0;border-style:none}textarea{resize:vertical}fieldset{min-width:0;padding:0;margin:0;border:0}legend{float:left;width:100%;padding:0;margin-bottom:.5rem;font-size:calc(1.275rem + .3vw);line-height:inherit}@media (min-width:1200px){legend{font-size:1.5rem}}legend+*{clear:left}::-webkit-datetime-edit-day-field,::-webkit-datetime-edit-fields-wrapper,::-webkit-datetime-edit-hour-field,::-webkit-datetime-edit-minute,::-webkit-datetime-edit-month-field,::-webkit-datetime-edit-text,::-webkit-datetime-edit-year-field{padding:0}::-webkit-inner-spin-button{height:auto}[type=search]{outline-offset:-2px;-webkit-appearance:textfield}::-webkit-search-decoration{-webkit-appearance:none}::-webkit-color-swatch-wrapper{padding:0}::file-selector-button{font:inherit}::-webkit-file-upload-button{font:inherit;-webkit-appearance:button}output{display:inline-block}iframe{border:0}summary{display:list-item;cursor:pointer}progress{vertical-align:baseline}[hidden]{display:none!important}</style>
    <style>/*! syntax-highlighting.css */pre{line-height:125%;}span.linenos{color:inherit; background-color:transparent; padding-left:5px; padding-right:20px;}.pdoc-code .hll{background-color:#ffffcc}.pdoc-code{background:#f8f8f8;}.pdoc-code .c{color:#3D7B7B; font-style:italic}.pdoc-code .err{border:1px solid #FF0000}.pdoc-code .k{color:#008000; font-weight:bold}.pdoc-code .o{color:#666666}.pdoc-code .ch{color:#3D7B7B; font-style:italic}.pdoc-code .cm{color:#3D7B7B; font-style:italic}.pdoc-code .cp{color:#9C6500}.pdoc-code .cpf{color:#3D7B7B; font-style:italic}.pdoc-code .c1{color:#3D7B7B; font-style:italic}.pdoc-code .cs{color:#3D7B7B; font-style:italic}.pdoc-code .gd{color:#A00000}.pdoc-code .ge{font-style:italic}.pdoc-code .gr{color:#E40000}.pdoc-code .gh{color:#000080; font-weight:bold}.pdoc-code .gi{color:#008400}.pdoc-code .go{color:#717171}.pdoc-code .gp{color:#000080; font-weight:bold}.pdoc-code .gs{font-weight:bold}.pdoc-code .gu{color:#800080; font-weight:bold}.pdoc-code .gt{color:#0044DD}.pdoc-code .kc{color:#008000; font-weight:bold}.pdoc-code .kd{color:#008000; font-weight:bold}.pdoc-code .kn{color:#008000; font-weight:bold}.pdoc-code .kp{color:#008000}.pdoc-code .kr{color:#008000; font-weight:bold}.pdoc-code .kt{color:#B00040}.pdoc-code .m{color:#666666}.pdoc-code .s{color:#BA2121}.pdoc-code .na{color:#687822}.pdoc-code .nb{color:#008000}.pdoc-code .nc{color:#0000FF; font-weight:bold}.pdoc-code .no{color:#880000}.pdoc-code .nd{color:#AA22FF}.pdoc-code .ni{color:#717171; font-weight:bold}.pdoc-code .ne{color:#CB3F38; font-weight:bold}.pdoc-code .nf{color:#0000FF}.pdoc-code .nl{color:#767600}.pdoc-code .nn{color:#0000FF; font-weight:bold}.pdoc-code .nt{color:#008000; font-weight:bold}.pdoc-code .nv{color:#19177C}.pdoc-code .ow{color:#AA22FF; font-weight:bold}.pdoc-code .w{color:#bbbbbb}.pdoc-code .mb{color:#666666}.pdoc-code .mf{color:#666666}.pdoc-code .mh{color:#666666}.pdoc-code .mi{color:#666666}.pdoc-code .mo{color:#666666}.pdoc-code .sa{color:#BA2121}.pdoc-code .sb{color:#BA2121}.pdoc-code .sc{color:#BA2121}.pdoc-code .dl{color:#BA2121}.pdoc-code .sd{color:#BA2121; font-style:italic}.pdoc-code .s2{color:#BA2121}.pdoc-code .se{color:#AA5D1F; font-weight:bold}.pdoc-code .sh{color:#BA2121}.pdoc-code .si{color:#A45A77; font-weight:bold}.pdoc-code .sx{color:#008000}.pdoc-code .sr{color:#A45A77}.pdoc-code .s1{color:#BA2121}.pdoc-code .ss{color:#19177C}.pdoc-code .bp{color:#008000}.pdoc-code .fm{color:#0000FF}.pdoc-code .vc{color:#19177C}.pdoc-code .vg{color:#19177C}.pdoc-code .vi{color:#19177C}.pdoc-code .vm{color:#19177C}.pdoc-code .il{color:#666666}</style>
    <style>/*! theme.css */:root{--pdoc-background:#fff;}.pdoc{--text:#212529;--muted:#6c757d;--link:#3660a5;--link-hover:#1659c5;--code:#f8f8f8;--active:#fff598;--accent:#eee;--accent2:#c1c1c1;--nav-hover:rgba(255, 255, 255, 0.5);--name:#0066BB;--def:#008800;--annotation:#007020;}</style>
    <style>/*! layout.css */html, body{width:100%;height:100%;}html, main{scroll-behavior:smooth;}body{background-color:var(--pdoc-background);}@media (max-width:769px){#navtoggle{cursor:pointer;position:absolute;width:50px;height:40px;top:1rem;right:1rem;border-color:var(--text);color:var(--text);display:flex;opacity:0.8;z-index:999;}#navtoggle:hover{opacity:1;}#togglestate + div{display:none;}#togglestate:checked + div{display:inherit;}main, header{padding:2rem 3vw;}header + main{margin-top:-3rem;}.git-button{display:none !important;}nav input[type="search"]{max-width:77%;}nav input[type="search"]:first-child{margin-top:-6px;}nav input[type="search"]:valid ~ *{display:none !important;}}@media (min-width:770px){:root{--sidebar-width:clamp(12.5rem, 28vw, 22rem);}nav{position:fixed;overflow:auto;height:100vh;width:var(--sidebar-width);}main, header{padding:3rem 2rem 3rem calc(var(--sidebar-width) + 3rem);width:calc(54rem + var(--sidebar-width));max-width:100%;}header + main{margin-top:-4rem;}#navtoggle{display:none;}}#togglestate{position:absolute;height:0;opacity:0;}nav.pdoc{--pad:clamp(0.5rem, 2vw, 1.75rem);--indent:1.5rem;background-color:var(--accent);border-right:1px solid var(--accent2);box-shadow:0 0 20px rgba(50, 50, 50, .2) inset;padding:0 0 0 var(--pad);overflow-wrap:anywhere;scrollbar-width:thin; scrollbar-color:var(--accent2) transparent; z-index:1}nav.pdoc::-webkit-scrollbar{width:.4rem; }nav.pdoc::-webkit-scrollbar-thumb{background-color:var(--accent2); }nav.pdoc > div{padding:var(--pad) 0;}nav.pdoc .module-list-button{display:inline-flex;align-items:center;color:var(--text);border-color:var(--muted);margin-bottom:1rem;}nav.pdoc .module-list-button:hover{border-color:var(--text);}nav.pdoc input[type=search]{display:block;outline-offset:0;width:calc(100% - var(--pad));}nav.pdoc .logo{max-width:calc(100% - var(--pad));max-height:35vh;display:block;margin:0 auto 1rem;transform:translate(calc(-.5 * var(--pad)), 0);}nav.pdoc ul{list-style:none;padding-left:0;}nav.pdoc > div > ul{margin-left:calc(0px - var(--pad));}nav.pdoc li a{padding:.2rem 0 .2rem calc(var(--pad) + var(--indent));}nav.pdoc > div > ul > li > a{padding-left:var(--pad);}nav.pdoc li{transition:all 100ms;}nav.pdoc li:hover{background-color:var(--nav-hover);}nav.pdoc a, nav.pdoc a:hover{color:var(--text);}nav.pdoc a{display:block;}nav.pdoc > h2:first-of-type{margin-top:1.5rem;}nav.pdoc .class:before{content:"class ";color:var(--muted);}nav.pdoc .function:after{content:"()";color:var(--muted);}nav.pdoc footer:before{content:"";display:block;width:calc(100% - var(--pad));border-top:solid var(--accent2) 1px;margin-top:1.5rem;padding-top:.5rem;}nav.pdoc footer{font-size:small;}</style>
    <style>/*! content.css */.pdoc{color:var(--text);box-sizing:border-box;line-height:1.5;background:none;}.pdoc .pdoc-button{cursor:pointer;display:inline-block;border:solid black 1px;border-radius:2px;font-size:.75rem;padding:calc(0.5em - 1px) 1em;transition:100ms all;}.pdoc .alert{padding:1rem 1rem 1rem calc(1.5rem + 24px);border:1px solid transparent;border-radius:.25rem;background-repeat:no-repeat;background-position:.75rem center;margin-bottom:1rem;}.pdoc .alert > em{display:none;}.pdoc .alert > *:last-child{margin-bottom:0;}.pdoc .alert.note {color:#084298;background-color:#cfe2ff;border-color:#b6d4fe;background-image:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A//www.w3.org/2000/svg%22%20width%3D%2224%22%20height%3D%2224%22%20fill%3D%22%23084298%22%20viewBox%3D%220%200%2016%2016%22%3E%3Cpath%20d%3D%22M8%2016A8%208%200%201%200%208%200a8%208%200%200%200%200%2016zm.93-9.412-1%204.705c-.07.34.029.533.304.533.194%200%20.487-.07.686-.246l-.088.416c-.287.346-.92.598-1.465.598-.703%200-1.002-.422-.808-1.319l.738-3.468c.064-.293.006-.399-.287-.47l-.451-.081.082-.381%202.29-.287zM8%205.5a1%201%200%201%201%200-2%201%201%200%200%201%200%202z%22/%3E%3C/svg%3E");}.pdoc .alert.warning{color:#664d03;background-color:#fff3cd;border-color:#ffecb5;background-image:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A//www.w3.org/2000/svg%22%20width%3D%2224%22%20height%3D%2224%22%20fill%3D%22%23664d03%22%20viewBox%3D%220%200%2016%2016%22%3E%3Cpath%20d%3D%22M8.982%201.566a1.13%201.13%200%200%200-1.96%200L.165%2013.233c-.457.778.091%201.767.98%201.767h13.713c.889%200%201.438-.99.98-1.767L8.982%201.566zM8%205c.535%200%20.954.462.9.995l-.35%203.507a.552.552%200%200%201-1.1%200L7.1%205.995A.905.905%200%200%201%208%205zm.002%206a1%201%200%201%201%200%202%201%201%200%200%201%200-2z%22/%3E%3C/svg%3E");}.pdoc .alert.danger{color:#842029;background-color:#f8d7da;border-color:#f5c2c7;background-image:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A//www.w3.org/2000/svg%22%20width%3D%2224%22%20height%3D%2224%22%20fill%3D%22%23842029%22%20viewBox%3D%220%200%2016%2016%22%3E%3Cpath%20d%3D%22M5.52.359A.5.5%200%200%201%206%200h4a.5.5%200%200%201%20.474.658L8.694%206H12.5a.5.5%200%200%201%20.395.807l-7%209a.5.5%200%200%201-.873-.454L6.823%209.5H3.5a.5.5%200%200%201-.48-.641l2.5-8.5z%22/%3E%3C/svg%3E");}.pdoc .visually-hidden{position:absolute !important;width:1px !important;height:1px !important;padding:0 !important;margin:-1px !important;overflow:hidden !important;clip:rect(0, 0, 0, 0) !important;white-space:nowrap !important;border:0 !important;}.pdoc h1, .pdoc h2, .pdoc h3{font-weight:300;margin:.3em 0;padding:.2em 0;}.pdoc > section:not(.module-info) h1{font-size:1.5rem;font-weight:500;}.pdoc > section:not(.module-info) h2{font-size:1.4rem;font-weight:500;}.pdoc > section:not(.module-info) h3{font-size:1.3rem;font-weight:500;}.pdoc > section:not(.module-info) h4{font-size:1.2rem;}.pdoc > section:not(.module-info) h5{font-size:1.1rem;}.pdoc a{text-decoration:none;color:var(--link);}.pdoc a:hover{color:var(--link-hover);}.pdoc blockquote{margin-left:2rem;}.pdoc pre{border-top:1px solid var(--accent2);border-bottom:1px solid var(--accent2);margin-top:0;margin-bottom:1em;padding:.5rem 0 .5rem .5rem;overflow-x:auto;background-color:var(--code);}.pdoc code{color:var(--text);padding:.2em .4em;margin:0;font-size:85%;background-color:var(--accent);border-radius:6px;}.pdoc a > code{color:inherit;}.pdoc pre > code{display:inline-block;font-size:inherit;background:none;border:none;padding:0;}.pdoc > section:not(.module-info){margin-bottom:1.5rem;}.pdoc .modulename{margin-top:0;font-weight:bold;}.pdoc .modulename a{color:var(--link);transition:100ms all;}.pdoc .git-button{float:right;border:solid var(--link) 1px;}.pdoc .git-button:hover{background-color:var(--link);color:var(--pdoc-background);}.view-source-toggle-state,.view-source-toggle-state ~ .pdoc-code{display:none;}.view-source-toggle-state:checked ~ .pdoc-code{display:block;}.view-source-button{display:inline-block;float:right;font-size:.75rem;line-height:1.5rem;color:var(--muted);padding:0 .4rem 0 1.3rem;cursor:pointer;text-indent:-2px;}.view-source-button > span{visibility:hidden;}.module-info .view-source-button{float:none;display:flex;justify-content:flex-end;margin:-1.2rem .4rem -.2rem 0;}.view-source-button::before{position:absolute;content:"View Source";display:list-item;list-style-type:disclosure-closed;}.view-source-toggle-state:checked ~ .attr .view-source-button::before,.view-source-toggle-state:checked ~ .view-source-button::before{list-style-type:disclosure-open;}.pdoc .docstring{margin-bottom:1.5rem;}.pdoc section:not(.module-info) .docstring{margin-left:clamp(0rem, 5vw - 2rem, 1rem);}.pdoc .docstring .pdoc-code{margin-left:1em;margin-right:1em;}.pdoc h1:target,.pdoc h2:target,.pdoc h3:target,.pdoc h4:target,.pdoc h5:target,.pdoc h6:target,.pdoc .pdoc-code > pre > span:target{background-color:var(--active);box-shadow:-1rem 0 0 0 var(--active);}.pdoc .pdoc-code > pre > span:target{display:block;}.pdoc div:target > .attr,.pdoc section:target > .attr,.pdoc dd:target > a{background-color:var(--active);}.pdoc *{scroll-margin:2rem;}.pdoc .pdoc-code .linenos{user-select:none;}.pdoc .attr:hover{filter:contrast(0.95);}.pdoc section, .pdoc .classattr{position:relative;}.pdoc .headerlink{--width:clamp(1rem, 3vw, 2rem);position:absolute;top:0;left:calc(0rem - var(--width));transition:all 100ms ease-in-out;opacity:0;}.pdoc .headerlink::before{content:"#";display:block;text-align:center;width:var(--width);height:2.3rem;line-height:2.3rem;font-size:1.5rem;}.pdoc .attr:hover ~ .headerlink,.pdoc *:target > .headerlink,.pdoc .headerlink:hover{opacity:1;}.pdoc .attr{display:block;margin:.5rem 0 .5rem;padding:.4rem .4rem .4rem 1rem;background-color:var(--accent);overflow-x:auto;}.pdoc .classattr{margin-left:2rem;}.pdoc .decorator-deprecated{color:#842029;}.pdoc .decorator-deprecated ~ span{filter:grayscale(1) opacity(0.8);}.pdoc .name{color:var(--name);font-weight:bold;}.pdoc .def{color:var(--def);font-weight:bold;}.pdoc .signature{background-color:transparent;}.pdoc .param, .pdoc .return-annotation{white-space:pre;}.pdoc .signature.multiline .param{display:block;}.pdoc .signature.condensed .param{display:inline-block;}.pdoc .annotation{color:var(--annotation);}.pdoc .view-value-toggle-state,.pdoc .view-value-toggle-state ~ .default_value{display:none;}.pdoc .view-value-toggle-state:checked ~ .default_value{display:inherit;}.pdoc .view-value-button{font-size:.5rem;vertical-align:middle;border-style:dashed;margin-top:-0.1rem;}.pdoc .view-value-button:hover{background:white;}.pdoc .view-value-button::before{content:"show";text-align:center;width:2.2em;display:inline-block;}.pdoc .view-value-toggle-state:checked ~ .view-value-button::before{content:"hide";}.pdoc .inherited{margin-left:2rem;}.pdoc .inherited dt{font-weight:700;}.pdoc .inherited dt, .pdoc .inherited dd{display:inline;margin-left:0;margin-bottom:.5rem;}.pdoc .inherited dd:not(:last-child):after{content:", ";}.pdoc .inherited .class:before{content:"class ";}.pdoc .inherited .function a:after{content:"()";}.pdoc .search-result .docstring{overflow:auto;max-height:25vh;}.pdoc .search-result.focused > .attr{background-color:var(--active);}.pdoc .attribution{margin-top:2rem;display:block;opacity:0.5;transition:all 200ms;filter:grayscale(100%);}.pdoc .attribution:hover{opacity:1;filter:grayscale(0%);}.pdoc .attribution img{margin-left:5px;height:35px;vertical-align:middle;width:70px;transition:all 200ms;}.pdoc table{display:block;width:max-content;max-width:100%;overflow:auto;margin-bottom:1rem;}.pdoc table th{font-weight:600;}.pdoc table th, .pdoc table td{padding:6px 13px;border:1px solid var(--accent2);}</style>
    <style>/*! custom.css */</style></head>
<body>
    <nav class="pdoc">
        <label id="navtoggle" for="togglestate" class="pdoc-button"><svg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 30 30'><path stroke-linecap='round' stroke="currentColor" stroke-miterlimit='10' stroke-width='2' d='M4 7h22M4 15h22M4 23h22'/></svg></label>
        <input id="togglestate" type="checkbox" aria-hidden="true" tabindex="-1">
        <div>

            <input type="search" placeholder="Search..." role="searchbox" aria-label="search"
                   pattern=".+" required>


            <h2>Submodules</h2>
            <ul>
                    <li><a href="jabs_mimir/Mimir.html">Mimir</a></li>
                    <li><a href="jabs_mimir/DataBlockWrapper.html">DataBlockWrapper</a></li>
                    <li><a href="jabs_mimir/UtilityModule.html">UtilityModule</a></li>
            </ul>

            <h2>API Documentation</h2>
                <ul class="memberlist">
            <li>
                    <a class="class" href="#Mimir">Mimir</a>
                            <ul class="memberlist">
                        <li>
                                <a class="function" href="#Mimir.__init__">Mimir</a>
                        </li>
                        <li>
                                <a class="variable" href="#Mimir.widthEntry">widthEntry</a>
                        </li>
                        <li>
                                <a class="variable" href="#Mimir.widthCombo">widthCombo</a>
                        </li>
                        <li>
                                <a class="variable" href="#Mimir.app">app</a>
                        </li>
                        <li>
                                <a class="variable" href="#Mimir.currentView">currentView</a>
                        </li>
                        <li>
                                <a class="variable" href="#Mimir.invalidEntries">invalidEntries</a>
                        </li>
                        <li>
                                <a class="variable" href="#Mimir.fieldRenderers">fieldRenderers</a>
                        </li>
                        <li>
                                <a class="function" href="#Mimir.allowDarkMode">allowDarkMode</a>
                        </li>
                        <li>
                                <a class="function" href="#Mimir.setValidatorResolver">setValidatorResolver</a>
                        </li>
                        <li>
                                <a class="function" href="#Mimir.switchView">switchView</a>
                        </li>
                        <li>
                                <a class="function" href="#Mimir.popupView">popupView</a>
                        </li>
                        <li>
                                <a class="function" href="#Mimir.renderBlockUI">renderBlockUI</a>
                        </li>
                        <li>
                                <a class="function" href="#Mimir.registerFieldType">registerFieldType</a>
                        </li>
                        <li>
                                <a class="function" href="#Mimir.markInvalid">markInvalid</a>
                        </li>
                        <li>
                                <a class="function" href="#Mimir.clearInvalid">clearInvalid</a>
                        </li>
                        <li>
                                <a class="function" href="#Mimir.addNextButton">addNextButton</a>
                        </li>
                        <li>
                                <a class="function" href="#Mimir.getCurrentFont">getCurrentFont</a>
                        </li>
                        <li>
                                <a class="function" href="#Mimir.addTooltip">addTooltip</a>
                        </li>
                        <li>
                                <a class="function" href="#Mimir.removeBlock">removeBlock</a>
                        </li>
                        <li>
                                <a class="function" href="#Mimir.addLabeledEntry">addLabeledEntry</a>
                        </li>
                        <li>
                                <a class="function" href="#Mimir.addLabeledCombobox">addLabeledCombobox</a>
                        </li>
                        <li>
                                <a class="function" href="#Mimir.getNextAvailableRow">getNextAvailableRow</a>
                        </li>
                </ul>

            </li>
            <li>
                    <a class="class" href="#DataBlockWrapper">DataBlockWrapper</a>
                            <ul class="memberlist">
                        <li>
                                <a class="function" href="#DataBlockWrapper.__init__">DataBlockWrapper</a>
                        </li>
                        <li>
                                <a class="variable" href="#DataBlockWrapper.meta">meta</a>
                        </li>
                        <li>
                                <a class="function" href="#DataBlockWrapper.get">get</a>
                        </li>
                        <li>
                                <a class="function" href="#DataBlockWrapper.set">set</a>
                        </li>
                        <li>
                                <a class="function" href="#DataBlockWrapper.items">items</a>
                        </li>
                        <li>
                                <a class="function" href="#DataBlockWrapper.values">values</a>
                        </li>
                        <li>
                                <a class="function" href="#DataBlockWrapper.keys">keys</a>
                        </li>
                </ul>

            </li>
            <li>
                    <a class="class" href="#UtilityModule">UtilityModule</a>
                            <ul class="memberlist">
                        <li>
                                <a class="function" href="#UtilityModule.buildBlockMeta">buildBlockMeta</a>
                        </li>
                        <li>
                                <a class="function" href="#UtilityModule.getBlockValues">getBlockValues</a>
                        </li>
                        <li>
                                <a class="function" href="#UtilityModule.updateBlockFromDict">updateBlockFromDict</a>
                        </li>
                        <li>
                                <a class="function" href="#UtilityModule.isBlockValid">isBlockValid</a>
                        </li>
                </ul>

            </li>
    </ul>



        <a class="attribution" title="pdoc: Python API documentation generator" href="https://pdoc.dev" target="_blank">
            built with <span class="visually-hidden">pdoc</span><img
                alt="pdoc logo"
                src="data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A//www.w3.org/2000/svg%22%20role%3D%22img%22%20aria-label%3D%22pdoc%20logo%22%20width%3D%22300%22%20height%3D%22150%22%20viewBox%3D%22-1%200%2060%2030%22%3E%3Ctitle%3Epdoc%3C/title%3E%3Cpath%20d%3D%22M29.621%2021.293c-.011-.273-.214-.475-.511-.481a.5.5%200%200%200-.489.503l-.044%201.393c-.097.551-.695%201.215-1.566%201.704-.577.428-1.306.486-2.193.182-1.426-.617-2.467-1.654-3.304-2.487l-.173-.172a3.43%203.43%200%200%200-.365-.306.49.49%200%200%200-.286-.196c-1.718-1.06-4.931-1.47-7.353.191l-.219.15c-1.707%201.187-3.413%202.131-4.328%201.03-.02-.027-.49-.685-.141-1.763.233-.721.546-2.408.772-4.076.042-.09.067-.187.046-.288.166-1.347.277-2.625.241-3.351%201.378-1.008%202.271-2.586%202.271-4.362%200-.976-.272-1.935-.788-2.774-.057-.094-.122-.18-.184-.268.033-.167.052-.339.052-.516%200-1.477-1.202-2.679-2.679-2.679-.791%200-1.496.352-1.987.9a6.3%206.3%200%200%200-1.001.029c-.492-.564-1.207-.929-2.012-.929-1.477%200-2.679%201.202-2.679%202.679A2.65%202.65%200%200%200%20.97%206.554c-.383.747-.595%201.572-.595%202.41%200%202.311%201.507%204.29%203.635%205.107-.037.699-.147%202.27-.423%203.294l-.137.461c-.622%202.042-2.515%208.257%201.727%2010.643%201.614.908%203.06%201.248%204.317%201.248%202.665%200%204.492-1.524%205.322-2.401%201.476-1.559%202.886-1.854%206.491.82%201.877%201.393%203.514%201.753%204.861%201.068%202.223-1.713%202.811-3.867%203.399-6.374.077-.846.056-1.469.054-1.537zm-4.835%204.313c-.054.305-.156.586-.242.629-.034-.007-.131-.022-.307-.157-.145-.111-.314-.478-.456-.908.221.121.432.25.675.355.115.039.219.051.33.081zm-2.251-1.238c-.05.33-.158.648-.252.694-.022.001-.125-.018-.307-.157-.217-.166-.488-.906-.639-1.573.358.344.754.693%201.198%201.036zm-3.887-2.337c-.006-.116-.018-.231-.041-.342.635.145%201.189.368%201.599.625.097.231.166.481.174.642-.03.049-.055.101-.067.158-.046.013-.128.026-.298.004-.278-.037-.901-.57-1.367-1.087zm-1.127-.497c.116.306.176.625.12.71-.019.014-.117.045-.345.016-.206-.027-.604-.332-.986-.695.41-.051.816-.056%201.211-.031zm-4.535%201.535c.209.22.379.47.358.598-.006.041-.088.138-.351.234-.144.055-.539-.063-.979-.259a11.66%2011.66%200%200%200%20.972-.573zm.983-.664c.359-.237.738-.418%201.126-.554.25.237.479.548.457.694-.006.042-.087.138-.351.235-.174.064-.694-.105-1.232-.375zm-3.381%201.794c-.022.145-.061.29-.149.401-.133.166-.358.248-.69.251h-.002c-.133%200-.306-.26-.45-.621.417.091.854.07%201.291-.031zm-2.066-8.077a4.78%204.78%200%200%201-.775-.584c.172-.115.505-.254.88-.378l-.105.962zm-.331%202.302a10.32%2010.32%200%200%201-.828-.502c.202-.143.576-.328.984-.49l-.156.992zm-.45%202.157l-.701-.403c.214-.115.536-.249.891-.376a11.57%2011.57%200%200%201-.19.779zm-.181%201.716c.064.398.194.702.298.893-.194-.051-.435-.162-.736-.398.061-.119.224-.3.438-.495zM8.87%204.141c0%20.152-.123.276-.276.276s-.275-.124-.275-.276.123-.276.276-.276.275.124.275.276zm-.735-.389a1.15%201.15%200%200%200-.314.783%201.16%201.16%200%200%200%201.162%201.162c.457%200%20.842-.27%201.032-.653.026.117.042.238.042.362a1.68%201.68%200%200%201-1.679%201.679%201.68%201.68%200%200%201-1.679-1.679c0-.843.626-1.535%201.436-1.654zM5.059%205.406A1.68%201.68%200%200%201%203.38%207.085a1.68%201.68%200%200%201-1.679-1.679c0-.037.009-.072.011-.109.21.3.541.508.935.508a1.16%201.16%200%200%200%201.162-1.162%201.14%201.14%200%200%200-.474-.912c.015%200%20.03-.005.045-.005.926.001%201.679.754%201.679%201.68zM3.198%204.141c0%20.152-.123.276-.276.276s-.275-.124-.275-.276.123-.276.276-.276.275.124.275.276zM1.375%208.964c0-.52.103-1.035.288-1.52.466.394%201.06.64%201.717.64%201.144%200%202.116-.725%202.499-1.738.383%201.012%201.355%201.738%202.499%201.738.867%200%201.631-.421%202.121-1.062.307.605.478%201.267.478%201.942%200%202.486-2.153%204.51-4.801%204.51s-4.801-2.023-4.801-4.51zm24.342%2019.349c-.985.498-2.267.168-3.813-.979-3.073-2.281-5.453-3.199-7.813-.705-1.315%201.391-4.163%203.365-8.423.97-3.174-1.786-2.239-6.266-1.261-9.479l.146-.492c.276-1.02.395-2.457.444-3.268a6.11%206.11%200%200%200%201.18.115%206.01%206.01%200%200%200%202.536-.562l-.006.175c-.802.215-1.848.612-2.021%201.25-.079.295.021.601.274.837.219.203.415.364.598.501-.667.304-1.243.698-1.311%201.179-.02.144-.022.507.393.787.213.144.395.26.564.365-1.285.521-1.361.96-1.381%201.126-.018.142-.011.496.427.746l.854.489c-.473.389-.971.914-.999%201.429-.018.278.095.532.316.713.675.556%201.231.721%201.653.721.059%200%20.104-.014.158-.02.207.707.641%201.64%201.513%201.64h.013c.8-.008%201.236-.345%201.462-.626.173-.216.268-.457.325-.692.424.195.93.374%201.372.374.151%200%20.294-.021.423-.068.732-.27.944-.704.993-1.021.009-.061.003-.119.002-.179.266.086.538.147.789.147.15%200%20.294-.021.423-.069.542-.2.797-.489.914-.754.237.147.478.258.704.288.106.014.205.021.296.021.356%200%20.595-.101.767-.229.438.435%201.094.992%201.656%201.067.106.014.205.021.296.021a1.56%201.56%200%200%200%20.323-.035c.17.575.453%201.289.866%201.605.358.273.665.362.914.362a.99.99%200%200%200%20.421-.093%201.03%201.03%200%200%200%20.245-.164c.168.428.39.846.68%201.068.358.273.665.362.913.362a.99.99%200%200%200%20.421-.093c.317-.148.512-.448.639-.762.251.157.495.257.726.257.127%200%20.25-.024.37-.071.427-.17.706-.617.841-1.314.022-.015.047-.022.068-.038.067-.051.133-.104.196-.159-.443%201.486-1.107%202.761-2.086%203.257zM8.66%209.925a.5.5%200%201%200-1%200c0%20.653-.818%201.205-1.787%201.205s-1.787-.552-1.787-1.205a.5.5%200%201%200-1%200c0%201.216%201.25%202.205%202.787%202.205s2.787-.989%202.787-2.205zm4.4%2015.965l-.208.097c-2.661%201.258-4.708%201.436-6.086.527-1.542-1.017-1.88-3.19-1.844-4.198a.4.4%200%200%200-.385-.414c-.242-.029-.406.164-.414.385-.046%201.249.367%203.686%202.202%204.896.708.467%201.547.7%202.51.7%201.248%200%202.706-.392%204.362-1.174l.185-.086a.4.4%200%200%200%20.205-.527c-.089-.204-.326-.291-.527-.206zM9.547%202.292c.093.077.205.114.317.114a.5.5%200%200%200%20.318-.886L8.817.397a.5.5%200%200%200-.703.068.5.5%200%200%200%20.069.703l1.364%201.124zm-7.661-.065c.086%200%20.173-.022.253-.068l1.523-.893a.5.5%200%200%200-.506-.863l-1.523.892a.5.5%200%200%200-.179.685c.094.158.261.247.432.247z%22%20transform%3D%22matrix%28-1%200%200%201%2058%200%29%22%20fill%3D%22%233bb300%22/%3E%3Cpath%20d%3D%22M.3%2021.86V10.18q0-.46.02-.68.04-.22.18-.5.28-.54%201.34-.54%201.06%200%201.42.28.38.26.44.78.76-1.04%202.38-1.04%201.64%200%203.1%201.54%201.46%201.54%201.46%203.58%200%202.04-1.46%203.58-1.44%201.54-3.08%201.54-1.64%200-2.38-.92v4.04q0%20.46-.04.68-.02.22-.18.5-.14.3-.5.42-.36.12-.98.12-.62%200-1-.12-.36-.12-.52-.4-.14-.28-.18-.5-.02-.22-.02-.68zm3.96-9.42q-.46.54-.46%201.18%200%20.64.46%201.18.48.52%201.2.52.74%200%201.24-.52.52-.52.52-1.18%200-.66-.48-1.18-.48-.54-1.26-.54-.76%200-1.22.54zm14.741-8.36q.16-.3.54-.42.38-.12%201-.12.64%200%201.02.12.38.12.52.42.16.3.18.54.04.22.04.68v11.94q0%20.46-.04.7-.02.22-.18.5-.3.54-1.7.54-1.38%200-1.54-.98-.84.96-2.34.96-1.8%200-3.28-1.56-1.48-1.58-1.48-3.66%200-2.1%201.48-3.68%201.5-1.58%203.28-1.58%201.48%200%202.3%201v-4.2q0-.46.02-.68.04-.24.18-.52zm-3.24%2010.86q.52.54%201.26.54.74%200%201.22-.54.5-.54.5-1.18%200-.66-.48-1.22-.46-.56-1.26-.56-.8%200-1.28.56-.48.54-.48%201.2%200%20.66.52%201.2zm7.833-1.2q0-2.4%201.68-3.96%201.68-1.56%203.84-1.56%202.16%200%203.82%201.56%201.66%201.54%201.66%203.94%200%201.66-.86%202.96-.86%201.28-2.1%201.9-1.22.6-2.54.6-1.32%200-2.56-.64-1.24-.66-2.1-1.92-.84-1.28-.84-2.88zm4.18%201.44q.64.48%201.3.48.66%200%201.32-.5.66-.5.66-1.48%200-.98-.62-1.46-.62-.48-1.34-.48-.72%200-1.34.5-.62.5-.62%201.48%200%20.96.64%201.46zm11.412-1.44q0%20.84.56%201.32.56.46%201.18.46.64%200%201.18-.36.56-.38.9-.38.6%200%201.46%201.06.46.58.46%201.04%200%20.76-1.1%201.42-1.14.8-2.8.8-1.86%200-3.58-1.34-.82-.64-1.34-1.7-.52-1.08-.52-2.36%200-1.3.52-2.34.52-1.06%201.34-1.7%201.66-1.32%203.54-1.32.76%200%201.48.22.72.2%201.06.4l.32.2q.36.24.56.38.52.4.52.92%200%20.5-.42%201.14-.72%201.1-1.38%201.1-.38%200-1.08-.44-.36-.34-1.04-.34-.66%200-1.24.48-.58.48-.58%201.34z%22%20fill%3D%22green%22/%3E%3C/svg%3E"/>
        </a>
</div>
    </nav>
    <main class="pdoc">
            <section class="module-info">
                    <h1 class="modulename">
jabs_mimir    </h1>

                
                        <input id="mod-jabs_mimir-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">

                        <label class="view-source-button" for="mod-jabs_mimir-view-source"><span>View Source</span></label>

                        <div class="pdoc-code codehilite"><pre><span></span><span id="L-1"><a href="#L-1"><span class="linenos">1</span></a><span class="kn">from</span><span class="w"> </span><span class="nn">.Mimir</span><span class="w"> </span><span class="kn">import</span> <span class="n">Mimir</span>
</span><span id="L-2"><a href="#L-2"><span class="linenos">2</span></a><span class="kn">from</span><span class="w"> </span><span class="nn">.DataBlockWrapper</span><span class="w"> </span><span class="kn">import</span> <span class="n">DataBlockWrapper</span>
</span><span id="L-3"><a href="#L-3"><span class="linenos">3</span></a><span class="kn">from</span><span class="w"> </span><span class="nn">.UtilityModule</span><span class="w"> </span><span class="kn">import</span> <span class="n">UtilityModule</span>
</span><span id="L-4"><a href="#L-4"><span class="linenos">4</span></a>
</span><span id="L-5"><a href="#L-5"><span class="linenos">5</span></a><span class="n">__all__</span> <span class="o">=</span> <span class="p">[</span><span class="s2">&quot;Mimir&quot;</span><span class="p">,</span> <span class="s2">&quot;DataBlockWrapper&quot;</span><span class="p">,</span> <span class="s2">&quot;UtilityModule&quot;</span><span class="p">]</span>
</span></pre></div>


            </section>
                <section id="Mimir">
                            <input id="Mimir-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr class">
            
    <span class="def">class</span>
    <span class="name">Mimir</span>:

                <label class="view-source-button" for="Mimir-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir-16"><a href="#Mimir-16"><span class="linenos"> 16</span></a><span class="k">class</span><span class="w"> </span><span class="nc">Mimir</span><span class="p">:</span>
</span><span id="Mimir-17"><a href="#Mimir-17"><span class="linenos"> 17</span></a>    <span class="n">widthEntry</span> <span class="o">=</span> <span class="mi">20</span>
</span><span id="Mimir-18"><a href="#Mimir-18"><span class="linenos"> 18</span></a>    <span class="n">widthCombo</span> <span class="o">=</span> <span class="n">widthEntry</span> <span class="o">-</span> <span class="mi">2</span>
</span><span id="Mimir-19"><a href="#Mimir-19"><span class="linenos"> 19</span></a>
</span><span id="Mimir-20"><a href="#Mimir-20"><span class="linenos"> 20</span></a>    <span class="k">def</span><span class="w"> </span><span class="fm">__init__</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">app</span><span class="p">):</span>
</span><span id="Mimir-21"><a href="#Mimir-21"><span class="linenos"> 21</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir-22"><a href="#Mimir-22"><span class="linenos"> 22</span></a><span class="sd">        Initialize Mimir with reference to the root app window.</span>
</span><span id="Mimir-23"><a href="#Mimir-23"><span class="linenos"> 23</span></a><span class="sd">        &quot;&quot;&quot;</span>
</span><span id="Mimir-24"><a href="#Mimir-24"><span class="linenos"> 24</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">app</span> <span class="o">=</span> <span class="n">app</span>
</span><span id="Mimir-25"><a href="#Mimir-25"><span class="linenos"> 25</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">currentView</span> <span class="o">=</span> <span class="kc">None</span>
</span><span id="Mimir-26"><a href="#Mimir-26"><span class="linenos"> 26</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">invalidEntries</span> <span class="o">=</span> <span class="nb">set</span><span class="p">()</span>
</span><span id="Mimir-27"><a href="#Mimir-27"><span class="linenos"> 27</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">_validatorResolver</span> <span class="o">=</span> <span class="k">lambda</span> <span class="n">name</span><span class="p">:</span> <span class="kc">None</span>
</span><span id="Mimir-28"><a href="#Mimir-28"><span class="linenos"> 28</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">fieldRenderers</span> <span class="o">=</span> <span class="p">{</span>
</span><span id="Mimir-29"><a href="#Mimir-29"><span class="linenos"> 29</span></a>            <span class="s2">&quot;entry&quot;</span><span class="p">:</span> <span class="bp">self</span><span class="o">.</span><span class="n">_renderEntryField</span><span class="p">,</span>
</span><span id="Mimir-30"><a href="#Mimir-30"><span class="linenos"> 30</span></a>            <span class="s2">&quot;combobox&quot;</span><span class="p">:</span> <span class="bp">self</span><span class="o">.</span><span class="n">_renderComboboxField</span><span class="p">,</span>
</span><span id="Mimir-31"><a href="#Mimir-31"><span class="linenos"> 31</span></a>            <span class="s2">&quot;heading&quot;</span><span class="p">:</span> <span class="bp">self</span><span class="o">.</span><span class="n">_renderHeading</span><span class="p">,</span>
</span><span id="Mimir-32"><a href="#Mimir-32"><span class="linenos"> 32</span></a>        <span class="p">}</span>
</span><span id="Mimir-33"><a href="#Mimir-33"><span class="linenos"> 33</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">_allowThemeToggle</span> <span class="o">=</span> <span class="kc">False</span>
</span><span id="Mimir-34"><a href="#Mimir-34"><span class="linenos"> 34</span></a>
</span><span id="Mimir-35"><a href="#Mimir-35"><span class="linenos"> 35</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">allowDarkMode</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">enabled</span><span class="o">=</span><span class="kc">True</span><span class="p">):</span>
</span><span id="Mimir-36"><a href="#Mimir-36"><span class="linenos"> 36</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Enable or disable theme toggle button in the top right.&quot;&quot;&quot;</span>
</span><span id="Mimir-37"><a href="#Mimir-37"><span class="linenos"> 37</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">_allowThemeToggle</span> <span class="o">=</span> <span class="n">enabled</span>
</span><span id="Mimir-38"><a href="#Mimir-38"><span class="linenos"> 38</span></a>
</span><span id="Mimir-39"><a href="#Mimir-39"><span class="linenos"> 39</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">_toggleTheme</span><span class="p">(</span><span class="bp">self</span><span class="p">):</span>
</span><span id="Mimir-40"><a href="#Mimir-40"><span class="linenos"> 40</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Switch between light and dark themes.&quot;&quot;&quot;</span>
</span><span id="Mimir-41"><a href="#Mimir-41"><span class="linenos"> 41</span></a>        <span class="n">current</span> <span class="o">=</span> <span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="o">.</span><span class="n">style</span><span class="o">.</span><span class="n">theme</span><span class="o">.</span><span class="n">name</span>
</span><span id="Mimir-42"><a href="#Mimir-42"><span class="linenos"> 42</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="o">.</span><span class="n">style</span><span class="o">.</span><span class="n">theme_use</span><span class="p">(</span><span class="s2">&quot;darkly&quot;</span> <span class="k">if</span> <span class="n">current</span> <span class="o">!=</span> <span class="s2">&quot;darkly&quot;</span> <span class="k">else</span> <span class="s2">&quot;cosmo&quot;</span><span class="p">)</span>
</span><span id="Mimir-43"><a href="#Mimir-43"><span class="linenos"> 43</span></a>
</span><span id="Mimir-44"><a href="#Mimir-44"><span class="linenos"> 44</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">setValidatorResolver</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">resolverFunc</span><span class="p">):</span>
</span><span id="Mimir-45"><a href="#Mimir-45"><span class="linenos"> 45</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Register a function that returns validation callbacks by name.&quot;&quot;&quot;</span>
</span><span id="Mimir-46"><a href="#Mimir-46"><span class="linenos"> 46</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">_validatorResolver</span> <span class="o">=</span> <span class="n">resolverFunc</span>
</span><span id="Mimir-47"><a href="#Mimir-47"><span class="linenos"> 47</span></a>
</span><span id="Mimir-48"><a href="#Mimir-48"><span class="linenos"> 48</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">switchView</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">newFrameFunc</span><span class="p">,</span> <span class="n">gridOptions</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="o">**</span><span class="n">kwargs</span><span class="p">):</span>
</span><span id="Mimir-49"><a href="#Mimir-49"><span class="linenos"> 49</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir-50"><a href="#Mimir-50"><span class="linenos"> 50</span></a><span class="sd">        Replace the current view with a new frame built by newFrameFunc.</span>
</span><span id="Mimir-51"><a href="#Mimir-51"><span class="linenos"> 51</span></a><span class="sd">        Automatically configures geometry and grid placement.</span>
</span><span id="Mimir-52"><a href="#Mimir-52"><span class="linenos"> 52</span></a><span class="sd">        &quot;&quot;&quot;</span>        
</span><span id="Mimir-53"><a href="#Mimir-53"><span class="linenos"> 53</span></a>        <span class="k">if</span> <span class="bp">self</span><span class="o">.</span><span class="n">currentView</span><span class="p">:</span>
</span><span id="Mimir-54"><a href="#Mimir-54"><span class="linenos"> 54</span></a>            <span class="bp">self</span><span class="o">.</span><span class="n">currentView</span><span class="o">.</span><span class="n">destroy</span><span class="p">()</span>
</span><span id="Mimir-55"><a href="#Mimir-55"><span class="linenos"> 55</span></a>
</span><span id="Mimir-56"><a href="#Mimir-56"><span class="linenos"> 56</span></a>        <span class="n">container</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Frame</span><span class="p">(</span><span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="p">)</span>
</span><span id="Mimir-57"><a href="#Mimir-57"><span class="linenos"> 57</span></a>        <span class="n">container</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;nsew&quot;</span><span class="p">)</span>
</span><span id="Mimir-58"><a href="#Mimir-58"><span class="linenos"> 58</span></a>        <span class="n">container</span><span class="o">.</span><span class="n">columnconfigure</span><span class="p">(</span><span class="mi">0</span><span class="p">,</span> <span class="n">weight</span><span class="o">=</span><span class="mi">1</span><span class="p">)</span>
</span><span id="Mimir-59"><a href="#Mimir-59"><span class="linenos"> 59</span></a>
</span><span id="Mimir-60"><a href="#Mimir-60"><span class="linenos"> 60</span></a>        <span class="n">topBar</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Frame</span><span class="p">(</span><span class="n">container</span><span class="p">)</span>
</span><span id="Mimir-61"><a href="#Mimir-61"><span class="linenos"> 61</span></a>        <span class="n">topBar</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;ew&quot;</span><span class="p">)</span>
</span><span id="Mimir-62"><a href="#Mimir-62"><span class="linenos"> 62</span></a>
</span><span id="Mimir-63"><a href="#Mimir-63"><span class="linenos"> 63</span></a>        <span class="k">if</span> <span class="bp">self</span><span class="o">.</span><span class="n">_allowThemeToggle</span><span class="p">:</span>
</span><span id="Mimir-64"><a href="#Mimir-64"><span class="linenos"> 64</span></a>            <span class="n">tb</span><span class="o">.</span><span class="n">Button</span><span class="p">(</span><span class="n">topBar</span><span class="p">,</span> <span class="n">text</span><span class="o">=</span><span class="s2">&quot;Theme&quot;</span><span class="p">,</span> <span class="n">command</span><span class="o">=</span><span class="bp">self</span><span class="o">.</span><span class="n">_toggleTheme</span><span class="p">)</span><span class="o">.</span><span class="n">pack</span><span class="p">(</span><span class="n">side</span><span class="o">=</span><span class="s2">&quot;right&quot;</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">10</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="mi">10</span><span class="p">)</span>
</span><span id="Mimir-65"><a href="#Mimir-65"><span class="linenos"> 65</span></a>
</span><span id="Mimir-66"><a href="#Mimir-66"><span class="linenos"> 66</span></a>        <span class="n">newFrame</span> <span class="o">=</span> <span class="n">newFrameFunc</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="o">**</span><span class="n">kwargs</span><span class="p">)</span>
</span><span id="Mimir-67"><a href="#Mimir-67"><span class="linenos"> 67</span></a>        <span class="n">options</span> <span class="o">=</span> <span class="n">gridOptions</span> <span class="ow">or</span> <span class="p">{</span><span class="s2">&quot;row&quot;</span><span class="p">:</span> <span class="mi">1</span><span class="p">,</span> <span class="s2">&quot;column&quot;</span><span class="p">:</span> <span class="mi">0</span><span class="p">,</span> <span class="s2">&quot;sticky&quot;</span><span class="p">:</span> <span class="s2">&quot;n&quot;</span><span class="p">,</span> <span class="s2">&quot;padx&quot;</span><span class="p">:</span> <span class="mi">50</span><span class="p">,</span> <span class="s2">&quot;pady&quot;</span><span class="p">:</span> <span class="mi">20</span><span class="p">}</span>
</span><span id="Mimir-68"><a href="#Mimir-68"><span class="linenos"> 68</span></a>        <span class="n">newFrame</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="o">**</span><span class="n">options</span><span class="p">)</span>
</span><span id="Mimir-69"><a href="#Mimir-69"><span class="linenos"> 69</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="o">.</span><span class="n">rowconfigure</span><span class="p">(</span><span class="n">options</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;row&quot;</span><span class="p">,</span> <span class="mi">0</span><span class="p">),</span> <span class="n">weight</span><span class="o">=</span><span class="mi">1</span><span class="p">)</span>
</span><span id="Mimir-70"><a href="#Mimir-70"><span class="linenos"> 70</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="o">.</span><span class="n">columnconfigure</span><span class="p">(</span><span class="n">options</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;column&quot;</span><span class="p">,</span> <span class="mi">0</span><span class="p">),</span> <span class="n">weight</span><span class="o">=</span><span class="mi">1</span><span class="p">)</span>
</span><span id="Mimir-71"><a href="#Mimir-71"><span class="linenos"> 71</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">currentView</span> <span class="o">=</span> <span class="n">newFrame</span>
</span><span id="Mimir-72"><a href="#Mimir-72"><span class="linenos"> 72</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="o">.</span><span class="n">update_idletasks</span><span class="p">()</span>
</span><span id="Mimir-73"><a href="#Mimir-73"><span class="linenos"> 73</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="o">.</span><span class="n">geometry</span><span class="p">(</span><span class="s2">&quot;&quot;</span><span class="p">)</span>
</span><span id="Mimir-74"><a href="#Mimir-74"><span class="linenos"> 74</span></a>        <span class="k">return</span> <span class="n">newFrame</span>
</span><span id="Mimir-75"><a href="#Mimir-75"><span class="linenos"> 75</span></a>
</span><span id="Mimir-76"><a href="#Mimir-76"><span class="linenos"> 76</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">popupView</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">viewFunc</span><span class="p">,</span> <span class="n">title</span><span class="o">=</span><span class="s2">&quot;Popup&quot;</span><span class="p">,</span> <span class="n">size</span><span class="o">=</span><span class="s2">&quot;fit&quot;</span><span class="p">,</span> <span class="n">width</span><span class="o">=</span><span class="mi">500</span><span class="p">,</span> <span class="n">height</span><span class="o">=</span><span class="mi">400</span><span class="p">,</span> <span class="n">modal</span><span class="o">=</span><span class="kc">False</span><span class="p">):</span>
</span><span id="Mimir-77"><a href="#Mimir-77"><span class="linenos"> 77</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir-78"><a href="#Mimir-78"><span class="linenos"> 78</span></a><span class="sd">        Open a popup window and embed a view frame into it.</span>
</span><span id="Mimir-79"><a href="#Mimir-79"><span class="linenos"> 79</span></a><span class="sd">        If `modal=True`, interaction with other windows is blocked.</span>
</span><span id="Mimir-80"><a href="#Mimir-80"><span class="linenos"> 80</span></a><span class="sd">        &quot;&quot;&quot;</span>        
</span><span id="Mimir-81"><a href="#Mimir-81"><span class="linenos"> 81</span></a>        <span class="n">popup</span> <span class="o">=</span> <span class="n">tk</span><span class="o">.</span><span class="n">Toplevel</span><span class="p">(</span><span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="p">)</span>
</span><span id="Mimir-82"><a href="#Mimir-82"><span class="linenos"> 82</span></a>        <span class="n">popup</span><span class="o">.</span><span class="n">title</span><span class="p">(</span><span class="n">title</span><span class="p">)</span>
</span><span id="Mimir-83"><a href="#Mimir-83"><span class="linenos"> 83</span></a>        <span class="n">popup</span><span class="o">.</span><span class="n">transient</span><span class="p">(</span><span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="p">)</span>
</span><span id="Mimir-84"><a href="#Mimir-84"><span class="linenos"> 84</span></a>        <span class="n">popup</span><span class="o">.</span><span class="n">resizable</span><span class="p">(</span><span class="kc">False</span><span class="p">,</span> <span class="kc">False</span><span class="p">)</span>
</span><span id="Mimir-85"><a href="#Mimir-85"><span class="linenos"> 85</span></a>
</span><span id="Mimir-86"><a href="#Mimir-86"><span class="linenos"> 86</span></a>        <span class="k">if</span> <span class="n">size</span> <span class="o">==</span> <span class="s2">&quot;fit&quot;</span><span class="p">:</span>
</span><span id="Mimir-87"><a href="#Mimir-87"><span class="linenos"> 87</span></a>            <span class="n">popup</span><span class="o">.</span><span class="n">update_idletasks</span><span class="p">()</span>
</span><span id="Mimir-88"><a href="#Mimir-88"><span class="linenos"> 88</span></a>            <span class="n">popup</span><span class="o">.</span><span class="n">geometry</span><span class="p">(</span><span class="s2">&quot;&quot;</span><span class="p">)</span>
</span><span id="Mimir-89"><a href="#Mimir-89"><span class="linenos"> 89</span></a>        <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir-90"><a href="#Mimir-90"><span class="linenos"> 90</span></a>            <span class="n">popup</span><span class="o">.</span><span class="n">geometry</span><span class="p">(</span><span class="sa">f</span><span class="s2">&quot;</span><span class="si">{</span><span class="n">width</span><span class="si">}</span><span class="s2">x</span><span class="si">{</span><span class="n">height</span><span class="si">}</span><span class="s2">&quot;</span><span class="p">)</span>
</span><span id="Mimir-91"><a href="#Mimir-91"><span class="linenos"> 91</span></a>
</span><span id="Mimir-92"><a href="#Mimir-92"><span class="linenos"> 92</span></a>        <span class="n">popup</span><span class="o">.</span><span class="n">grid_rowconfigure</span><span class="p">(</span><span class="mi">0</span><span class="p">,</span> <span class="n">weight</span><span class="o">=</span><span class="mi">1</span><span class="p">)</span>
</span><span id="Mimir-93"><a href="#Mimir-93"><span class="linenos"> 93</span></a>        <span class="n">popup</span><span class="o">.</span><span class="n">grid_columnconfigure</span><span class="p">(</span><span class="mi">0</span><span class="p">,</span> <span class="n">weight</span><span class="o">=</span><span class="mi">1</span><span class="p">)</span>
</span><span id="Mimir-94"><a href="#Mimir-94"><span class="linenos"> 94</span></a>        <span class="n">viewFrame</span> <span class="o">=</span> <span class="n">viewFunc</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">popup</span><span class="p">)</span>
</span><span id="Mimir-95"><a href="#Mimir-95"><span class="linenos"> 95</span></a>        <span class="n">viewFrame</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;nsew&quot;</span><span class="p">)</span>
</span><span id="Mimir-96"><a href="#Mimir-96"><span class="linenos"> 96</span></a>
</span><span id="Mimir-97"><a href="#Mimir-97"><span class="linenos"> 97</span></a>        <span class="k">if</span> <span class="n">modal</span><span class="p">:</span>
</span><span id="Mimir-98"><a href="#Mimir-98"><span class="linenos"> 98</span></a>            <span class="n">popup</span><span class="o">.</span><span class="n">grab_set</span><span class="p">()</span>
</span><span id="Mimir-99"><a href="#Mimir-99"><span class="linenos"> 99</span></a>            <span class="n">popup</span><span class="o">.</span><span class="n">focus_set</span><span class="p">()</span>
</span><span id="Mimir-100"><a href="#Mimir-100"><span class="linenos">100</span></a>
</span><span id="Mimir-101"><a href="#Mimir-101"><span class="linenos">101</span></a>        <span class="k">return</span> <span class="n">popup</span>
</span><span id="Mimir-102"><a href="#Mimir-102"><span class="linenos">102</span></a>
</span><span id="Mimir-103"><a href="#Mimir-103"><span class="linenos">103</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">renderBlockUI</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">container</span><span class="p">,</span> <span class="n">fields</span><span class="p">,</span> <span class="n">blockList</span><span class="p">,</span> <span class="n">layout</span><span class="o">=</span><span class="s2">&quot;horizontal&quot;</span><span class="p">,</span> <span class="n">meta</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="n">label</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="n">noRemove</span><span class="o">=</span><span class="kc">False</span><span class="p">):</span>
</span><span id="Mimir-104"><a href="#Mimir-104"><span class="linenos">104</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir-105"><a href="#Mimir-105"><span class="linenos">105</span></a><span class="sd">        Render a labeled input block with specified fields and layout style.</span>
</span><span id="Mimir-106"><a href="#Mimir-106"><span class="linenos">106</span></a><span class="sd">        Automatically attaches the block to blockList and handles UI labels and deletion logic.</span>
</span><span id="Mimir-107"><a href="#Mimir-107"><span class="linenos">107</span></a><span class="sd">        &quot;&quot;&quot;</span>        
</span><span id="Mimir-108"><a href="#Mimir-108"><span class="linenos">108</span></a>        <span class="n">index</span> <span class="o">=</span> <span class="nb">len</span><span class="p">(</span><span class="n">blockList</span><span class="p">)</span> <span class="o">+</span> <span class="mi">1</span>
</span><span id="Mimir-109"><a href="#Mimir-109"><span class="linenos">109</span></a>        <span class="n">labelText</span> <span class="o">=</span> <span class="n">label</span> <span class="ow">or</span> <span class="sa">f</span><span class="s2">&quot;Block </span><span class="si">{</span><span class="n">index</span><span class="si">}</span><span class="s2">&quot;</span>
</span><span id="Mimir-110"><a href="#Mimir-110"><span class="linenos">110</span></a>        <span class="n">frame</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">LabelFrame</span><span class="p">(</span><span class="n">container</span><span class="p">,</span> <span class="n">text</span><span class="o">=</span><span class="n">labelText</span><span class="p">)</span>
</span><span id="Mimir-111"><a href="#Mimir-111"><span class="linenos">111</span></a>        <span class="n">frame</span><span class="o">.</span><span class="n">configure</span><span class="p">(</span><span class="n">labelanchor</span><span class="o">=</span><span class="s2">&quot;n&quot;</span><span class="p">)</span> 
</span><span id="Mimir-112"><a href="#Mimir-112"><span class="linenos">112</span></a>        
</span><span id="Mimir-113"><a href="#Mimir-113"><span class="linenos">113</span></a>        <span class="k">if</span> <span class="n">layout</span> <span class="o">==</span> <span class="s2">&quot;vertical&quot;</span><span class="p">:</span>
</span><span id="Mimir-114"><a href="#Mimir-114"><span class="linenos">114</span></a>            <span class="n">frame</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">index</span> <span class="o">-</span> <span class="mi">1</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">10</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="mi">10</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;w&quot;</span><span class="p">)</span>
</span><span id="Mimir-115"><a href="#Mimir-115"><span class="linenos">115</span></a>        <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir-116"><a href="#Mimir-116"><span class="linenos">116</span></a>            <span class="n">frame</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">index</span> <span class="o">-</span> <span class="mi">1</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">10</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="mi">10</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;n&quot;</span><span class="p">)</span>
</span><span id="Mimir-117"><a href="#Mimir-117"><span class="linenos">117</span></a>
</span><span id="Mimir-118"><a href="#Mimir-118"><span class="linenos">118</span></a>        <span class="n">boldFont</span> <span class="o">=</span> <span class="bp">self</span><span class="o">.</span><span class="n">getCurrentFont</span><span class="p">()</span>
</span><span id="Mimir-119"><a href="#Mimir-119"><span class="linenos">119</span></a>        <span class="n">boldFont</span><span class="o">.</span><span class="n">configure</span><span class="p">(</span><span class="n">weight</span><span class="o">=</span><span class="s2">&quot;bold&quot;</span><span class="p">)</span>
</span><span id="Mimir-120"><a href="#Mimir-120"><span class="linenos">120</span></a>
</span><span id="Mimir-121"><a href="#Mimir-121"><span class="linenos">121</span></a>        <span class="k">if</span> <span class="n">meta</span> <span class="ow">is</span> <span class="kc">None</span><span class="p">:</span>
</span><span id="Mimir-122"><a href="#Mimir-122"><span class="linenos">122</span></a>            <span class="n">meta</span> <span class="o">=</span> <span class="p">{}</span>
</span><span id="Mimir-123"><a href="#Mimir-123"><span class="linenos">123</span></a>        <span class="k">if</span> <span class="s2">&quot;custom_label&quot;</span> <span class="ow">not</span> <span class="ow">in</span> <span class="n">meta</span> <span class="ow">and</span> <span class="n">label</span><span class="p">:</span>
</span><span id="Mimir-124"><a href="#Mimir-124"><span class="linenos">124</span></a>            <span class="n">meta</span><span class="p">[</span><span class="s2">&quot;custom_label&quot;</span><span class="p">]</span> <span class="o">=</span> <span class="n">label</span>
</span><span id="Mimir-125"><a href="#Mimir-125"><span class="linenos">125</span></a>
</span><span id="Mimir-126"><a href="#Mimir-126"><span class="linenos">126</span></a>        <span class="n">blockMeta</span> <span class="o">=</span> <span class="p">{</span>
</span><span id="Mimir-127"><a href="#Mimir-127"><span class="linenos">127</span></a>            <span class="s2">&quot;frame&quot;</span><span class="p">:</span> <span class="n">frame</span><span class="p">,</span>
</span><span id="Mimir-128"><a href="#Mimir-128"><span class="linenos">128</span></a>            <span class="s2">&quot;fields&quot;</span><span class="p">:</span> <span class="n">fields</span><span class="p">,</span>
</span><span id="Mimir-129"><a href="#Mimir-129"><span class="linenos">129</span></a>            <span class="s2">&quot;fields_by_key&quot;</span><span class="p">:</span> <span class="p">{</span><span class="n">f</span><span class="p">[</span><span class="s2">&quot;key&quot;</span><span class="p">]:</span> <span class="n">f</span><span class="p">[</span><span class="s2">&quot;variable&quot;</span><span class="p">]</span> <span class="k">for</span> <span class="n">f</span> <span class="ow">in</span> <span class="n">fields</span> <span class="k">if</span> <span class="s2">&quot;key&quot;</span> <span class="ow">in</span> <span class="n">f</span><span class="p">},</span>
</span><span id="Mimir-130"><a href="#Mimir-130"><span class="linenos">130</span></a>            <span class="s2">&quot;layout&quot;</span><span class="p">:</span> <span class="n">layout</span><span class="p">,</span>
</span><span id="Mimir-131"><a href="#Mimir-131"><span class="linenos">131</span></a>            <span class="o">**</span><span class="n">meta</span>
</span><span id="Mimir-132"><a href="#Mimir-132"><span class="linenos">132</span></a>        <span class="p">}</span>
</span><span id="Mimir-133"><a href="#Mimir-133"><span class="linenos">133</span></a>
</span><span id="Mimir-134"><a href="#Mimir-134"><span class="linenos">134</span></a>
</span><span id="Mimir-135"><a href="#Mimir-135"><span class="linenos">135</span></a>        <span class="n">wrapper</span> <span class="o">=</span> <span class="n">DataBlockWrapper</span><span class="p">(</span><span class="n">blockMeta</span><span class="p">)</span>
</span><span id="Mimir-136"><a href="#Mimir-136"><span class="linenos">136</span></a>        <span class="n">blockList</span><span class="o">.</span><span class="n">append</span><span class="p">(</span><span class="n">wrapper</span><span class="p">)</span>
</span><span id="Mimir-137"><a href="#Mimir-137"><span class="linenos">137</span></a>
</span><span id="Mimir-138"><a href="#Mimir-138"><span class="linenos">138</span></a>        <span class="n">row</span> <span class="o">=</span> <span class="mi">0</span>
</span><span id="Mimir-139"><a href="#Mimir-139"><span class="linenos">139</span></a>        <span class="k">for</span> <span class="n">field</span> <span class="ow">in</span> <span class="n">fields</span><span class="p">:</span>
</span><span id="Mimir-140"><a href="#Mimir-140"><span class="linenos">140</span></a>            <span class="n">ftype</span> <span class="o">=</span> <span class="n">field</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;type&quot;</span><span class="p">,</span> <span class="s2">&quot;entry&quot;</span><span class="p">)</span>
</span><span id="Mimir-141"><a href="#Mimir-141"><span class="linenos">141</span></a>            <span class="n">renderer</span> <span class="o">=</span> <span class="bp">self</span><span class="o">.</span><span class="n">fieldRenderers</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="n">ftype</span><span class="p">)</span>
</span><span id="Mimir-142"><a href="#Mimir-142"><span class="linenos">142</span></a>            <span class="k">if</span> <span class="n">renderer</span><span class="p">:</span>
</span><span id="Mimir-143"><a href="#Mimir-143"><span class="linenos">143</span></a>                <span class="n">renderer</span><span class="p">(</span><span class="n">frame</span><span class="p">,</span> <span class="n">row</span><span class="p">,</span> <span class="n">field</span><span class="p">)</span>
</span><span id="Mimir-144"><a href="#Mimir-144"><span class="linenos">144</span></a>
</span><span id="Mimir-145"><a href="#Mimir-145"><span class="linenos">145</span></a>                <span class="c1"># Adjust row offset based on field type</span>
</span><span id="Mimir-146"><a href="#Mimir-146"><span class="linenos">146</span></a>                <span class="k">if</span> <span class="n">ftype</span> <span class="o">==</span> <span class="s2">&quot;fileupload&quot;</span><span class="p">:</span>
</span><span id="Mimir-147"><a href="#Mimir-147"><span class="linenos">147</span></a>                    <span class="n">row</span> <span class="o">+=</span> <span class="mi">3</span>
</span><span id="Mimir-148"><a href="#Mimir-148"><span class="linenos">148</span></a>                <span class="k">elif</span> <span class="n">ftype</span> <span class="o">==</span> <span class="s2">&quot;heading&quot;</span><span class="p">:</span>
</span><span id="Mimir-149"><a href="#Mimir-149"><span class="linenos">149</span></a>                    <span class="n">row</span> <span class="o">+=</span> <span class="mi">1</span>
</span><span id="Mimir-150"><a href="#Mimir-150"><span class="linenos">150</span></a>                <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir-151"><a href="#Mimir-151"><span class="linenos">151</span></a>                    <span class="n">row</span> <span class="o">+=</span> <span class="mi">2</span>
</span><span id="Mimir-152"><a href="#Mimir-152"><span class="linenos">152</span></a>
</span><span id="Mimir-153"><a href="#Mimir-153"><span class="linenos">153</span></a>        <span class="k">if</span> <span class="ow">not</span> <span class="n">noRemove</span><span class="p">:</span>
</span><span id="Mimir-154"><a href="#Mimir-154"><span class="linenos">154</span></a>            <span class="n">removeBtn</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Button</span><span class="p">(</span>
</span><span id="Mimir-155"><a href="#Mimir-155"><span class="linenos">155</span></a>                <span class="n">frame</span><span class="p">,</span>
</span><span id="Mimir-156"><a href="#Mimir-156"><span class="linenos">156</span></a>                <span class="n">text</span><span class="o">=</span><span class="s2">&quot;🗑️ Ta bort&quot;</span><span class="p">,</span>
</span><span id="Mimir-157"><a href="#Mimir-157"><span class="linenos">157</span></a>                <span class="n">bootstyle</span><span class="o">=</span><span class="s2">&quot;danger-outline&quot;</span><span class="p">,</span>
</span><span id="Mimir-158"><a href="#Mimir-158"><span class="linenos">158</span></a>                <span class="n">command</span><span class="o">=</span><span class="k">lambda</span> <span class="n">w</span><span class="o">=</span><span class="n">wrapper</span><span class="p">:</span> <span class="bp">self</span><span class="o">.</span><span class="n">removeBlock</span><span class="p">(</span><span class="n">blockList</span><span class="p">,</span> <span class="n">w</span><span class="p">)</span>
</span><span id="Mimir-159"><a href="#Mimir-159"><span class="linenos">159</span></a>            <span class="p">)</span>
</span><span id="Mimir-160"><a href="#Mimir-160"><span class="linenos">160</span></a>            <span class="n">removeBtn</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="mi">2</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="p">(</span><span class="mi">10</span><span class="p">,</span> <span class="mi">5</span><span class="p">))</span>
</span><span id="Mimir-161"><a href="#Mimir-161"><span class="linenos">161</span></a>
</span><span id="Mimir-162"><a href="#Mimir-162"><span class="linenos">162</span></a>        <span class="k">return</span> <span class="n">wrapper</span>
</span><span id="Mimir-163"><a href="#Mimir-163"><span class="linenos">163</span></a>
</span><span id="Mimir-164"><a href="#Mimir-164"><span class="linenos">164</span></a>
</span><span id="Mimir-165"><a href="#Mimir-165"><span class="linenos">165</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">registerFieldType</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">name</span><span class="p">,</span> <span class="n">renderer</span><span class="p">):</span>
</span><span id="Mimir-166"><a href="#Mimir-166"><span class="linenos">166</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Register a custom field renderer by name.&quot;&quot;&quot;</span>        
</span><span id="Mimir-167"><a href="#Mimir-167"><span class="linenos">167</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">fieldRenderers</span><span class="p">[</span><span class="n">name</span><span class="p">]</span> <span class="o">=</span> <span class="n">renderer</span>
</span><span id="Mimir-168"><a href="#Mimir-168"><span class="linenos">168</span></a>
</span><span id="Mimir-169"><a href="#Mimir-169"><span class="linenos">169</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">_renderEntryField</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">parent</span><span class="p">,</span> <span class="n">row</span><span class="p">,</span> <span class="n">field</span><span class="p">):</span>
</span><span id="Mimir-170"><a href="#Mimir-170"><span class="linenos">170</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Internal: Render an entry field with label.&quot;&quot;&quot;</span>
</span><span id="Mimir-171"><a href="#Mimir-171"><span class="linenos">171</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">addLabeledEntry</span><span class="p">(</span>
</span><span id="Mimir-172"><a href="#Mimir-172"><span class="linenos">172</span></a>            <span class="n">parent</span><span class="o">=</span><span class="n">parent</span><span class="p">,</span>
</span><span id="Mimir-173"><a href="#Mimir-173"><span class="linenos">173</span></a>            <span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span>
</span><span id="Mimir-174"><a href="#Mimir-174"><span class="linenos">174</span></a>            <span class="n">label</span><span class="o">=</span><span class="n">field</span><span class="p">[</span><span class="s2">&quot;label&quot;</span><span class="p">],</span>
</span><span id="Mimir-175"><a href="#Mimir-175"><span class="linenos">175</span></a>            <span class="n">variable</span><span class="o">=</span><span class="n">field</span><span class="p">[</span><span class="s2">&quot;variable&quot;</span><span class="p">],</span>
</span><span id="Mimir-176"><a href="#Mimir-176"><span class="linenos">176</span></a>            <span class="n">validation</span><span class="o">=</span><span class="n">field</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;validation&quot;</span><span class="p">),</span>
</span><span id="Mimir-177"><a href="#Mimir-177"><span class="linenos">177</span></a>            <span class="o">**</span><span class="n">field</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;options&quot;</span><span class="p">,</span> <span class="p">{})</span>  <span class="c1"># &lt;-- wildcard pass-through</span>
</span><span id="Mimir-178"><a href="#Mimir-178"><span class="linenos">178</span></a>        <span class="p">)</span>
</span><span id="Mimir-179"><a href="#Mimir-179"><span class="linenos">179</span></a>
</span><span id="Mimir-180"><a href="#Mimir-180"><span class="linenos">180</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">_renderComboboxField</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">parent</span><span class="p">,</span> <span class="n">row</span><span class="p">,</span> <span class="n">field</span><span class="p">):</span>
</span><span id="Mimir-181"><a href="#Mimir-181"><span class="linenos">181</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Internal: Render a combobox field with label.&quot;&quot;&quot;</span>
</span><span id="Mimir-182"><a href="#Mimir-182"><span class="linenos">182</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">addLabeledCombobox</span><span class="p">(</span>
</span><span id="Mimir-183"><a href="#Mimir-183"><span class="linenos">183</span></a>            <span class="n">parent</span><span class="o">=</span><span class="n">parent</span><span class="p">,</span>
</span><span id="Mimir-184"><a href="#Mimir-184"><span class="linenos">184</span></a>            <span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span>
</span><span id="Mimir-185"><a href="#Mimir-185"><span class="linenos">185</span></a>            <span class="n">label</span><span class="o">=</span><span class="n">field</span><span class="p">[</span><span class="s2">&quot;label&quot;</span><span class="p">],</span>
</span><span id="Mimir-186"><a href="#Mimir-186"><span class="linenos">186</span></a>            <span class="n">variable</span><span class="o">=</span><span class="n">field</span><span class="p">[</span><span class="s2">&quot;variable&quot;</span><span class="p">],</span>
</span><span id="Mimir-187"><a href="#Mimir-187"><span class="linenos">187</span></a>            <span class="n">values</span><span class="o">=</span><span class="n">field</span><span class="p">[</span><span class="s2">&quot;values&quot;</span><span class="p">],</span>
</span><span id="Mimir-188"><a href="#Mimir-188"><span class="linenos">188</span></a>            <span class="o">**</span><span class="n">field</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;options&quot;</span><span class="p">,</span> <span class="p">{})</span>  <span class="c1"># &lt;-- pass all UI customization</span>
</span><span id="Mimir-189"><a href="#Mimir-189"><span class="linenos">189</span></a>        <span class="p">)</span>
</span><span id="Mimir-190"><a href="#Mimir-190"><span class="linenos">190</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">_renderHeading</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">parent</span><span class="p">,</span> <span class="n">row</span><span class="p">,</span> <span class="n">field</span><span class="p">):</span>
</span><span id="Mimir-191"><a href="#Mimir-191"><span class="linenos">191</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Internal: Render a bold heading label within a form.&quot;&quot;&quot;</span>
</span><span id="Mimir-192"><a href="#Mimir-192"><span class="linenos">192</span></a>        <span class="n">font</span> <span class="o">=</span> <span class="bp">self</span><span class="o">.</span><span class="n">getCurrentFont</span><span class="p">()</span>
</span><span id="Mimir-193"><a href="#Mimir-193"><span class="linenos">193</span></a>        <span class="n">font</span><span class="o">.</span><span class="n">configure</span><span class="p">(</span><span class="n">weight</span><span class="o">=</span><span class="s2">&quot;bold&quot;</span><span class="p">)</span>
</span><span id="Mimir-194"><a href="#Mimir-194"><span class="linenos">194</span></a>
</span><span id="Mimir-195"><a href="#Mimir-195"><span class="linenos">195</span></a>        <span class="n">align</span> <span class="o">=</span> <span class="n">field</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;options&quot;</span><span class="p">,</span> <span class="p">{})</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;align&quot;</span><span class="p">,</span> <span class="s2">&quot;w&quot;</span><span class="p">)</span>  <span class="c1"># default to left</span>
</span><span id="Mimir-196"><a href="#Mimir-196"><span class="linenos">196</span></a>
</span><span id="Mimir-197"><a href="#Mimir-197"><span class="linenos">197</span></a>        <span class="n">tb</span><span class="o">.</span><span class="n">Label</span><span class="p">(</span>
</span><span id="Mimir-198"><a href="#Mimir-198"><span class="linenos">198</span></a>            <span class="n">parent</span><span class="p">,</span>
</span><span id="Mimir-199"><a href="#Mimir-199"><span class="linenos">199</span></a>            <span class="n">text</span><span class="o">=</span><span class="n">field</span><span class="p">[</span><span class="s2">&quot;label&quot;</span><span class="p">],</span>
</span><span id="Mimir-200"><a href="#Mimir-200"><span class="linenos">200</span></a>            <span class="n">font</span><span class="o">=</span><span class="n">font</span>
</span><span id="Mimir-201"><a href="#Mimir-201"><span class="linenos">201</span></a>        <span class="p">)</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="mi">2</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="p">(</span><span class="mi">10</span><span class="p">,</span> <span class="mi">5</span><span class="p">),</span> <span class="n">sticky</span><span class="o">=</span><span class="n">align</span><span class="p">)</span>
</span><span id="Mimir-202"><a href="#Mimir-202"><span class="linenos">202</span></a>
</span><span id="Mimir-203"><a href="#Mimir-203"><span class="linenos">203</span></a>
</span><span id="Mimir-204"><a href="#Mimir-204"><span class="linenos">204</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">_setupValidation</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">widget</span><span class="p">,</span> <span class="n">validationName</span><span class="p">):</span>
</span><span id="Mimir-205"><a href="#Mimir-205"><span class="linenos">205</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Bind focus-out validation using a named validation function.&quot;&quot;&quot;</span>
</span><span id="Mimir-206"><a href="#Mimir-206"><span class="linenos">206</span></a>        <span class="n">fn</span> <span class="o">=</span> <span class="bp">self</span><span class="o">.</span><span class="n">_validatorResolver</span><span class="p">(</span><span class="n">validationName</span><span class="p">)</span> <span class="k">if</span> <span class="n">validationName</span> <span class="k">else</span> <span class="kc">None</span>
</span><span id="Mimir-207"><a href="#Mimir-207"><span class="linenos">207</span></a>        <span class="k">if</span> <span class="ow">not</span> <span class="nb">callable</span><span class="p">(</span><span class="n">fn</span><span class="p">):</span>
</span><span id="Mimir-208"><a href="#Mimir-208"><span class="linenos">208</span></a>            <span class="k">return</span>
</span><span id="Mimir-209"><a href="#Mimir-209"><span class="linenos">209</span></a>
</span><span id="Mimir-210"><a href="#Mimir-210"><span class="linenos">210</span></a>        <span class="k">def</span><span class="w"> </span><span class="nf">validate</span><span class="p">(</span><span class="n">event</span><span class="p">):</span>
</span><span id="Mimir-211"><a href="#Mimir-211"><span class="linenos">211</span></a>            <span class="k">if</span> <span class="n">fn</span><span class="p">(</span><span class="n">widget</span><span class="o">.</span><span class="n">get</span><span class="p">()):</span>
</span><span id="Mimir-212"><a href="#Mimir-212"><span class="linenos">212</span></a>                <span class="bp">self</span><span class="o">.</span><span class="n">clearInvalid</span><span class="p">(</span><span class="n">widget</span><span class="p">)</span>
</span><span id="Mimir-213"><a href="#Mimir-213"><span class="linenos">213</span></a>            <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir-214"><a href="#Mimir-214"><span class="linenos">214</span></a>                <span class="bp">self</span><span class="o">.</span><span class="n">markInvalid</span><span class="p">(</span><span class="n">widget</span><span class="p">)</span>
</span><span id="Mimir-215"><a href="#Mimir-215"><span class="linenos">215</span></a>
</span><span id="Mimir-216"><a href="#Mimir-216"><span class="linenos">216</span></a>        <span class="n">widget</span><span class="o">.</span><span class="n">bind</span><span class="p">(</span><span class="s2">&quot;&lt;FocusOut&gt;&quot;</span><span class="p">,</span> <span class="n">validate</span><span class="p">)</span>
</span><span id="Mimir-217"><a href="#Mimir-217"><span class="linenos">217</span></a>
</span><span id="Mimir-218"><a href="#Mimir-218"><span class="linenos">218</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">markInvalid</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">widget</span><span class="p">):</span>
</span><span id="Mimir-219"><a href="#Mimir-219"><span class="linenos">219</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Mark a widget as invalid with red border styling.&quot;&quot;&quot;</span>
</span><span id="Mimir-220"><a href="#Mimir-220"><span class="linenos">220</span></a>        <span class="n">widget</span><span class="o">.</span><span class="n">config</span><span class="p">(</span><span class="n">bootstyle</span><span class="o">=</span><span class="s2">&quot;danger&quot;</span><span class="p">)</span>
</span><span id="Mimir-221"><a href="#Mimir-221"><span class="linenos">221</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">invalidEntries</span><span class="o">.</span><span class="n">add</span><span class="p">(</span><span class="n">widget</span><span class="p">)</span>
</span><span id="Mimir-222"><a href="#Mimir-222"><span class="linenos">222</span></a>
</span><span id="Mimir-223"><a href="#Mimir-223"><span class="linenos">223</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">clearInvalid</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">widget</span><span class="p">):</span>
</span><span id="Mimir-224"><a href="#Mimir-224"><span class="linenos">224</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Clear error styling from a widget.&quot;&quot;&quot;</span>
</span><span id="Mimir-225"><a href="#Mimir-225"><span class="linenos">225</span></a>        <span class="n">widget</span><span class="o">.</span><span class="n">config</span><span class="p">(</span><span class="n">bootstyle</span><span class="o">=</span><span class="s2">&quot;none&quot;</span><span class="p">)</span>
</span><span id="Mimir-226"><a href="#Mimir-226"><span class="linenos">226</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">invalidEntries</span><span class="o">.</span><span class="n">discard</span><span class="p">(</span><span class="n">widget</span><span class="p">)</span>
</span><span id="Mimir-227"><a href="#Mimir-227"><span class="linenos">227</span></a>
</span><span id="Mimir-228"><a href="#Mimir-228"><span class="linenos">228</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">addNextButton</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">parent</span><span class="p">,</span> <span class="n">row</span><span class="p">,</span> <span class="n">command</span><span class="p">,</span> <span class="n">label</span><span class="o">=</span><span class="s2">&quot;Nästa&quot;</span><span class="p">,</span> <span class="n">tooltip</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="kc">None</span><span class="p">):</span>
</span><span id="Mimir-229"><a href="#Mimir-229"><span class="linenos">229</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir-230"><a href="#Mimir-230"><span class="linenos">230</span></a><span class="sd">        Add a &#39;Next&#39; button to the given row. Automatically spans the full width</span>
</span><span id="Mimir-231"><a href="#Mimir-231"><span class="linenos">231</span></a><span class="sd">        of horizontal blocks if no column or columnspan is provided.</span>
</span><span id="Mimir-232"><a href="#Mimir-232"><span class="linenos">232</span></a><span class="sd">        Blocks execution if any fields are invalid.</span>
</span><span id="Mimir-233"><a href="#Mimir-233"><span class="linenos">233</span></a><span class="sd">        &quot;&quot;&quot;</span>        
</span><span id="Mimir-234"><a href="#Mimir-234"><span class="linenos">234</span></a>        <span class="k">def</span><span class="w"> </span><span class="nf">wrapped</span><span class="p">():</span>
</span><span id="Mimir-235"><a href="#Mimir-235"><span class="linenos">235</span></a>            <span class="k">if</span> <span class="bp">self</span><span class="o">.</span><span class="n">invalidEntries</span><span class="p">:</span>
</span><span id="Mimir-236"><a href="#Mimir-236"><span class="linenos">236</span></a>                <span class="n">msg</span><span class="o">.</span><span class="n">showerror</span><span class="p">(</span><span class="s2">&quot;Fel&quot;</span><span class="p">,</span> <span class="s2">&quot;Alla fält måste vara giltiga innan du kan fortsätta.&quot;</span><span class="p">)</span>
</span><span id="Mimir-237"><a href="#Mimir-237"><span class="linenos">237</span></a>                <span class="k">return</span>
</span><span id="Mimir-238"><a href="#Mimir-238"><span class="linenos">238</span></a>            <span class="n">command</span><span class="p">()</span>
</span><span id="Mimir-239"><a href="#Mimir-239"><span class="linenos">239</span></a>
</span><span id="Mimir-240"><a href="#Mimir-240"><span class="linenos">240</span></a>        <span class="c1"># Calculate default columnspan if not explicitly provided</span>
</span><span id="Mimir-241"><a href="#Mimir-241"><span class="linenos">241</span></a>        <span class="k">if</span> <span class="n">column</span> <span class="ow">is</span> <span class="kc">None</span> <span class="ow">or</span> <span class="n">columnspan</span> <span class="ow">is</span> <span class="kc">None</span><span class="p">:</span>
</span><span id="Mimir-242"><a href="#Mimir-242"><span class="linenos">242</span></a>            <span class="c1"># Find all unique columns used in row=0 (block layout)</span>
</span><span id="Mimir-243"><a href="#Mimir-243"><span class="linenos">243</span></a>            <span class="n">used_columns</span> <span class="o">=</span> <span class="nb">set</span><span class="p">()</span>
</span><span id="Mimir-244"><a href="#Mimir-244"><span class="linenos">244</span></a>            <span class="k">for</span> <span class="n">child</span> <span class="ow">in</span> <span class="n">parent</span><span class="o">.</span><span class="n">winfo_children</span><span class="p">():</span>
</span><span id="Mimir-245"><a href="#Mimir-245"><span class="linenos">245</span></a>                <span class="n">grid</span> <span class="o">=</span> <span class="n">child</span><span class="o">.</span><span class="n">grid_info</span><span class="p">()</span>
</span><span id="Mimir-246"><a href="#Mimir-246"><span class="linenos">246</span></a>                <span class="k">if</span> <span class="n">grid</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;row&quot;</span><span class="p">)</span> <span class="o">==</span> <span class="mi">0</span> <span class="ow">and</span> <span class="s2">&quot;column&quot;</span> <span class="ow">in</span> <span class="n">grid</span><span class="p">:</span>
</span><span id="Mimir-247"><a href="#Mimir-247"><span class="linenos">247</span></a>                    <span class="n">used_columns</span><span class="o">.</span><span class="n">add</span><span class="p">(</span><span class="nb">int</span><span class="p">(</span><span class="n">grid</span><span class="p">[</span><span class="s2">&quot;column&quot;</span><span class="p">]))</span>
</span><span id="Mimir-248"><a href="#Mimir-248"><span class="linenos">248</span></a>
</span><span id="Mimir-249"><a href="#Mimir-249"><span class="linenos">249</span></a>            <span class="k">if</span> <span class="ow">not</span> <span class="n">used_columns</span><span class="p">:</span>
</span><span id="Mimir-250"><a href="#Mimir-250"><span class="linenos">250</span></a>                <span class="n">column</span> <span class="o">=</span> <span class="mi">0</span>
</span><span id="Mimir-251"><a href="#Mimir-251"><span class="linenos">251</span></a>                <span class="n">columnspan</span> <span class="o">=</span> <span class="mi">1</span>
</span><span id="Mimir-252"><a href="#Mimir-252"><span class="linenos">252</span></a>            <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir-253"><a href="#Mimir-253"><span class="linenos">253</span></a>                <span class="n">column</span> <span class="o">=</span> <span class="mi">0</span>
</span><span id="Mimir-254"><a href="#Mimir-254"><span class="linenos">254</span></a>                <span class="n">columnspan</span> <span class="o">=</span> <span class="nb">max</span><span class="p">(</span><span class="n">used_columns</span><span class="p">)</span> <span class="o">+</span> <span class="mi">1</span>
</span><span id="Mimir-255"><a href="#Mimir-255"><span class="linenos">255</span></a>
</span><span id="Mimir-256"><a href="#Mimir-256"><span class="linenos">256</span></a>        <span class="n">btn</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Button</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">text</span><span class="o">=</span><span class="n">label</span><span class="p">,</span> <span class="n">command</span><span class="o">=</span><span class="n">wrapped</span><span class="p">,</span> <span class="n">bootstyle</span><span class="o">=</span><span class="s2">&quot;success&quot;</span><span class="p">)</span>
</span><span id="Mimir-257"><a href="#Mimir-257"><span class="linenos">257</span></a>        <span class="n">btn</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="n">columnspan</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="p">(</span><span class="mi">10</span><span class="p">,</span> <span class="mi">5</span><span class="p">))</span>
</span><span id="Mimir-258"><a href="#Mimir-258"><span class="linenos">258</span></a>
</span><span id="Mimir-259"><a href="#Mimir-259"><span class="linenos">259</span></a>        <span class="k">if</span> <span class="n">tooltip</span><span class="p">:</span>
</span><span id="Mimir-260"><a href="#Mimir-260"><span class="linenos">260</span></a>            <span class="bp">self</span><span class="o">.</span><span class="n">addTooltip</span><span class="p">(</span><span class="n">btn</span><span class="p">,</span> <span class="n">tooltip</span><span class="p">)</span>
</span><span id="Mimir-261"><a href="#Mimir-261"><span class="linenos">261</span></a>
</span><span id="Mimir-262"><a href="#Mimir-262"><span class="linenos">262</span></a>        <span class="k">return</span> <span class="n">btn</span>
</span><span id="Mimir-263"><a href="#Mimir-263"><span class="linenos">263</span></a>
</span><span id="Mimir-264"><a href="#Mimir-264"><span class="linenos">264</span></a>
</span><span id="Mimir-265"><a href="#Mimir-265"><span class="linenos">265</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">getCurrentFont</span><span class="p">(</span><span class="bp">self</span><span class="p">):</span>
</span><span id="Mimir-266"><a href="#Mimir-266"><span class="linenos">266</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Return the default font currently used by ttkbootstrap widgets.&quot;&quot;&quot;</span>
</span><span id="Mimir-267"><a href="#Mimir-267"><span class="linenos">267</span></a>        <span class="n">style</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Style</span><span class="p">()</span>
</span><span id="Mimir-268"><a href="#Mimir-268"><span class="linenos">268</span></a>        <span class="n">fontName</span> <span class="o">=</span> <span class="n">style</span><span class="o">.</span><span class="n">lookup</span><span class="p">(</span><span class="s2">&quot;TLabel&quot;</span><span class="p">,</span> <span class="s2">&quot;font&quot;</span><span class="p">)</span>
</span><span id="Mimir-269"><a href="#Mimir-269"><span class="linenos">269</span></a>        <span class="k">return</span> <span class="n">tkFont</span><span class="o">.</span><span class="n">Font</span><span class="p">(</span><span class="n">font</span><span class="o">=</span><span class="n">fontName</span><span class="p">)</span>
</span><span id="Mimir-270"><a href="#Mimir-270"><span class="linenos">270</span></a>
</span><span id="Mimir-271"><a href="#Mimir-271"><span class="linenos">271</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">addTooltip</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">widget</span><span class="p">,</span> <span class="n">text</span><span class="p">):</span>
</span><span id="Mimir-272"><a href="#Mimir-272"><span class="linenos">272</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Attach a styled tooltip to a widget.&quot;&quot;&quot;</span>
</span><span id="Mimir-273"><a href="#Mimir-273"><span class="linenos">273</span></a>        <span class="n">ToolTip</span><span class="p">(</span><span class="n">widget</span><span class="p">,</span> <span class="n">text</span><span class="p">,</span> <span class="n">bootstyle</span><span class="o">=</span><span class="p">(</span><span class="n">SECONDARY</span><span class="p">,</span> <span class="n">INVERSE</span><span class="p">))</span>
</span><span id="Mimir-274"><a href="#Mimir-274"><span class="linenos">274</span></a>
</span><span id="Mimir-275"><a href="#Mimir-275"><span class="linenos">275</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">removeBlock</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">blockList</span><span class="p">,</span> <span class="n">wrapper</span><span class="p">):</span>
</span><span id="Mimir-276"><a href="#Mimir-276"><span class="linenos">276</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir-277"><a href="#Mimir-277"><span class="linenos">277</span></a><span class="sd">        Remove a UI block from view and list. Automatically reindexes</span>
</span><span id="Mimir-278"><a href="#Mimir-278"><span class="linenos">278</span></a><span class="sd">        and repositions remaining blocks.</span>
</span><span id="Mimir-279"><a href="#Mimir-279"><span class="linenos">279</span></a><span class="sd">        &quot;&quot;&quot;</span>        
</span><span id="Mimir-280"><a href="#Mimir-280"><span class="linenos">280</span></a>        <span class="n">frame</span> <span class="o">=</span> <span class="n">wrapper</span><span class="o">.</span><span class="n">meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;frame&quot;</span><span class="p">)</span>
</span><span id="Mimir-281"><a href="#Mimir-281"><span class="linenos">281</span></a>        <span class="n">layout</span> <span class="o">=</span> <span class="n">wrapper</span><span class="o">.</span><span class="n">meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;layout&quot;</span><span class="p">,</span> <span class="s2">&quot;horizontal&quot;</span><span class="p">)</span>
</span><span id="Mimir-282"><a href="#Mimir-282"><span class="linenos">282</span></a>        <span class="k">if</span> <span class="n">frame</span><span class="p">:</span>
</span><span id="Mimir-283"><a href="#Mimir-283"><span class="linenos">283</span></a>            <span class="n">frame</span><span class="o">.</span><span class="n">destroy</span><span class="p">()</span>
</span><span id="Mimir-284"><a href="#Mimir-284"><span class="linenos">284</span></a>        <span class="n">blockList</span><span class="o">.</span><span class="n">remove</span><span class="p">(</span><span class="n">wrapper</span><span class="p">)</span>
</span><span id="Mimir-285"><a href="#Mimir-285"><span class="linenos">285</span></a>
</span><span id="Mimir-286"><a href="#Mimir-286"><span class="linenos">286</span></a>        <span class="k">for</span> <span class="n">i</span><span class="p">,</span> <span class="n">block</span> <span class="ow">in</span> <span class="nb">enumerate</span><span class="p">(</span><span class="n">blockList</span><span class="p">):</span>
</span><span id="Mimir-287"><a href="#Mimir-287"><span class="linenos">287</span></a>            <span class="n">frame</span> <span class="o">=</span> <span class="n">block</span><span class="o">.</span><span class="n">meta</span><span class="p">[</span><span class="s2">&quot;frame&quot;</span><span class="p">]</span>
</span><span id="Mimir-288"><a href="#Mimir-288"><span class="linenos">288</span></a>            <span class="n">new_label</span> <span class="o">=</span> <span class="sa">f</span><span class="s2">&quot;Butik </span><span class="si">{</span><span class="n">i</span><span class="w"> </span><span class="o">+</span><span class="w"> </span><span class="mi">1</span><span class="si">}</span><span class="s2">&quot;</span>
</span><span id="Mimir-289"><a href="#Mimir-289"><span class="linenos">289</span></a>
</span><span id="Mimir-290"><a href="#Mimir-290"><span class="linenos">290</span></a>            <span class="c1"># Update visual label</span>
</span><span id="Mimir-291"><a href="#Mimir-291"><span class="linenos">291</span></a>            <span class="n">frame</span><span class="o">.</span><span class="n">config</span><span class="p">(</span><span class="n">text</span><span class="o">=</span><span class="n">new_label</span><span class="p">)</span>
</span><span id="Mimir-292"><a href="#Mimir-292"><span class="linenos">292</span></a>
</span><span id="Mimir-293"><a href="#Mimir-293"><span class="linenos">293</span></a>            <span class="c1"># Update meta so future logic knows the new label/index</span>
</span><span id="Mimir-294"><a href="#Mimir-294"><span class="linenos">294</span></a>            <span class="n">block</span><span class="o">.</span><span class="n">meta</span><span class="p">[</span><span class="s2">&quot;custom_label&quot;</span><span class="p">]</span> <span class="o">=</span> <span class="n">new_label</span>
</span><span id="Mimir-295"><a href="#Mimir-295"><span class="linenos">295</span></a>            <span class="n">block</span><span class="o">.</span><span class="n">meta</span><span class="p">[</span><span class="s2">&quot;store_id&quot;</span><span class="p">]</span> <span class="o">=</span> <span class="n">i</span> <span class="o">+</span> <span class="mi">1</span>  <span class="c1"># optional if store_id should follow display order</span>
</span><span id="Mimir-296"><a href="#Mimir-296"><span class="linenos">296</span></a>
</span><span id="Mimir-297"><a href="#Mimir-297"><span class="linenos">297</span></a>            <span class="c1"># Reposition depending on layout</span>
</span><span id="Mimir-298"><a href="#Mimir-298"><span class="linenos">298</span></a>            <span class="k">if</span> <span class="n">layout</span> <span class="o">==</span> <span class="s2">&quot;vertical&quot;</span><span class="p">:</span>
</span><span id="Mimir-299"><a href="#Mimir-299"><span class="linenos">299</span></a>                <span class="n">frame</span><span class="o">.</span><span class="n">grid_configure</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">i</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">)</span>
</span><span id="Mimir-300"><a href="#Mimir-300"><span class="linenos">300</span></a>            <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir-301"><a href="#Mimir-301"><span class="linenos">301</span></a>                <span class="n">frame</span><span class="o">.</span><span class="n">grid_configure</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">i</span><span class="p">)</span>
</span><span id="Mimir-302"><a href="#Mimir-302"><span class="linenos">302</span></a>
</span><span id="Mimir-303"><a href="#Mimir-303"><span class="linenos">303</span></a>
</span><span id="Mimir-304"><a href="#Mimir-304"><span class="linenos">304</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">addLabeledEntry</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">parent</span><span class="p">,</span> <span class="n">row</span><span class="p">,</span> <span class="n">label</span><span class="p">,</span> <span class="n">variable</span><span class="p">,</span> <span class="n">state</span><span class="o">=</span><span class="s2">&quot;normal&quot;</span><span class="p">,</span> <span class="n">tooltip</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="n">validation</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">vertical</span><span class="o">=</span><span class="kc">False</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="mi">1</span><span class="p">):</span>
</span><span id="Mimir-305"><a href="#Mimir-305"><span class="linenos">305</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir-306"><a href="#Mimir-306"><span class="linenos">306</span></a><span class="sd">        Add a labeled entry input field. Supports vertical and horizontal layouts,</span>
</span><span id="Mimir-307"><a href="#Mimir-307"><span class="linenos">307</span></a><span class="sd">        optional tooltips and validation.</span>
</span><span id="Mimir-308"><a href="#Mimir-308"><span class="linenos">308</span></a><span class="sd">        &quot;&quot;&quot;</span>        
</span><span id="Mimir-309"><a href="#Mimir-309"><span class="linenos">309</span></a>        <span class="k">if</span> <span class="n">vertical</span><span class="p">:</span>
</span><span id="Mimir-310"><a href="#Mimir-310"><span class="linenos">310</span></a>            <span class="n">tb</span><span class="o">.</span><span class="n">Label</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">text</span><span class="o">=</span><span class="n">label</span><span class="p">)</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="n">columnspan</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="p">(</span><span class="mi">5</span><span class="p">,</span> <span class="mi">5</span><span class="p">))</span>
</span><span id="Mimir-311"><a href="#Mimir-311"><span class="linenos">311</span></a>            <span class="n">entry</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Entry</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">textvariable</span><span class="o">=</span><span class="n">variable</span><span class="p">,</span> <span class="n">state</span><span class="o">=</span><span class="n">state</span><span class="p">,</span> <span class="n">width</span><span class="o">=</span><span class="bp">self</span><span class="o">.</span><span class="n">widthEntry</span><span class="p">)</span>
</span><span id="Mimir-312"><a href="#Mimir-312"><span class="linenos">312</span></a>            <span class="n">entry</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span> <span class="o">+</span> <span class="mi">1</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="n">columnspan</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="p">(</span><span class="mi">0</span><span class="p">,</span> <span class="mi">10</span><span class="p">),</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;ew&quot;</span><span class="p">)</span>
</span><span id="Mimir-313"><a href="#Mimir-313"><span class="linenos">313</span></a>        <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir-314"><a href="#Mimir-314"><span class="linenos">314</span></a>            <span class="n">tb</span><span class="o">.</span><span class="n">Label</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">text</span><span class="o">=</span><span class="n">label</span><span class="p">)</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;e&quot;</span><span class="p">)</span>
</span><span id="Mimir-315"><a href="#Mimir-315"><span class="linenos">315</span></a>            <span class="n">entry</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Entry</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">textvariable</span><span class="o">=</span><span class="n">variable</span><span class="p">,</span> <span class="n">state</span><span class="o">=</span><span class="n">state</span><span class="p">,</span> <span class="n">width</span><span class="o">=</span><span class="bp">self</span><span class="o">.</span><span class="n">widthEntry</span><span class="p">)</span>
</span><span id="Mimir-316"><a href="#Mimir-316"><span class="linenos">316</span></a>            <span class="n">entry</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span> <span class="o">+</span> <span class="mi">1</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="n">columnspan</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;ew&quot;</span><span class="p">)</span>
</span><span id="Mimir-317"><a href="#Mimir-317"><span class="linenos">317</span></a>
</span><span id="Mimir-318"><a href="#Mimir-318"><span class="linenos">318</span></a>        <span class="k">if</span> <span class="n">tooltip</span><span class="p">:</span>
</span><span id="Mimir-319"><a href="#Mimir-319"><span class="linenos">319</span></a>            <span class="bp">self</span><span class="o">.</span><span class="n">addTooltip</span><span class="p">(</span><span class="n">entry</span><span class="p">,</span> <span class="n">tooltip</span><span class="p">)</span>
</span><span id="Mimir-320"><a href="#Mimir-320"><span class="linenos">320</span></a>
</span><span id="Mimir-321"><a href="#Mimir-321"><span class="linenos">321</span></a>        <span class="n">validationFunc</span> <span class="o">=</span> <span class="bp">self</span><span class="o">.</span><span class="n">resolveValidator</span><span class="p">(</span><span class="n">validation</span><span class="p">)</span> <span class="k">if</span> <span class="n">validation</span> <span class="k">else</span> <span class="kc">None</span>
</span><span id="Mimir-322"><a href="#Mimir-322"><span class="linenos">322</span></a>        <span class="k">if</span> <span class="nb">callable</span><span class="p">(</span><span class="n">validationFunc</span><span class="p">):</span>
</span><span id="Mimir-323"><a href="#Mimir-323"><span class="linenos">323</span></a>            <span class="k">def</span><span class="w"> </span><span class="nf">onFocusOut</span><span class="p">(</span><span class="n">event</span><span class="p">):</span>
</span><span id="Mimir-324"><a href="#Mimir-324"><span class="linenos">324</span></a>                <span class="n">value</span> <span class="o">=</span> <span class="n">entry</span><span class="o">.</span><span class="n">get</span><span class="p">()</span>
</span><span id="Mimir-325"><a href="#Mimir-325"><span class="linenos">325</span></a>                <span class="k">if</span> <span class="n">validationFunc</span><span class="p">(</span><span class="n">value</span><span class="p">):</span>
</span><span id="Mimir-326"><a href="#Mimir-326"><span class="linenos">326</span></a>                    <span class="bp">self</span><span class="o">.</span><span class="n">clearInvalid</span><span class="p">(</span><span class="n">entry</span><span class="p">)</span>
</span><span id="Mimir-327"><a href="#Mimir-327"><span class="linenos">327</span></a>                <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir-328"><a href="#Mimir-328"><span class="linenos">328</span></a>                    <span class="bp">self</span><span class="o">.</span><span class="n">markInvalid</span><span class="p">(</span><span class="n">entry</span><span class="p">)</span>
</span><span id="Mimir-329"><a href="#Mimir-329"><span class="linenos">329</span></a>            <span class="n">entry</span><span class="o">.</span><span class="n">bind</span><span class="p">(</span><span class="s2">&quot;&lt;FocusOut&gt;&quot;</span><span class="p">,</span> <span class="n">onFocusOut</span><span class="p">)</span>
</span><span id="Mimir-330"><a href="#Mimir-330"><span class="linenos">330</span></a>
</span><span id="Mimir-331"><a href="#Mimir-331"><span class="linenos">331</span></a>        <span class="k">return</span> <span class="n">entry</span>
</span><span id="Mimir-332"><a href="#Mimir-332"><span class="linenos">332</span></a>
</span><span id="Mimir-333"><a href="#Mimir-333"><span class="linenos">333</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">addLabeledCombobox</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">parent</span><span class="p">,</span> <span class="n">row</span><span class="p">,</span> <span class="n">label</span><span class="p">,</span> <span class="n">variable</span><span class="p">,</span> <span class="n">values</span><span class="p">,</span> <span class="n">tooltip</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="n">state</span><span class="o">=</span><span class="s2">&quot;readonly&quot;</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">vertical</span><span class="o">=</span><span class="kc">False</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="mi">1</span><span class="p">):</span>
</span><span id="Mimir-334"><a href="#Mimir-334"><span class="linenos">334</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Add a labeled combobox dropdown field with options.&quot;&quot;&quot;</span>
</span><span id="Mimir-335"><a href="#Mimir-335"><span class="linenos">335</span></a>        <span class="k">if</span> <span class="n">vertical</span><span class="p">:</span>
</span><span id="Mimir-336"><a href="#Mimir-336"><span class="linenos">336</span></a>            <span class="n">tb</span><span class="o">.</span><span class="n">Label</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">text</span><span class="o">=</span><span class="n">label</span><span class="p">)</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="n">columnspan</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="p">(</span><span class="mi">5</span><span class="p">,</span> <span class="mi">0</span><span class="p">))</span>
</span><span id="Mimir-337"><a href="#Mimir-337"><span class="linenos">337</span></a>            <span class="n">combo</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Combobox</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">textvariable</span><span class="o">=</span><span class="n">variable</span><span class="p">,</span> <span class="n">values</span><span class="o">=</span><span class="n">values</span><span class="p">,</span> <span class="n">state</span><span class="o">=</span><span class="n">state</span><span class="p">,</span> <span class="n">width</span><span class="o">=</span><span class="bp">self</span><span class="o">.</span><span class="n">widthCombo</span><span class="p">)</span>
</span><span id="Mimir-338"><a href="#Mimir-338"><span class="linenos">338</span></a>            <span class="n">combo</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span> <span class="o">+</span> <span class="mi">1</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="n">columnspan</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="p">(</span><span class="mi">0</span><span class="p">,</span> <span class="mi">10</span><span class="p">))</span>
</span><span id="Mimir-339"><a href="#Mimir-339"><span class="linenos">339</span></a>        <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir-340"><a href="#Mimir-340"><span class="linenos">340</span></a>            <span class="n">tb</span><span class="o">.</span><span class="n">Label</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">text</span><span class="o">=</span><span class="n">label</span><span class="p">)</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;e&quot;</span><span class="p">)</span>
</span><span id="Mimir-341"><a href="#Mimir-341"><span class="linenos">341</span></a>            <span class="n">combo</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Combobox</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">textvariable</span><span class="o">=</span><span class="n">variable</span><span class="p">,</span> <span class="n">values</span><span class="o">=</span><span class="n">values</span><span class="p">,</span> <span class="n">state</span><span class="o">=</span><span class="n">state</span><span class="p">,</span> <span class="n">width</span><span class="o">=</span><span class="bp">self</span><span class="o">.</span><span class="n">widthCombo</span><span class="p">)</span>
</span><span id="Mimir-342"><a href="#Mimir-342"><span class="linenos">342</span></a>            <span class="n">combo</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span> <span class="o">+</span> <span class="mi">1</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="n">columnspan</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;ew&quot;</span><span class="p">)</span>
</span><span id="Mimir-343"><a href="#Mimir-343"><span class="linenos">343</span></a>
</span><span id="Mimir-344"><a href="#Mimir-344"><span class="linenos">344</span></a>        <span class="k">if</span> <span class="n">tooltip</span><span class="p">:</span>
</span><span id="Mimir-345"><a href="#Mimir-345"><span class="linenos">345</span></a>            <span class="bp">self</span><span class="o">.</span><span class="n">addTooltip</span><span class="p">(</span><span class="n">combo</span><span class="p">,</span> <span class="n">tooltip</span><span class="p">)</span>
</span><span id="Mimir-346"><a href="#Mimir-346"><span class="linenos">346</span></a>
</span><span id="Mimir-347"><a href="#Mimir-347"><span class="linenos">347</span></a>        <span class="k">return</span> <span class="n">combo</span>
</span><span id="Mimir-348"><a href="#Mimir-348"><span class="linenos">348</span></a>
</span><span id="Mimir-349"><a href="#Mimir-349"><span class="linenos">349</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">getNextAvailableRow</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">frame</span><span class="p">:</span> <span class="n">tk</span><span class="o">.</span><span class="n">Widget</span><span class="p">)</span> <span class="o">-&gt;</span> <span class="nb">int</span><span class="p">:</span>
</span><span id="Mimir-350"><a href="#Mimir-350"><span class="linenos">350</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir-351"><a href="#Mimir-351"><span class="linenos">351</span></a><span class="sd">        Return the next available row in a container by checking occupied grid rows.</span>
</span><span id="Mimir-352"><a href="#Mimir-352"><span class="linenos">352</span></a><span class="sd">        &quot;&quot;&quot;</span>        
</span><span id="Mimir-353"><a href="#Mimir-353"><span class="linenos">353</span></a>        <span class="n">used_rows</span> <span class="o">=</span> <span class="p">[</span>
</span><span id="Mimir-354"><a href="#Mimir-354"><span class="linenos">354</span></a>            <span class="n">child</span><span class="o">.</span><span class="n">grid_info</span><span class="p">()</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;row&quot;</span><span class="p">)</span>
</span><span id="Mimir-355"><a href="#Mimir-355"><span class="linenos">355</span></a>            <span class="k">for</span> <span class="n">child</span> <span class="ow">in</span> <span class="n">frame</span><span class="o">.</span><span class="n">winfo_children</span><span class="p">()</span>
</span><span id="Mimir-356"><a href="#Mimir-356"><span class="linenos">356</span></a>            <span class="k">if</span> <span class="s2">&quot;row&quot;</span> <span class="ow">in</span> <span class="n">child</span><span class="o">.</span><span class="n">grid_info</span><span class="p">()</span>
</span><span id="Mimir-357"><a href="#Mimir-357"><span class="linenos">357</span></a>        <span class="p">]</span>
</span><span id="Mimir-358"><a href="#Mimir-358"><span class="linenos">358</span></a>        <span class="k">return</span> <span class="nb">max</span><span class="p">(</span><span class="n">used_rows</span><span class="p">,</span> <span class="n">default</span><span class="o">=-</span><span class="mi">1</span><span class="p">)</span> <span class="o">+</span> <span class="mi">1</span>
</span></pre></div>


    

                            <div id="Mimir.__init__" class="classattr">
                                        <input id="Mimir.__init__-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="name">Mimir</span><span class="signature pdoc-code condensed">(<span class="param"><span class="n">app</span></span>)</span>

                <label class="view-source-button" for="Mimir.__init__-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.__init__"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.__init__-20"><a href="#Mimir.__init__-20"><span class="linenos">20</span></a>    <span class="k">def</span><span class="w"> </span><span class="fm">__init__</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">app</span><span class="p">):</span>
</span><span id="Mimir.__init__-21"><a href="#Mimir.__init__-21"><span class="linenos">21</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir.__init__-22"><a href="#Mimir.__init__-22"><span class="linenos">22</span></a><span class="sd">        Initialize Mimir with reference to the root app window.</span>
</span><span id="Mimir.__init__-23"><a href="#Mimir.__init__-23"><span class="linenos">23</span></a><span class="sd">        &quot;&quot;&quot;</span>
</span><span id="Mimir.__init__-24"><a href="#Mimir.__init__-24"><span class="linenos">24</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">app</span> <span class="o">=</span> <span class="n">app</span>
</span><span id="Mimir.__init__-25"><a href="#Mimir.__init__-25"><span class="linenos">25</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">currentView</span> <span class="o">=</span> <span class="kc">None</span>
</span><span id="Mimir.__init__-26"><a href="#Mimir.__init__-26"><span class="linenos">26</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">invalidEntries</span> <span class="o">=</span> <span class="nb">set</span><span class="p">()</span>
</span><span id="Mimir.__init__-27"><a href="#Mimir.__init__-27"><span class="linenos">27</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">_validatorResolver</span> <span class="o">=</span> <span class="k">lambda</span> <span class="n">name</span><span class="p">:</span> <span class="kc">None</span>
</span><span id="Mimir.__init__-28"><a href="#Mimir.__init__-28"><span class="linenos">28</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">fieldRenderers</span> <span class="o">=</span> <span class="p">{</span>
</span><span id="Mimir.__init__-29"><a href="#Mimir.__init__-29"><span class="linenos">29</span></a>            <span class="s2">&quot;entry&quot;</span><span class="p">:</span> <span class="bp">self</span><span class="o">.</span><span class="n">_renderEntryField</span><span class="p">,</span>
</span><span id="Mimir.__init__-30"><a href="#Mimir.__init__-30"><span class="linenos">30</span></a>            <span class="s2">&quot;combobox&quot;</span><span class="p">:</span> <span class="bp">self</span><span class="o">.</span><span class="n">_renderComboboxField</span><span class="p">,</span>
</span><span id="Mimir.__init__-31"><a href="#Mimir.__init__-31"><span class="linenos">31</span></a>            <span class="s2">&quot;heading&quot;</span><span class="p">:</span> <span class="bp">self</span><span class="o">.</span><span class="n">_renderHeading</span><span class="p">,</span>
</span><span id="Mimir.__init__-32"><a href="#Mimir.__init__-32"><span class="linenos">32</span></a>        <span class="p">}</span>
</span><span id="Mimir.__init__-33"><a href="#Mimir.__init__-33"><span class="linenos">33</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">_allowThemeToggle</span> <span class="o">=</span> <span class="kc">False</span>
</span></pre></div>


            <div class="docstring"><p>Initialize Mimir with reference to the root app window.</p>
</div>


                            </div>
                            <div id="Mimir.widthEntry" class="classattr">
                                <div class="attr variable">
            <span class="name">widthEntry</span>        =
<span class="default_value">20</span>

        
    </div>
    <a class="headerlink" href="#Mimir.widthEntry"></a>
    
    

                            </div>
                            <div id="Mimir.widthCombo" class="classattr">
                                <div class="attr variable">
            <span class="name">widthCombo</span>        =
<span class="default_value">18</span>

        
    </div>
    <a class="headerlink" href="#Mimir.widthCombo"></a>
    
    

                            </div>
                            <div id="Mimir.app" class="classattr">
                                <div class="attr variable">
            <span class="name">app</span>

        
    </div>
    <a class="headerlink" href="#Mimir.app"></a>
    
    

                            </div>
                            <div id="Mimir.currentView" class="classattr">
                                <div class="attr variable">
            <span class="name">currentView</span>

        
    </div>
    <a class="headerlink" href="#Mimir.currentView"></a>
    
    

                            </div>
                            <div id="Mimir.invalidEntries" class="classattr">
                                <div class="attr variable">
            <span class="name">invalidEntries</span>

        
    </div>
    <a class="headerlink" href="#Mimir.invalidEntries"></a>
    
    

                            </div>
                            <div id="Mimir.fieldRenderers" class="classattr">
                                <div class="attr variable">
            <span class="name">fieldRenderers</span>

        
    </div>
    <a class="headerlink" href="#Mimir.fieldRenderers"></a>
    
    

                            </div>
                            <div id="Mimir.allowDarkMode" class="classattr">
                                        <input id="Mimir.allowDarkMode-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">allowDarkMode</span><span class="signature pdoc-code condensed">(<span class="param"><span class="bp">self</span>, </span><span class="param"><span class="n">enabled</span><span class="o">=</span><span class="kc">True</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="Mimir.allowDarkMode-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.allowDarkMode"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.allowDarkMode-35"><a href="#Mimir.allowDarkMode-35"><span class="linenos">35</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">allowDarkMode</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">enabled</span><span class="o">=</span><span class="kc">True</span><span class="p">):</span>
</span><span id="Mimir.allowDarkMode-36"><a href="#Mimir.allowDarkMode-36"><span class="linenos">36</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Enable or disable theme toggle button in the top right.&quot;&quot;&quot;</span>
</span><span id="Mimir.allowDarkMode-37"><a href="#Mimir.allowDarkMode-37"><span class="linenos">37</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">_allowThemeToggle</span> <span class="o">=</span> <span class="n">enabled</span>
</span></pre></div>


            <div class="docstring"><p>Enable or disable theme toggle button in the top right.</p>
</div>


                            </div>
                            <div id="Mimir.setValidatorResolver" class="classattr">
                                        <input id="Mimir.setValidatorResolver-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">setValidatorResolver</span><span class="signature pdoc-code condensed">(<span class="param"><span class="bp">self</span>, </span><span class="param"><span class="n">resolverFunc</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="Mimir.setValidatorResolver-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.setValidatorResolver"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.setValidatorResolver-44"><a href="#Mimir.setValidatorResolver-44"><span class="linenos">44</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">setValidatorResolver</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">resolverFunc</span><span class="p">):</span>
</span><span id="Mimir.setValidatorResolver-45"><a href="#Mimir.setValidatorResolver-45"><span class="linenos">45</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Register a function that returns validation callbacks by name.&quot;&quot;&quot;</span>
</span><span id="Mimir.setValidatorResolver-46"><a href="#Mimir.setValidatorResolver-46"><span class="linenos">46</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">_validatorResolver</span> <span class="o">=</span> <span class="n">resolverFunc</span>
</span></pre></div>


            <div class="docstring"><p>Register a function that returns validation callbacks by name.</p>
</div>


                            </div>
                            <div id="Mimir.switchView" class="classattr">
                                        <input id="Mimir.switchView-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">switchView</span><span class="signature pdoc-code condensed">(<span class="param"><span class="bp">self</span>, </span><span class="param"><span class="n">newFrameFunc</span>, </span><span class="param"><span class="n">gridOptions</span><span class="o">=</span><span class="kc">None</span>, </span><span class="param"><span class="o">**</span><span class="n">kwargs</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="Mimir.switchView-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.switchView"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.switchView-48"><a href="#Mimir.switchView-48"><span class="linenos">48</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">switchView</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">newFrameFunc</span><span class="p">,</span> <span class="n">gridOptions</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="o">**</span><span class="n">kwargs</span><span class="p">):</span>
</span><span id="Mimir.switchView-49"><a href="#Mimir.switchView-49"><span class="linenos">49</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir.switchView-50"><a href="#Mimir.switchView-50"><span class="linenos">50</span></a><span class="sd">        Replace the current view with a new frame built by newFrameFunc.</span>
</span><span id="Mimir.switchView-51"><a href="#Mimir.switchView-51"><span class="linenos">51</span></a><span class="sd">        Automatically configures geometry and grid placement.</span>
</span><span id="Mimir.switchView-52"><a href="#Mimir.switchView-52"><span class="linenos">52</span></a><span class="sd">        &quot;&quot;&quot;</span>        
</span><span id="Mimir.switchView-53"><a href="#Mimir.switchView-53"><span class="linenos">53</span></a>        <span class="k">if</span> <span class="bp">self</span><span class="o">.</span><span class="n">currentView</span><span class="p">:</span>
</span><span id="Mimir.switchView-54"><a href="#Mimir.switchView-54"><span class="linenos">54</span></a>            <span class="bp">self</span><span class="o">.</span><span class="n">currentView</span><span class="o">.</span><span class="n">destroy</span><span class="p">()</span>
</span><span id="Mimir.switchView-55"><a href="#Mimir.switchView-55"><span class="linenos">55</span></a>
</span><span id="Mimir.switchView-56"><a href="#Mimir.switchView-56"><span class="linenos">56</span></a>        <span class="n">container</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Frame</span><span class="p">(</span><span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="p">)</span>
</span><span id="Mimir.switchView-57"><a href="#Mimir.switchView-57"><span class="linenos">57</span></a>        <span class="n">container</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;nsew&quot;</span><span class="p">)</span>
</span><span id="Mimir.switchView-58"><a href="#Mimir.switchView-58"><span class="linenos">58</span></a>        <span class="n">container</span><span class="o">.</span><span class="n">columnconfigure</span><span class="p">(</span><span class="mi">0</span><span class="p">,</span> <span class="n">weight</span><span class="o">=</span><span class="mi">1</span><span class="p">)</span>
</span><span id="Mimir.switchView-59"><a href="#Mimir.switchView-59"><span class="linenos">59</span></a>
</span><span id="Mimir.switchView-60"><a href="#Mimir.switchView-60"><span class="linenos">60</span></a>        <span class="n">topBar</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Frame</span><span class="p">(</span><span class="n">container</span><span class="p">)</span>
</span><span id="Mimir.switchView-61"><a href="#Mimir.switchView-61"><span class="linenos">61</span></a>        <span class="n">topBar</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;ew&quot;</span><span class="p">)</span>
</span><span id="Mimir.switchView-62"><a href="#Mimir.switchView-62"><span class="linenos">62</span></a>
</span><span id="Mimir.switchView-63"><a href="#Mimir.switchView-63"><span class="linenos">63</span></a>        <span class="k">if</span> <span class="bp">self</span><span class="o">.</span><span class="n">_allowThemeToggle</span><span class="p">:</span>
</span><span id="Mimir.switchView-64"><a href="#Mimir.switchView-64"><span class="linenos">64</span></a>            <span class="n">tb</span><span class="o">.</span><span class="n">Button</span><span class="p">(</span><span class="n">topBar</span><span class="p">,</span> <span class="n">text</span><span class="o">=</span><span class="s2">&quot;Theme&quot;</span><span class="p">,</span> <span class="n">command</span><span class="o">=</span><span class="bp">self</span><span class="o">.</span><span class="n">_toggleTheme</span><span class="p">)</span><span class="o">.</span><span class="n">pack</span><span class="p">(</span><span class="n">side</span><span class="o">=</span><span class="s2">&quot;right&quot;</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">10</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="mi">10</span><span class="p">)</span>
</span><span id="Mimir.switchView-65"><a href="#Mimir.switchView-65"><span class="linenos">65</span></a>
</span><span id="Mimir.switchView-66"><a href="#Mimir.switchView-66"><span class="linenos">66</span></a>        <span class="n">newFrame</span> <span class="o">=</span> <span class="n">newFrameFunc</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="o">**</span><span class="n">kwargs</span><span class="p">)</span>
</span><span id="Mimir.switchView-67"><a href="#Mimir.switchView-67"><span class="linenos">67</span></a>        <span class="n">options</span> <span class="o">=</span> <span class="n">gridOptions</span> <span class="ow">or</span> <span class="p">{</span><span class="s2">&quot;row&quot;</span><span class="p">:</span> <span class="mi">1</span><span class="p">,</span> <span class="s2">&quot;column&quot;</span><span class="p">:</span> <span class="mi">0</span><span class="p">,</span> <span class="s2">&quot;sticky&quot;</span><span class="p">:</span> <span class="s2">&quot;n&quot;</span><span class="p">,</span> <span class="s2">&quot;padx&quot;</span><span class="p">:</span> <span class="mi">50</span><span class="p">,</span> <span class="s2">&quot;pady&quot;</span><span class="p">:</span> <span class="mi">20</span><span class="p">}</span>
</span><span id="Mimir.switchView-68"><a href="#Mimir.switchView-68"><span class="linenos">68</span></a>        <span class="n">newFrame</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="o">**</span><span class="n">options</span><span class="p">)</span>
</span><span id="Mimir.switchView-69"><a href="#Mimir.switchView-69"><span class="linenos">69</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="o">.</span><span class="n">rowconfigure</span><span class="p">(</span><span class="n">options</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;row&quot;</span><span class="p">,</span> <span class="mi">0</span><span class="p">),</span> <span class="n">weight</span><span class="o">=</span><span class="mi">1</span><span class="p">)</span>
</span><span id="Mimir.switchView-70"><a href="#Mimir.switchView-70"><span class="linenos">70</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="o">.</span><span class="n">columnconfigure</span><span class="p">(</span><span class="n">options</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;column&quot;</span><span class="p">,</span> <span class="mi">0</span><span class="p">),</span> <span class="n">weight</span><span class="o">=</span><span class="mi">1</span><span class="p">)</span>
</span><span id="Mimir.switchView-71"><a href="#Mimir.switchView-71"><span class="linenos">71</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">currentView</span> <span class="o">=</span> <span class="n">newFrame</span>
</span><span id="Mimir.switchView-72"><a href="#Mimir.switchView-72"><span class="linenos">72</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="o">.</span><span class="n">update_idletasks</span><span class="p">()</span>
</span><span id="Mimir.switchView-73"><a href="#Mimir.switchView-73"><span class="linenos">73</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="o">.</span><span class="n">geometry</span><span class="p">(</span><span class="s2">&quot;&quot;</span><span class="p">)</span>
</span><span id="Mimir.switchView-74"><a href="#Mimir.switchView-74"><span class="linenos">74</span></a>        <span class="k">return</span> <span class="n">newFrame</span>
</span></pre></div>


            <div class="docstring"><p>Replace the current view with a new frame built by newFrameFunc.
Automatically configures geometry and grid placement.</p>
</div>


                            </div>
                            <div id="Mimir.popupView" class="classattr">
                                        <input id="Mimir.popupView-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">popupView</span><span class="signature pdoc-code multiline">(<span class="param">	<span class="bp">self</span>,</span><span class="param">	<span class="n">viewFunc</span>,</span><span class="param">	<span class="n">title</span><span class="o">=</span><span class="s1">&#39;Popup&#39;</span>,</span><span class="param">	<span class="n">size</span><span class="o">=</span><span class="s1">&#39;fit&#39;</span>,</span><span class="param">	<span class="n">width</span><span class="o">=</span><span class="mi">500</span>,</span><span class="param">	<span class="n">height</span><span class="o">=</span><span class="mi">400</span>,</span><span class="param">	<span class="n">modal</span><span class="o">=</span><span class="kc">False</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="Mimir.popupView-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.popupView"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.popupView-76"><a href="#Mimir.popupView-76"><span class="linenos"> 76</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">popupView</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">viewFunc</span><span class="p">,</span> <span class="n">title</span><span class="o">=</span><span class="s2">&quot;Popup&quot;</span><span class="p">,</span> <span class="n">size</span><span class="o">=</span><span class="s2">&quot;fit&quot;</span><span class="p">,</span> <span class="n">width</span><span class="o">=</span><span class="mi">500</span><span class="p">,</span> <span class="n">height</span><span class="o">=</span><span class="mi">400</span><span class="p">,</span> <span class="n">modal</span><span class="o">=</span><span class="kc">False</span><span class="p">):</span>
</span><span id="Mimir.popupView-77"><a href="#Mimir.popupView-77"><span class="linenos"> 77</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir.popupView-78"><a href="#Mimir.popupView-78"><span class="linenos"> 78</span></a><span class="sd">        Open a popup window and embed a view frame into it.</span>
</span><span id="Mimir.popupView-79"><a href="#Mimir.popupView-79"><span class="linenos"> 79</span></a><span class="sd">        If `modal=True`, interaction with other windows is blocked.</span>
</span><span id="Mimir.popupView-80"><a href="#Mimir.popupView-80"><span class="linenos"> 80</span></a><span class="sd">        &quot;&quot;&quot;</span>        
</span><span id="Mimir.popupView-81"><a href="#Mimir.popupView-81"><span class="linenos"> 81</span></a>        <span class="n">popup</span> <span class="o">=</span> <span class="n">tk</span><span class="o">.</span><span class="n">Toplevel</span><span class="p">(</span><span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="p">)</span>
</span><span id="Mimir.popupView-82"><a href="#Mimir.popupView-82"><span class="linenos"> 82</span></a>        <span class="n">popup</span><span class="o">.</span><span class="n">title</span><span class="p">(</span><span class="n">title</span><span class="p">)</span>
</span><span id="Mimir.popupView-83"><a href="#Mimir.popupView-83"><span class="linenos"> 83</span></a>        <span class="n">popup</span><span class="o">.</span><span class="n">transient</span><span class="p">(</span><span class="bp">self</span><span class="o">.</span><span class="n">app</span><span class="p">)</span>
</span><span id="Mimir.popupView-84"><a href="#Mimir.popupView-84"><span class="linenos"> 84</span></a>        <span class="n">popup</span><span class="o">.</span><span class="n">resizable</span><span class="p">(</span><span class="kc">False</span><span class="p">,</span> <span class="kc">False</span><span class="p">)</span>
</span><span id="Mimir.popupView-85"><a href="#Mimir.popupView-85"><span class="linenos"> 85</span></a>
</span><span id="Mimir.popupView-86"><a href="#Mimir.popupView-86"><span class="linenos"> 86</span></a>        <span class="k">if</span> <span class="n">size</span> <span class="o">==</span> <span class="s2">&quot;fit&quot;</span><span class="p">:</span>
</span><span id="Mimir.popupView-87"><a href="#Mimir.popupView-87"><span class="linenos"> 87</span></a>            <span class="n">popup</span><span class="o">.</span><span class="n">update_idletasks</span><span class="p">()</span>
</span><span id="Mimir.popupView-88"><a href="#Mimir.popupView-88"><span class="linenos"> 88</span></a>            <span class="n">popup</span><span class="o">.</span><span class="n">geometry</span><span class="p">(</span><span class="s2">&quot;&quot;</span><span class="p">)</span>
</span><span id="Mimir.popupView-89"><a href="#Mimir.popupView-89"><span class="linenos"> 89</span></a>        <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir.popupView-90"><a href="#Mimir.popupView-90"><span class="linenos"> 90</span></a>            <span class="n">popup</span><span class="o">.</span><span class="n">geometry</span><span class="p">(</span><span class="sa">f</span><span class="s2">&quot;</span><span class="si">{</span><span class="n">width</span><span class="si">}</span><span class="s2">x</span><span class="si">{</span><span class="n">height</span><span class="si">}</span><span class="s2">&quot;</span><span class="p">)</span>
</span><span id="Mimir.popupView-91"><a href="#Mimir.popupView-91"><span class="linenos"> 91</span></a>
</span><span id="Mimir.popupView-92"><a href="#Mimir.popupView-92"><span class="linenos"> 92</span></a>        <span class="n">popup</span><span class="o">.</span><span class="n">grid_rowconfigure</span><span class="p">(</span><span class="mi">0</span><span class="p">,</span> <span class="n">weight</span><span class="o">=</span><span class="mi">1</span><span class="p">)</span>
</span><span id="Mimir.popupView-93"><a href="#Mimir.popupView-93"><span class="linenos"> 93</span></a>        <span class="n">popup</span><span class="o">.</span><span class="n">grid_columnconfigure</span><span class="p">(</span><span class="mi">0</span><span class="p">,</span> <span class="n">weight</span><span class="o">=</span><span class="mi">1</span><span class="p">)</span>
</span><span id="Mimir.popupView-94"><a href="#Mimir.popupView-94"><span class="linenos"> 94</span></a>        <span class="n">viewFrame</span> <span class="o">=</span> <span class="n">viewFunc</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">popup</span><span class="p">)</span>
</span><span id="Mimir.popupView-95"><a href="#Mimir.popupView-95"><span class="linenos"> 95</span></a>        <span class="n">viewFrame</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;nsew&quot;</span><span class="p">)</span>
</span><span id="Mimir.popupView-96"><a href="#Mimir.popupView-96"><span class="linenos"> 96</span></a>
</span><span id="Mimir.popupView-97"><a href="#Mimir.popupView-97"><span class="linenos"> 97</span></a>        <span class="k">if</span> <span class="n">modal</span><span class="p">:</span>
</span><span id="Mimir.popupView-98"><a href="#Mimir.popupView-98"><span class="linenos"> 98</span></a>            <span class="n">popup</span><span class="o">.</span><span class="n">grab_set</span><span class="p">()</span>
</span><span id="Mimir.popupView-99"><a href="#Mimir.popupView-99"><span class="linenos"> 99</span></a>            <span class="n">popup</span><span class="o">.</span><span class="n">focus_set</span><span class="p">()</span>
</span><span id="Mimir.popupView-100"><a href="#Mimir.popupView-100"><span class="linenos">100</span></a>
</span><span id="Mimir.popupView-101"><a href="#Mimir.popupView-101"><span class="linenos">101</span></a>        <span class="k">return</span> <span class="n">popup</span>
</span></pre></div>


            <div class="docstring"><p>Open a popup window and embed a view frame into it.
If <code>modal=True</code>, interaction with other windows is blocked.</p>
</div>


                            </div>
                            <div id="Mimir.renderBlockUI" class="classattr">
                                        <input id="Mimir.renderBlockUI-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">renderBlockUI</span><span class="signature pdoc-code multiline">(<span class="param">	<span class="bp">self</span>,</span><span class="param">	<span class="n">container</span>,</span><span class="param">	<span class="n">fields</span>,</span><span class="param">	<span class="n">blockList</span>,</span><span class="param">	<span class="n">layout</span><span class="o">=</span><span class="s1">&#39;horizontal&#39;</span>,</span><span class="param">	<span class="n">meta</span><span class="o">=</span><span class="kc">None</span>,</span><span class="param">	<span class="n">label</span><span class="o">=</span><span class="kc">None</span>,</span><span class="param">	<span class="n">noRemove</span><span class="o">=</span><span class="kc">False</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="Mimir.renderBlockUI-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.renderBlockUI"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.renderBlockUI-103"><a href="#Mimir.renderBlockUI-103"><span class="linenos">103</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">renderBlockUI</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">container</span><span class="p">,</span> <span class="n">fields</span><span class="p">,</span> <span class="n">blockList</span><span class="p">,</span> <span class="n">layout</span><span class="o">=</span><span class="s2">&quot;horizontal&quot;</span><span class="p">,</span> <span class="n">meta</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="n">label</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="n">noRemove</span><span class="o">=</span><span class="kc">False</span><span class="p">):</span>
</span><span id="Mimir.renderBlockUI-104"><a href="#Mimir.renderBlockUI-104"><span class="linenos">104</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir.renderBlockUI-105"><a href="#Mimir.renderBlockUI-105"><span class="linenos">105</span></a><span class="sd">        Render a labeled input block with specified fields and layout style.</span>
</span><span id="Mimir.renderBlockUI-106"><a href="#Mimir.renderBlockUI-106"><span class="linenos">106</span></a><span class="sd">        Automatically attaches the block to blockList and handles UI labels and deletion logic.</span>
</span><span id="Mimir.renderBlockUI-107"><a href="#Mimir.renderBlockUI-107"><span class="linenos">107</span></a><span class="sd">        &quot;&quot;&quot;</span>        
</span><span id="Mimir.renderBlockUI-108"><a href="#Mimir.renderBlockUI-108"><span class="linenos">108</span></a>        <span class="n">index</span> <span class="o">=</span> <span class="nb">len</span><span class="p">(</span><span class="n">blockList</span><span class="p">)</span> <span class="o">+</span> <span class="mi">1</span>
</span><span id="Mimir.renderBlockUI-109"><a href="#Mimir.renderBlockUI-109"><span class="linenos">109</span></a>        <span class="n">labelText</span> <span class="o">=</span> <span class="n">label</span> <span class="ow">or</span> <span class="sa">f</span><span class="s2">&quot;Block </span><span class="si">{</span><span class="n">index</span><span class="si">}</span><span class="s2">&quot;</span>
</span><span id="Mimir.renderBlockUI-110"><a href="#Mimir.renderBlockUI-110"><span class="linenos">110</span></a>        <span class="n">frame</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">LabelFrame</span><span class="p">(</span><span class="n">container</span><span class="p">,</span> <span class="n">text</span><span class="o">=</span><span class="n">labelText</span><span class="p">)</span>
</span><span id="Mimir.renderBlockUI-111"><a href="#Mimir.renderBlockUI-111"><span class="linenos">111</span></a>        <span class="n">frame</span><span class="o">.</span><span class="n">configure</span><span class="p">(</span><span class="n">labelanchor</span><span class="o">=</span><span class="s2">&quot;n&quot;</span><span class="p">)</span> 
</span><span id="Mimir.renderBlockUI-112"><a href="#Mimir.renderBlockUI-112"><span class="linenos">112</span></a>        
</span><span id="Mimir.renderBlockUI-113"><a href="#Mimir.renderBlockUI-113"><span class="linenos">113</span></a>        <span class="k">if</span> <span class="n">layout</span> <span class="o">==</span> <span class="s2">&quot;vertical&quot;</span><span class="p">:</span>
</span><span id="Mimir.renderBlockUI-114"><a href="#Mimir.renderBlockUI-114"><span class="linenos">114</span></a>            <span class="n">frame</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">index</span> <span class="o">-</span> <span class="mi">1</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">10</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="mi">10</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;w&quot;</span><span class="p">)</span>
</span><span id="Mimir.renderBlockUI-115"><a href="#Mimir.renderBlockUI-115"><span class="linenos">115</span></a>        <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir.renderBlockUI-116"><a href="#Mimir.renderBlockUI-116"><span class="linenos">116</span></a>            <span class="n">frame</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">index</span> <span class="o">-</span> <span class="mi">1</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">10</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="mi">10</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;n&quot;</span><span class="p">)</span>
</span><span id="Mimir.renderBlockUI-117"><a href="#Mimir.renderBlockUI-117"><span class="linenos">117</span></a>
</span><span id="Mimir.renderBlockUI-118"><a href="#Mimir.renderBlockUI-118"><span class="linenos">118</span></a>        <span class="n">boldFont</span> <span class="o">=</span> <span class="bp">self</span><span class="o">.</span><span class="n">getCurrentFont</span><span class="p">()</span>
</span><span id="Mimir.renderBlockUI-119"><a href="#Mimir.renderBlockUI-119"><span class="linenos">119</span></a>        <span class="n">boldFont</span><span class="o">.</span><span class="n">configure</span><span class="p">(</span><span class="n">weight</span><span class="o">=</span><span class="s2">&quot;bold&quot;</span><span class="p">)</span>
</span><span id="Mimir.renderBlockUI-120"><a href="#Mimir.renderBlockUI-120"><span class="linenos">120</span></a>
</span><span id="Mimir.renderBlockUI-121"><a href="#Mimir.renderBlockUI-121"><span class="linenos">121</span></a>        <span class="k">if</span> <span class="n">meta</span> <span class="ow">is</span> <span class="kc">None</span><span class="p">:</span>
</span><span id="Mimir.renderBlockUI-122"><a href="#Mimir.renderBlockUI-122"><span class="linenos">122</span></a>            <span class="n">meta</span> <span class="o">=</span> <span class="p">{}</span>
</span><span id="Mimir.renderBlockUI-123"><a href="#Mimir.renderBlockUI-123"><span class="linenos">123</span></a>        <span class="k">if</span> <span class="s2">&quot;custom_label&quot;</span> <span class="ow">not</span> <span class="ow">in</span> <span class="n">meta</span> <span class="ow">and</span> <span class="n">label</span><span class="p">:</span>
</span><span id="Mimir.renderBlockUI-124"><a href="#Mimir.renderBlockUI-124"><span class="linenos">124</span></a>            <span class="n">meta</span><span class="p">[</span><span class="s2">&quot;custom_label&quot;</span><span class="p">]</span> <span class="o">=</span> <span class="n">label</span>
</span><span id="Mimir.renderBlockUI-125"><a href="#Mimir.renderBlockUI-125"><span class="linenos">125</span></a>
</span><span id="Mimir.renderBlockUI-126"><a href="#Mimir.renderBlockUI-126"><span class="linenos">126</span></a>        <span class="n">blockMeta</span> <span class="o">=</span> <span class="p">{</span>
</span><span id="Mimir.renderBlockUI-127"><a href="#Mimir.renderBlockUI-127"><span class="linenos">127</span></a>            <span class="s2">&quot;frame&quot;</span><span class="p">:</span> <span class="n">frame</span><span class="p">,</span>
</span><span id="Mimir.renderBlockUI-128"><a href="#Mimir.renderBlockUI-128"><span class="linenos">128</span></a>            <span class="s2">&quot;fields&quot;</span><span class="p">:</span> <span class="n">fields</span><span class="p">,</span>
</span><span id="Mimir.renderBlockUI-129"><a href="#Mimir.renderBlockUI-129"><span class="linenos">129</span></a>            <span class="s2">&quot;fields_by_key&quot;</span><span class="p">:</span> <span class="p">{</span><span class="n">f</span><span class="p">[</span><span class="s2">&quot;key&quot;</span><span class="p">]:</span> <span class="n">f</span><span class="p">[</span><span class="s2">&quot;variable&quot;</span><span class="p">]</span> <span class="k">for</span> <span class="n">f</span> <span class="ow">in</span> <span class="n">fields</span> <span class="k">if</span> <span class="s2">&quot;key&quot;</span> <span class="ow">in</span> <span class="n">f</span><span class="p">},</span>
</span><span id="Mimir.renderBlockUI-130"><a href="#Mimir.renderBlockUI-130"><span class="linenos">130</span></a>            <span class="s2">&quot;layout&quot;</span><span class="p">:</span> <span class="n">layout</span><span class="p">,</span>
</span><span id="Mimir.renderBlockUI-131"><a href="#Mimir.renderBlockUI-131"><span class="linenos">131</span></a>            <span class="o">**</span><span class="n">meta</span>
</span><span id="Mimir.renderBlockUI-132"><a href="#Mimir.renderBlockUI-132"><span class="linenos">132</span></a>        <span class="p">}</span>
</span><span id="Mimir.renderBlockUI-133"><a href="#Mimir.renderBlockUI-133"><span class="linenos">133</span></a>
</span><span id="Mimir.renderBlockUI-134"><a href="#Mimir.renderBlockUI-134"><span class="linenos">134</span></a>
</span><span id="Mimir.renderBlockUI-135"><a href="#Mimir.renderBlockUI-135"><span class="linenos">135</span></a>        <span class="n">wrapper</span> <span class="o">=</span> <span class="n">DataBlockWrapper</span><span class="p">(</span><span class="n">blockMeta</span><span class="p">)</span>
</span><span id="Mimir.renderBlockUI-136"><a href="#Mimir.renderBlockUI-136"><span class="linenos">136</span></a>        <span class="n">blockList</span><span class="o">.</span><span class="n">append</span><span class="p">(</span><span class="n">wrapper</span><span class="p">)</span>
</span><span id="Mimir.renderBlockUI-137"><a href="#Mimir.renderBlockUI-137"><span class="linenos">137</span></a>
</span><span id="Mimir.renderBlockUI-138"><a href="#Mimir.renderBlockUI-138"><span class="linenos">138</span></a>        <span class="n">row</span> <span class="o">=</span> <span class="mi">0</span>
</span><span id="Mimir.renderBlockUI-139"><a href="#Mimir.renderBlockUI-139"><span class="linenos">139</span></a>        <span class="k">for</span> <span class="n">field</span> <span class="ow">in</span> <span class="n">fields</span><span class="p">:</span>
</span><span id="Mimir.renderBlockUI-140"><a href="#Mimir.renderBlockUI-140"><span class="linenos">140</span></a>            <span class="n">ftype</span> <span class="o">=</span> <span class="n">field</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;type&quot;</span><span class="p">,</span> <span class="s2">&quot;entry&quot;</span><span class="p">)</span>
</span><span id="Mimir.renderBlockUI-141"><a href="#Mimir.renderBlockUI-141"><span class="linenos">141</span></a>            <span class="n">renderer</span> <span class="o">=</span> <span class="bp">self</span><span class="o">.</span><span class="n">fieldRenderers</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="n">ftype</span><span class="p">)</span>
</span><span id="Mimir.renderBlockUI-142"><a href="#Mimir.renderBlockUI-142"><span class="linenos">142</span></a>            <span class="k">if</span> <span class="n">renderer</span><span class="p">:</span>
</span><span id="Mimir.renderBlockUI-143"><a href="#Mimir.renderBlockUI-143"><span class="linenos">143</span></a>                <span class="n">renderer</span><span class="p">(</span><span class="n">frame</span><span class="p">,</span> <span class="n">row</span><span class="p">,</span> <span class="n">field</span><span class="p">)</span>
</span><span id="Mimir.renderBlockUI-144"><a href="#Mimir.renderBlockUI-144"><span class="linenos">144</span></a>
</span><span id="Mimir.renderBlockUI-145"><a href="#Mimir.renderBlockUI-145"><span class="linenos">145</span></a>                <span class="c1"># Adjust row offset based on field type</span>
</span><span id="Mimir.renderBlockUI-146"><a href="#Mimir.renderBlockUI-146"><span class="linenos">146</span></a>                <span class="k">if</span> <span class="n">ftype</span> <span class="o">==</span> <span class="s2">&quot;fileupload&quot;</span><span class="p">:</span>
</span><span id="Mimir.renderBlockUI-147"><a href="#Mimir.renderBlockUI-147"><span class="linenos">147</span></a>                    <span class="n">row</span> <span class="o">+=</span> <span class="mi">3</span>
</span><span id="Mimir.renderBlockUI-148"><a href="#Mimir.renderBlockUI-148"><span class="linenos">148</span></a>                <span class="k">elif</span> <span class="n">ftype</span> <span class="o">==</span> <span class="s2">&quot;heading&quot;</span><span class="p">:</span>
</span><span id="Mimir.renderBlockUI-149"><a href="#Mimir.renderBlockUI-149"><span class="linenos">149</span></a>                    <span class="n">row</span> <span class="o">+=</span> <span class="mi">1</span>
</span><span id="Mimir.renderBlockUI-150"><a href="#Mimir.renderBlockUI-150"><span class="linenos">150</span></a>                <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir.renderBlockUI-151"><a href="#Mimir.renderBlockUI-151"><span class="linenos">151</span></a>                    <span class="n">row</span> <span class="o">+=</span> <span class="mi">2</span>
</span><span id="Mimir.renderBlockUI-152"><a href="#Mimir.renderBlockUI-152"><span class="linenos">152</span></a>
</span><span id="Mimir.renderBlockUI-153"><a href="#Mimir.renderBlockUI-153"><span class="linenos">153</span></a>        <span class="k">if</span> <span class="ow">not</span> <span class="n">noRemove</span><span class="p">:</span>
</span><span id="Mimir.renderBlockUI-154"><a href="#Mimir.renderBlockUI-154"><span class="linenos">154</span></a>            <span class="n">removeBtn</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Button</span><span class="p">(</span>
</span><span id="Mimir.renderBlockUI-155"><a href="#Mimir.renderBlockUI-155"><span class="linenos">155</span></a>                <span class="n">frame</span><span class="p">,</span>
</span><span id="Mimir.renderBlockUI-156"><a href="#Mimir.renderBlockUI-156"><span class="linenos">156</span></a>                <span class="n">text</span><span class="o">=</span><span class="s2">&quot;🗑️ Ta bort&quot;</span><span class="p">,</span>
</span><span id="Mimir.renderBlockUI-157"><a href="#Mimir.renderBlockUI-157"><span class="linenos">157</span></a>                <span class="n">bootstyle</span><span class="o">=</span><span class="s2">&quot;danger-outline&quot;</span><span class="p">,</span>
</span><span id="Mimir.renderBlockUI-158"><a href="#Mimir.renderBlockUI-158"><span class="linenos">158</span></a>                <span class="n">command</span><span class="o">=</span><span class="k">lambda</span> <span class="n">w</span><span class="o">=</span><span class="n">wrapper</span><span class="p">:</span> <span class="bp">self</span><span class="o">.</span><span class="n">removeBlock</span><span class="p">(</span><span class="n">blockList</span><span class="p">,</span> <span class="n">w</span><span class="p">)</span>
</span><span id="Mimir.renderBlockUI-159"><a href="#Mimir.renderBlockUI-159"><span class="linenos">159</span></a>            <span class="p">)</span>
</span><span id="Mimir.renderBlockUI-160"><a href="#Mimir.renderBlockUI-160"><span class="linenos">160</span></a>            <span class="n">removeBtn</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="mi">2</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="p">(</span><span class="mi">10</span><span class="p">,</span> <span class="mi">5</span><span class="p">))</span>
</span><span id="Mimir.renderBlockUI-161"><a href="#Mimir.renderBlockUI-161"><span class="linenos">161</span></a>
</span><span id="Mimir.renderBlockUI-162"><a href="#Mimir.renderBlockUI-162"><span class="linenos">162</span></a>        <span class="k">return</span> <span class="n">wrapper</span>
</span></pre></div>


            <div class="docstring"><p>Render a labeled input block with specified fields and layout style.
Automatically attaches the block to blockList and handles UI labels and deletion logic.</p>
</div>


                            </div>
                            <div id="Mimir.registerFieldType" class="classattr">
                                        <input id="Mimir.registerFieldType-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">registerFieldType</span><span class="signature pdoc-code condensed">(<span class="param"><span class="bp">self</span>, </span><span class="param"><span class="n">name</span>, </span><span class="param"><span class="n">renderer</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="Mimir.registerFieldType-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.registerFieldType"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.registerFieldType-165"><a href="#Mimir.registerFieldType-165"><span class="linenos">165</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">registerFieldType</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">name</span><span class="p">,</span> <span class="n">renderer</span><span class="p">):</span>
</span><span id="Mimir.registerFieldType-166"><a href="#Mimir.registerFieldType-166"><span class="linenos">166</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Register a custom field renderer by name.&quot;&quot;&quot;</span>        
</span><span id="Mimir.registerFieldType-167"><a href="#Mimir.registerFieldType-167"><span class="linenos">167</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">fieldRenderers</span><span class="p">[</span><span class="n">name</span><span class="p">]</span> <span class="o">=</span> <span class="n">renderer</span>
</span></pre></div>


            <div class="docstring"><p>Register a custom field renderer by name.</p>
</div>


                            </div>
                            <div id="Mimir.markInvalid" class="classattr">
                                        <input id="Mimir.markInvalid-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">markInvalid</span><span class="signature pdoc-code condensed">(<span class="param"><span class="bp">self</span>, </span><span class="param"><span class="n">widget</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="Mimir.markInvalid-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.markInvalid"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.markInvalid-218"><a href="#Mimir.markInvalid-218"><span class="linenos">218</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">markInvalid</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">widget</span><span class="p">):</span>
</span><span id="Mimir.markInvalid-219"><a href="#Mimir.markInvalid-219"><span class="linenos">219</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Mark a widget as invalid with red border styling.&quot;&quot;&quot;</span>
</span><span id="Mimir.markInvalid-220"><a href="#Mimir.markInvalid-220"><span class="linenos">220</span></a>        <span class="n">widget</span><span class="o">.</span><span class="n">config</span><span class="p">(</span><span class="n">bootstyle</span><span class="o">=</span><span class="s2">&quot;danger&quot;</span><span class="p">)</span>
</span><span id="Mimir.markInvalid-221"><a href="#Mimir.markInvalid-221"><span class="linenos">221</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">invalidEntries</span><span class="o">.</span><span class="n">add</span><span class="p">(</span><span class="n">widget</span><span class="p">)</span>
</span></pre></div>


            <div class="docstring"><p>Mark a widget as invalid with red border styling.</p>
</div>


                            </div>
                            <div id="Mimir.clearInvalid" class="classattr">
                                        <input id="Mimir.clearInvalid-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">clearInvalid</span><span class="signature pdoc-code condensed">(<span class="param"><span class="bp">self</span>, </span><span class="param"><span class="n">widget</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="Mimir.clearInvalid-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.clearInvalid"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.clearInvalid-223"><a href="#Mimir.clearInvalid-223"><span class="linenos">223</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">clearInvalid</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">widget</span><span class="p">):</span>
</span><span id="Mimir.clearInvalid-224"><a href="#Mimir.clearInvalid-224"><span class="linenos">224</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Clear error styling from a widget.&quot;&quot;&quot;</span>
</span><span id="Mimir.clearInvalid-225"><a href="#Mimir.clearInvalid-225"><span class="linenos">225</span></a>        <span class="n">widget</span><span class="o">.</span><span class="n">config</span><span class="p">(</span><span class="n">bootstyle</span><span class="o">=</span><span class="s2">&quot;none&quot;</span><span class="p">)</span>
</span><span id="Mimir.clearInvalid-226"><a href="#Mimir.clearInvalid-226"><span class="linenos">226</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">invalidEntries</span><span class="o">.</span><span class="n">discard</span><span class="p">(</span><span class="n">widget</span><span class="p">)</span>
</span></pre></div>


            <div class="docstring"><p>Clear error styling from a widget.</p>
</div>


                            </div>
                            <div id="Mimir.addNextButton" class="classattr">
                                        <input id="Mimir.addNextButton-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">addNextButton</span><span class="signature pdoc-code multiline">(<span class="param">	<span class="bp">self</span>,</span><span class="param">	<span class="n">parent</span>,</span><span class="param">	<span class="n">row</span>,</span><span class="param">	<span class="n">command</span>,</span><span class="param">	<span class="n">label</span><span class="o">=</span><span class="s1">&#39;Nästa&#39;</span>,</span><span class="param">	<span class="n">tooltip</span><span class="o">=</span><span class="kc">None</span>,</span><span class="param">	<span class="n">column</span><span class="o">=</span><span class="kc">None</span>,</span><span class="param">	<span class="n">columnspan</span><span class="o">=</span><span class="kc">None</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="Mimir.addNextButton-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.addNextButton"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.addNextButton-228"><a href="#Mimir.addNextButton-228"><span class="linenos">228</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">addNextButton</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">parent</span><span class="p">,</span> <span class="n">row</span><span class="p">,</span> <span class="n">command</span><span class="p">,</span> <span class="n">label</span><span class="o">=</span><span class="s2">&quot;Nästa&quot;</span><span class="p">,</span> <span class="n">tooltip</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="kc">None</span><span class="p">):</span>
</span><span id="Mimir.addNextButton-229"><a href="#Mimir.addNextButton-229"><span class="linenos">229</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir.addNextButton-230"><a href="#Mimir.addNextButton-230"><span class="linenos">230</span></a><span class="sd">        Add a &#39;Next&#39; button to the given row. Automatically spans the full width</span>
</span><span id="Mimir.addNextButton-231"><a href="#Mimir.addNextButton-231"><span class="linenos">231</span></a><span class="sd">        of horizontal blocks if no column or columnspan is provided.</span>
</span><span id="Mimir.addNextButton-232"><a href="#Mimir.addNextButton-232"><span class="linenos">232</span></a><span class="sd">        Blocks execution if any fields are invalid.</span>
</span><span id="Mimir.addNextButton-233"><a href="#Mimir.addNextButton-233"><span class="linenos">233</span></a><span class="sd">        &quot;&quot;&quot;</span>        
</span><span id="Mimir.addNextButton-234"><a href="#Mimir.addNextButton-234"><span class="linenos">234</span></a>        <span class="k">def</span><span class="w"> </span><span class="nf">wrapped</span><span class="p">():</span>
</span><span id="Mimir.addNextButton-235"><a href="#Mimir.addNextButton-235"><span class="linenos">235</span></a>            <span class="k">if</span> <span class="bp">self</span><span class="o">.</span><span class="n">invalidEntries</span><span class="p">:</span>
</span><span id="Mimir.addNextButton-236"><a href="#Mimir.addNextButton-236"><span class="linenos">236</span></a>                <span class="n">msg</span><span class="o">.</span><span class="n">showerror</span><span class="p">(</span><span class="s2">&quot;Fel&quot;</span><span class="p">,</span> <span class="s2">&quot;Alla fält måste vara giltiga innan du kan fortsätta.&quot;</span><span class="p">)</span>
</span><span id="Mimir.addNextButton-237"><a href="#Mimir.addNextButton-237"><span class="linenos">237</span></a>                <span class="k">return</span>
</span><span id="Mimir.addNextButton-238"><a href="#Mimir.addNextButton-238"><span class="linenos">238</span></a>            <span class="n">command</span><span class="p">()</span>
</span><span id="Mimir.addNextButton-239"><a href="#Mimir.addNextButton-239"><span class="linenos">239</span></a>
</span><span id="Mimir.addNextButton-240"><a href="#Mimir.addNextButton-240"><span class="linenos">240</span></a>        <span class="c1"># Calculate default columnspan if not explicitly provided</span>
</span><span id="Mimir.addNextButton-241"><a href="#Mimir.addNextButton-241"><span class="linenos">241</span></a>        <span class="k">if</span> <span class="n">column</span> <span class="ow">is</span> <span class="kc">None</span> <span class="ow">or</span> <span class="n">columnspan</span> <span class="ow">is</span> <span class="kc">None</span><span class="p">:</span>
</span><span id="Mimir.addNextButton-242"><a href="#Mimir.addNextButton-242"><span class="linenos">242</span></a>            <span class="c1"># Find all unique columns used in row=0 (block layout)</span>
</span><span id="Mimir.addNextButton-243"><a href="#Mimir.addNextButton-243"><span class="linenos">243</span></a>            <span class="n">used_columns</span> <span class="o">=</span> <span class="nb">set</span><span class="p">()</span>
</span><span id="Mimir.addNextButton-244"><a href="#Mimir.addNextButton-244"><span class="linenos">244</span></a>            <span class="k">for</span> <span class="n">child</span> <span class="ow">in</span> <span class="n">parent</span><span class="o">.</span><span class="n">winfo_children</span><span class="p">():</span>
</span><span id="Mimir.addNextButton-245"><a href="#Mimir.addNextButton-245"><span class="linenos">245</span></a>                <span class="n">grid</span> <span class="o">=</span> <span class="n">child</span><span class="o">.</span><span class="n">grid_info</span><span class="p">()</span>
</span><span id="Mimir.addNextButton-246"><a href="#Mimir.addNextButton-246"><span class="linenos">246</span></a>                <span class="k">if</span> <span class="n">grid</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;row&quot;</span><span class="p">)</span> <span class="o">==</span> <span class="mi">0</span> <span class="ow">and</span> <span class="s2">&quot;column&quot;</span> <span class="ow">in</span> <span class="n">grid</span><span class="p">:</span>
</span><span id="Mimir.addNextButton-247"><a href="#Mimir.addNextButton-247"><span class="linenos">247</span></a>                    <span class="n">used_columns</span><span class="o">.</span><span class="n">add</span><span class="p">(</span><span class="nb">int</span><span class="p">(</span><span class="n">grid</span><span class="p">[</span><span class="s2">&quot;column&quot;</span><span class="p">]))</span>
</span><span id="Mimir.addNextButton-248"><a href="#Mimir.addNextButton-248"><span class="linenos">248</span></a>
</span><span id="Mimir.addNextButton-249"><a href="#Mimir.addNextButton-249"><span class="linenos">249</span></a>            <span class="k">if</span> <span class="ow">not</span> <span class="n">used_columns</span><span class="p">:</span>
</span><span id="Mimir.addNextButton-250"><a href="#Mimir.addNextButton-250"><span class="linenos">250</span></a>                <span class="n">column</span> <span class="o">=</span> <span class="mi">0</span>
</span><span id="Mimir.addNextButton-251"><a href="#Mimir.addNextButton-251"><span class="linenos">251</span></a>                <span class="n">columnspan</span> <span class="o">=</span> <span class="mi">1</span>
</span><span id="Mimir.addNextButton-252"><a href="#Mimir.addNextButton-252"><span class="linenos">252</span></a>            <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir.addNextButton-253"><a href="#Mimir.addNextButton-253"><span class="linenos">253</span></a>                <span class="n">column</span> <span class="o">=</span> <span class="mi">0</span>
</span><span id="Mimir.addNextButton-254"><a href="#Mimir.addNextButton-254"><span class="linenos">254</span></a>                <span class="n">columnspan</span> <span class="o">=</span> <span class="nb">max</span><span class="p">(</span><span class="n">used_columns</span><span class="p">)</span> <span class="o">+</span> <span class="mi">1</span>
</span><span id="Mimir.addNextButton-255"><a href="#Mimir.addNextButton-255"><span class="linenos">255</span></a>
</span><span id="Mimir.addNextButton-256"><a href="#Mimir.addNextButton-256"><span class="linenos">256</span></a>        <span class="n">btn</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Button</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">text</span><span class="o">=</span><span class="n">label</span><span class="p">,</span> <span class="n">command</span><span class="o">=</span><span class="n">wrapped</span><span class="p">,</span> <span class="n">bootstyle</span><span class="o">=</span><span class="s2">&quot;success&quot;</span><span class="p">)</span>
</span><span id="Mimir.addNextButton-257"><a href="#Mimir.addNextButton-257"><span class="linenos">257</span></a>        <span class="n">btn</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="n">columnspan</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="p">(</span><span class="mi">10</span><span class="p">,</span> <span class="mi">5</span><span class="p">))</span>
</span><span id="Mimir.addNextButton-258"><a href="#Mimir.addNextButton-258"><span class="linenos">258</span></a>
</span><span id="Mimir.addNextButton-259"><a href="#Mimir.addNextButton-259"><span class="linenos">259</span></a>        <span class="k">if</span> <span class="n">tooltip</span><span class="p">:</span>
</span><span id="Mimir.addNextButton-260"><a href="#Mimir.addNextButton-260"><span class="linenos">260</span></a>            <span class="bp">self</span><span class="o">.</span><span class="n">addTooltip</span><span class="p">(</span><span class="n">btn</span><span class="p">,</span> <span class="n">tooltip</span><span class="p">)</span>
</span><span id="Mimir.addNextButton-261"><a href="#Mimir.addNextButton-261"><span class="linenos">261</span></a>
</span><span id="Mimir.addNextButton-262"><a href="#Mimir.addNextButton-262"><span class="linenos">262</span></a>        <span class="k">return</span> <span class="n">btn</span>
</span></pre></div>


            <div class="docstring"><p>Add a 'Next' button to the given row. Automatically spans the full width
of horizontal blocks if no column or columnspan is provided.
Blocks execution if any fields are invalid.</p>
</div>


                            </div>
                            <div id="Mimir.getCurrentFont" class="classattr">
                                        <input id="Mimir.getCurrentFont-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">getCurrentFont</span><span class="signature pdoc-code condensed">(<span class="param"><span class="bp">self</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="Mimir.getCurrentFont-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.getCurrentFont"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.getCurrentFont-265"><a href="#Mimir.getCurrentFont-265"><span class="linenos">265</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">getCurrentFont</span><span class="p">(</span><span class="bp">self</span><span class="p">):</span>
</span><span id="Mimir.getCurrentFont-266"><a href="#Mimir.getCurrentFont-266"><span class="linenos">266</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Return the default font currently used by ttkbootstrap widgets.&quot;&quot;&quot;</span>
</span><span id="Mimir.getCurrentFont-267"><a href="#Mimir.getCurrentFont-267"><span class="linenos">267</span></a>        <span class="n">style</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Style</span><span class="p">()</span>
</span><span id="Mimir.getCurrentFont-268"><a href="#Mimir.getCurrentFont-268"><span class="linenos">268</span></a>        <span class="n">fontName</span> <span class="o">=</span> <span class="n">style</span><span class="o">.</span><span class="n">lookup</span><span class="p">(</span><span class="s2">&quot;TLabel&quot;</span><span class="p">,</span> <span class="s2">&quot;font&quot;</span><span class="p">)</span>
</span><span id="Mimir.getCurrentFont-269"><a href="#Mimir.getCurrentFont-269"><span class="linenos">269</span></a>        <span class="k">return</span> <span class="n">tkFont</span><span class="o">.</span><span class="n">Font</span><span class="p">(</span><span class="n">font</span><span class="o">=</span><span class="n">fontName</span><span class="p">)</span>
</span></pre></div>


            <div class="docstring"><p>Return the default font currently used by ttkbootstrap widgets.</p>
</div>


                            </div>
                            <div id="Mimir.addTooltip" class="classattr">
                                        <input id="Mimir.addTooltip-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">addTooltip</span><span class="signature pdoc-code condensed">(<span class="param"><span class="bp">self</span>, </span><span class="param"><span class="n">widget</span>, </span><span class="param"><span class="n">text</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="Mimir.addTooltip-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.addTooltip"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.addTooltip-271"><a href="#Mimir.addTooltip-271"><span class="linenos">271</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">addTooltip</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">widget</span><span class="p">,</span> <span class="n">text</span><span class="p">):</span>
</span><span id="Mimir.addTooltip-272"><a href="#Mimir.addTooltip-272"><span class="linenos">272</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Attach a styled tooltip to a widget.&quot;&quot;&quot;</span>
</span><span id="Mimir.addTooltip-273"><a href="#Mimir.addTooltip-273"><span class="linenos">273</span></a>        <span class="n">ToolTip</span><span class="p">(</span><span class="n">widget</span><span class="p">,</span> <span class="n">text</span><span class="p">,</span> <span class="n">bootstyle</span><span class="o">=</span><span class="p">(</span><span class="n">SECONDARY</span><span class="p">,</span> <span class="n">INVERSE</span><span class="p">))</span>
</span></pre></div>


            <div class="docstring"><p>Attach a styled tooltip to a widget.</p>
</div>


                            </div>
                            <div id="Mimir.removeBlock" class="classattr">
                                        <input id="Mimir.removeBlock-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">removeBlock</span><span class="signature pdoc-code condensed">(<span class="param"><span class="bp">self</span>, </span><span class="param"><span class="n">blockList</span>, </span><span class="param"><span class="n">wrapper</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="Mimir.removeBlock-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.removeBlock"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.removeBlock-275"><a href="#Mimir.removeBlock-275"><span class="linenos">275</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">removeBlock</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">blockList</span><span class="p">,</span> <span class="n">wrapper</span><span class="p">):</span>
</span><span id="Mimir.removeBlock-276"><a href="#Mimir.removeBlock-276"><span class="linenos">276</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir.removeBlock-277"><a href="#Mimir.removeBlock-277"><span class="linenos">277</span></a><span class="sd">        Remove a UI block from view and list. Automatically reindexes</span>
</span><span id="Mimir.removeBlock-278"><a href="#Mimir.removeBlock-278"><span class="linenos">278</span></a><span class="sd">        and repositions remaining blocks.</span>
</span><span id="Mimir.removeBlock-279"><a href="#Mimir.removeBlock-279"><span class="linenos">279</span></a><span class="sd">        &quot;&quot;&quot;</span>        
</span><span id="Mimir.removeBlock-280"><a href="#Mimir.removeBlock-280"><span class="linenos">280</span></a>        <span class="n">frame</span> <span class="o">=</span> <span class="n">wrapper</span><span class="o">.</span><span class="n">meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;frame&quot;</span><span class="p">)</span>
</span><span id="Mimir.removeBlock-281"><a href="#Mimir.removeBlock-281"><span class="linenos">281</span></a>        <span class="n">layout</span> <span class="o">=</span> <span class="n">wrapper</span><span class="o">.</span><span class="n">meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;layout&quot;</span><span class="p">,</span> <span class="s2">&quot;horizontal&quot;</span><span class="p">)</span>
</span><span id="Mimir.removeBlock-282"><a href="#Mimir.removeBlock-282"><span class="linenos">282</span></a>        <span class="k">if</span> <span class="n">frame</span><span class="p">:</span>
</span><span id="Mimir.removeBlock-283"><a href="#Mimir.removeBlock-283"><span class="linenos">283</span></a>            <span class="n">frame</span><span class="o">.</span><span class="n">destroy</span><span class="p">()</span>
</span><span id="Mimir.removeBlock-284"><a href="#Mimir.removeBlock-284"><span class="linenos">284</span></a>        <span class="n">blockList</span><span class="o">.</span><span class="n">remove</span><span class="p">(</span><span class="n">wrapper</span><span class="p">)</span>
</span><span id="Mimir.removeBlock-285"><a href="#Mimir.removeBlock-285"><span class="linenos">285</span></a>
</span><span id="Mimir.removeBlock-286"><a href="#Mimir.removeBlock-286"><span class="linenos">286</span></a>        <span class="k">for</span> <span class="n">i</span><span class="p">,</span> <span class="n">block</span> <span class="ow">in</span> <span class="nb">enumerate</span><span class="p">(</span><span class="n">blockList</span><span class="p">):</span>
</span><span id="Mimir.removeBlock-287"><a href="#Mimir.removeBlock-287"><span class="linenos">287</span></a>            <span class="n">frame</span> <span class="o">=</span> <span class="n">block</span><span class="o">.</span><span class="n">meta</span><span class="p">[</span><span class="s2">&quot;frame&quot;</span><span class="p">]</span>
</span><span id="Mimir.removeBlock-288"><a href="#Mimir.removeBlock-288"><span class="linenos">288</span></a>            <span class="n">new_label</span> <span class="o">=</span> <span class="sa">f</span><span class="s2">&quot;Butik </span><span class="si">{</span><span class="n">i</span><span class="w"> </span><span class="o">+</span><span class="w"> </span><span class="mi">1</span><span class="si">}</span><span class="s2">&quot;</span>
</span><span id="Mimir.removeBlock-289"><a href="#Mimir.removeBlock-289"><span class="linenos">289</span></a>
</span><span id="Mimir.removeBlock-290"><a href="#Mimir.removeBlock-290"><span class="linenos">290</span></a>            <span class="c1"># Update visual label</span>
</span><span id="Mimir.removeBlock-291"><a href="#Mimir.removeBlock-291"><span class="linenos">291</span></a>            <span class="n">frame</span><span class="o">.</span><span class="n">config</span><span class="p">(</span><span class="n">text</span><span class="o">=</span><span class="n">new_label</span><span class="p">)</span>
</span><span id="Mimir.removeBlock-292"><a href="#Mimir.removeBlock-292"><span class="linenos">292</span></a>
</span><span id="Mimir.removeBlock-293"><a href="#Mimir.removeBlock-293"><span class="linenos">293</span></a>            <span class="c1"># Update meta so future logic knows the new label/index</span>
</span><span id="Mimir.removeBlock-294"><a href="#Mimir.removeBlock-294"><span class="linenos">294</span></a>            <span class="n">block</span><span class="o">.</span><span class="n">meta</span><span class="p">[</span><span class="s2">&quot;custom_label&quot;</span><span class="p">]</span> <span class="o">=</span> <span class="n">new_label</span>
</span><span id="Mimir.removeBlock-295"><a href="#Mimir.removeBlock-295"><span class="linenos">295</span></a>            <span class="n">block</span><span class="o">.</span><span class="n">meta</span><span class="p">[</span><span class="s2">&quot;store_id&quot;</span><span class="p">]</span> <span class="o">=</span> <span class="n">i</span> <span class="o">+</span> <span class="mi">1</span>  <span class="c1"># optional if store_id should follow display order</span>
</span><span id="Mimir.removeBlock-296"><a href="#Mimir.removeBlock-296"><span class="linenos">296</span></a>
</span><span id="Mimir.removeBlock-297"><a href="#Mimir.removeBlock-297"><span class="linenos">297</span></a>            <span class="c1"># Reposition depending on layout</span>
</span><span id="Mimir.removeBlock-298"><a href="#Mimir.removeBlock-298"><span class="linenos">298</span></a>            <span class="k">if</span> <span class="n">layout</span> <span class="o">==</span> <span class="s2">&quot;vertical&quot;</span><span class="p">:</span>
</span><span id="Mimir.removeBlock-299"><a href="#Mimir.removeBlock-299"><span class="linenos">299</span></a>                <span class="n">frame</span><span class="o">.</span><span class="n">grid_configure</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">i</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">)</span>
</span><span id="Mimir.removeBlock-300"><a href="#Mimir.removeBlock-300"><span class="linenos">300</span></a>            <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir.removeBlock-301"><a href="#Mimir.removeBlock-301"><span class="linenos">301</span></a>                <span class="n">frame</span><span class="o">.</span><span class="n">grid_configure</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">i</span><span class="p">)</span>
</span></pre></div>


            <div class="docstring"><p>Remove a UI block from view and list. Automatically reindexes
and repositions remaining blocks.</p>
</div>


                            </div>
                            <div id="Mimir.addLabeledEntry" class="classattr">
                                        <input id="Mimir.addLabeledEntry-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">addLabeledEntry</span><span class="signature pdoc-code multiline">(<span class="param">	<span class="bp">self</span>,</span><span class="param">	<span class="n">parent</span>,</span><span class="param">	<span class="n">row</span>,</span><span class="param">	<span class="n">label</span>,</span><span class="param">	<span class="n">variable</span>,</span><span class="param">	<span class="n">state</span><span class="o">=</span><span class="s1">&#39;normal&#39;</span>,</span><span class="param">	<span class="n">tooltip</span><span class="o">=</span><span class="kc">None</span>,</span><span class="param">	<span class="n">validation</span><span class="o">=</span><span class="kc">None</span>,</span><span class="param">	<span class="n">column</span><span class="o">=</span><span class="mi">0</span>,</span><span class="param">	<span class="n">vertical</span><span class="o">=</span><span class="kc">False</span>,</span><span class="param">	<span class="n">columnspan</span><span class="o">=</span><span class="mi">1</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="Mimir.addLabeledEntry-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.addLabeledEntry"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.addLabeledEntry-304"><a href="#Mimir.addLabeledEntry-304"><span class="linenos">304</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">addLabeledEntry</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">parent</span><span class="p">,</span> <span class="n">row</span><span class="p">,</span> <span class="n">label</span><span class="p">,</span> <span class="n">variable</span><span class="p">,</span> <span class="n">state</span><span class="o">=</span><span class="s2">&quot;normal&quot;</span><span class="p">,</span> <span class="n">tooltip</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="n">validation</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">vertical</span><span class="o">=</span><span class="kc">False</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="mi">1</span><span class="p">):</span>
</span><span id="Mimir.addLabeledEntry-305"><a href="#Mimir.addLabeledEntry-305"><span class="linenos">305</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir.addLabeledEntry-306"><a href="#Mimir.addLabeledEntry-306"><span class="linenos">306</span></a><span class="sd">        Add a labeled entry input field. Supports vertical and horizontal layouts,</span>
</span><span id="Mimir.addLabeledEntry-307"><a href="#Mimir.addLabeledEntry-307"><span class="linenos">307</span></a><span class="sd">        optional tooltips and validation.</span>
</span><span id="Mimir.addLabeledEntry-308"><a href="#Mimir.addLabeledEntry-308"><span class="linenos">308</span></a><span class="sd">        &quot;&quot;&quot;</span>        
</span><span id="Mimir.addLabeledEntry-309"><a href="#Mimir.addLabeledEntry-309"><span class="linenos">309</span></a>        <span class="k">if</span> <span class="n">vertical</span><span class="p">:</span>
</span><span id="Mimir.addLabeledEntry-310"><a href="#Mimir.addLabeledEntry-310"><span class="linenos">310</span></a>            <span class="n">tb</span><span class="o">.</span><span class="n">Label</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">text</span><span class="o">=</span><span class="n">label</span><span class="p">)</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="n">columnspan</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="p">(</span><span class="mi">5</span><span class="p">,</span> <span class="mi">5</span><span class="p">))</span>
</span><span id="Mimir.addLabeledEntry-311"><a href="#Mimir.addLabeledEntry-311"><span class="linenos">311</span></a>            <span class="n">entry</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Entry</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">textvariable</span><span class="o">=</span><span class="n">variable</span><span class="p">,</span> <span class="n">state</span><span class="o">=</span><span class="n">state</span><span class="p">,</span> <span class="n">width</span><span class="o">=</span><span class="bp">self</span><span class="o">.</span><span class="n">widthEntry</span><span class="p">)</span>
</span><span id="Mimir.addLabeledEntry-312"><a href="#Mimir.addLabeledEntry-312"><span class="linenos">312</span></a>            <span class="n">entry</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span> <span class="o">+</span> <span class="mi">1</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="n">columnspan</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="p">(</span><span class="mi">0</span><span class="p">,</span> <span class="mi">10</span><span class="p">),</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;ew&quot;</span><span class="p">)</span>
</span><span id="Mimir.addLabeledEntry-313"><a href="#Mimir.addLabeledEntry-313"><span class="linenos">313</span></a>        <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir.addLabeledEntry-314"><a href="#Mimir.addLabeledEntry-314"><span class="linenos">314</span></a>            <span class="n">tb</span><span class="o">.</span><span class="n">Label</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">text</span><span class="o">=</span><span class="n">label</span><span class="p">)</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;e&quot;</span><span class="p">)</span>
</span><span id="Mimir.addLabeledEntry-315"><a href="#Mimir.addLabeledEntry-315"><span class="linenos">315</span></a>            <span class="n">entry</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Entry</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">textvariable</span><span class="o">=</span><span class="n">variable</span><span class="p">,</span> <span class="n">state</span><span class="o">=</span><span class="n">state</span><span class="p">,</span> <span class="n">width</span><span class="o">=</span><span class="bp">self</span><span class="o">.</span><span class="n">widthEntry</span><span class="p">)</span>
</span><span id="Mimir.addLabeledEntry-316"><a href="#Mimir.addLabeledEntry-316"><span class="linenos">316</span></a>            <span class="n">entry</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span> <span class="o">+</span> <span class="mi">1</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="n">columnspan</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;ew&quot;</span><span class="p">)</span>
</span><span id="Mimir.addLabeledEntry-317"><a href="#Mimir.addLabeledEntry-317"><span class="linenos">317</span></a>
</span><span id="Mimir.addLabeledEntry-318"><a href="#Mimir.addLabeledEntry-318"><span class="linenos">318</span></a>        <span class="k">if</span> <span class="n">tooltip</span><span class="p">:</span>
</span><span id="Mimir.addLabeledEntry-319"><a href="#Mimir.addLabeledEntry-319"><span class="linenos">319</span></a>            <span class="bp">self</span><span class="o">.</span><span class="n">addTooltip</span><span class="p">(</span><span class="n">entry</span><span class="p">,</span> <span class="n">tooltip</span><span class="p">)</span>
</span><span id="Mimir.addLabeledEntry-320"><a href="#Mimir.addLabeledEntry-320"><span class="linenos">320</span></a>
</span><span id="Mimir.addLabeledEntry-321"><a href="#Mimir.addLabeledEntry-321"><span class="linenos">321</span></a>        <span class="n">validationFunc</span> <span class="o">=</span> <span class="bp">self</span><span class="o">.</span><span class="n">resolveValidator</span><span class="p">(</span><span class="n">validation</span><span class="p">)</span> <span class="k">if</span> <span class="n">validation</span> <span class="k">else</span> <span class="kc">None</span>
</span><span id="Mimir.addLabeledEntry-322"><a href="#Mimir.addLabeledEntry-322"><span class="linenos">322</span></a>        <span class="k">if</span> <span class="nb">callable</span><span class="p">(</span><span class="n">validationFunc</span><span class="p">):</span>
</span><span id="Mimir.addLabeledEntry-323"><a href="#Mimir.addLabeledEntry-323"><span class="linenos">323</span></a>            <span class="k">def</span><span class="w"> </span><span class="nf">onFocusOut</span><span class="p">(</span><span class="n">event</span><span class="p">):</span>
</span><span id="Mimir.addLabeledEntry-324"><a href="#Mimir.addLabeledEntry-324"><span class="linenos">324</span></a>                <span class="n">value</span> <span class="o">=</span> <span class="n">entry</span><span class="o">.</span><span class="n">get</span><span class="p">()</span>
</span><span id="Mimir.addLabeledEntry-325"><a href="#Mimir.addLabeledEntry-325"><span class="linenos">325</span></a>                <span class="k">if</span> <span class="n">validationFunc</span><span class="p">(</span><span class="n">value</span><span class="p">):</span>
</span><span id="Mimir.addLabeledEntry-326"><a href="#Mimir.addLabeledEntry-326"><span class="linenos">326</span></a>                    <span class="bp">self</span><span class="o">.</span><span class="n">clearInvalid</span><span class="p">(</span><span class="n">entry</span><span class="p">)</span>
</span><span id="Mimir.addLabeledEntry-327"><a href="#Mimir.addLabeledEntry-327"><span class="linenos">327</span></a>                <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir.addLabeledEntry-328"><a href="#Mimir.addLabeledEntry-328"><span class="linenos">328</span></a>                    <span class="bp">self</span><span class="o">.</span><span class="n">markInvalid</span><span class="p">(</span><span class="n">entry</span><span class="p">)</span>
</span><span id="Mimir.addLabeledEntry-329"><a href="#Mimir.addLabeledEntry-329"><span class="linenos">329</span></a>            <span class="n">entry</span><span class="o">.</span><span class="n">bind</span><span class="p">(</span><span class="s2">&quot;&lt;FocusOut&gt;&quot;</span><span class="p">,</span> <span class="n">onFocusOut</span><span class="p">)</span>
</span><span id="Mimir.addLabeledEntry-330"><a href="#Mimir.addLabeledEntry-330"><span class="linenos">330</span></a>
</span><span id="Mimir.addLabeledEntry-331"><a href="#Mimir.addLabeledEntry-331"><span class="linenos">331</span></a>        <span class="k">return</span> <span class="n">entry</span>
</span></pre></div>


            <div class="docstring"><p>Add a labeled entry input field. Supports vertical and horizontal layouts,
optional tooltips and validation.</p>
</div>


                            </div>
                            <div id="Mimir.addLabeledCombobox" class="classattr">
                                        <input id="Mimir.addLabeledCombobox-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">addLabeledCombobox</span><span class="signature pdoc-code multiline">(<span class="param">	<span class="bp">self</span>,</span><span class="param">	<span class="n">parent</span>,</span><span class="param">	<span class="n">row</span>,</span><span class="param">	<span class="n">label</span>,</span><span class="param">	<span class="n">variable</span>,</span><span class="param">	<span class="n">values</span>,</span><span class="param">	<span class="n">tooltip</span><span class="o">=</span><span class="kc">None</span>,</span><span class="param">	<span class="n">state</span><span class="o">=</span><span class="s1">&#39;readonly&#39;</span>,</span><span class="param">	<span class="n">column</span><span class="o">=</span><span class="mi">0</span>,</span><span class="param">	<span class="n">vertical</span><span class="o">=</span><span class="kc">False</span>,</span><span class="param">	<span class="n">columnspan</span><span class="o">=</span><span class="mi">1</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="Mimir.addLabeledCombobox-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.addLabeledCombobox"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.addLabeledCombobox-333"><a href="#Mimir.addLabeledCombobox-333"><span class="linenos">333</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">addLabeledCombobox</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">parent</span><span class="p">,</span> <span class="n">row</span><span class="p">,</span> <span class="n">label</span><span class="p">,</span> <span class="n">variable</span><span class="p">,</span> <span class="n">values</span><span class="p">,</span> <span class="n">tooltip</span><span class="o">=</span><span class="kc">None</span><span class="p">,</span> <span class="n">state</span><span class="o">=</span><span class="s2">&quot;readonly&quot;</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="mi">0</span><span class="p">,</span> <span class="n">vertical</span><span class="o">=</span><span class="kc">False</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="mi">1</span><span class="p">):</span>
</span><span id="Mimir.addLabeledCombobox-334"><a href="#Mimir.addLabeledCombobox-334"><span class="linenos">334</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;Add a labeled combobox dropdown field with options.&quot;&quot;&quot;</span>
</span><span id="Mimir.addLabeledCombobox-335"><a href="#Mimir.addLabeledCombobox-335"><span class="linenos">335</span></a>        <span class="k">if</span> <span class="n">vertical</span><span class="p">:</span>
</span><span id="Mimir.addLabeledCombobox-336"><a href="#Mimir.addLabeledCombobox-336"><span class="linenos">336</span></a>            <span class="n">tb</span><span class="o">.</span><span class="n">Label</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">text</span><span class="o">=</span><span class="n">label</span><span class="p">)</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="n">columnspan</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="p">(</span><span class="mi">5</span><span class="p">,</span> <span class="mi">0</span><span class="p">))</span>
</span><span id="Mimir.addLabeledCombobox-337"><a href="#Mimir.addLabeledCombobox-337"><span class="linenos">337</span></a>            <span class="n">combo</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Combobox</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">textvariable</span><span class="o">=</span><span class="n">variable</span><span class="p">,</span> <span class="n">values</span><span class="o">=</span><span class="n">values</span><span class="p">,</span> <span class="n">state</span><span class="o">=</span><span class="n">state</span><span class="p">,</span> <span class="n">width</span><span class="o">=</span><span class="bp">self</span><span class="o">.</span><span class="n">widthCombo</span><span class="p">)</span>
</span><span id="Mimir.addLabeledCombobox-338"><a href="#Mimir.addLabeledCombobox-338"><span class="linenos">338</span></a>            <span class="n">combo</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span> <span class="o">+</span> <span class="mi">1</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="n">columnspan</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="p">(</span><span class="mi">0</span><span class="p">,</span> <span class="mi">10</span><span class="p">))</span>
</span><span id="Mimir.addLabeledCombobox-339"><a href="#Mimir.addLabeledCombobox-339"><span class="linenos">339</span></a>        <span class="k">else</span><span class="p">:</span>
</span><span id="Mimir.addLabeledCombobox-340"><a href="#Mimir.addLabeledCombobox-340"><span class="linenos">340</span></a>            <span class="n">tb</span><span class="o">.</span><span class="n">Label</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">text</span><span class="o">=</span><span class="n">label</span><span class="p">)</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;e&quot;</span><span class="p">)</span>
</span><span id="Mimir.addLabeledCombobox-341"><a href="#Mimir.addLabeledCombobox-341"><span class="linenos">341</span></a>            <span class="n">combo</span> <span class="o">=</span> <span class="n">tb</span><span class="o">.</span><span class="n">Combobox</span><span class="p">(</span><span class="n">parent</span><span class="p">,</span> <span class="n">textvariable</span><span class="o">=</span><span class="n">variable</span><span class="p">,</span> <span class="n">values</span><span class="o">=</span><span class="n">values</span><span class="p">,</span> <span class="n">state</span><span class="o">=</span><span class="n">state</span><span class="p">,</span> <span class="n">width</span><span class="o">=</span><span class="bp">self</span><span class="o">.</span><span class="n">widthCombo</span><span class="p">)</span>
</span><span id="Mimir.addLabeledCombobox-342"><a href="#Mimir.addLabeledCombobox-342"><span class="linenos">342</span></a>            <span class="n">combo</span><span class="o">.</span><span class="n">grid</span><span class="p">(</span><span class="n">row</span><span class="o">=</span><span class="n">row</span><span class="p">,</span> <span class="n">column</span><span class="o">=</span><span class="n">column</span> <span class="o">+</span> <span class="mi">1</span><span class="p">,</span> <span class="n">columnspan</span><span class="o">=</span><span class="n">columnspan</span><span class="p">,</span> <span class="n">padx</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">pady</span><span class="o">=</span><span class="mi">5</span><span class="p">,</span> <span class="n">sticky</span><span class="o">=</span><span class="s2">&quot;ew&quot;</span><span class="p">)</span>
</span><span id="Mimir.addLabeledCombobox-343"><a href="#Mimir.addLabeledCombobox-343"><span class="linenos">343</span></a>
</span><span id="Mimir.addLabeledCombobox-344"><a href="#Mimir.addLabeledCombobox-344"><span class="linenos">344</span></a>        <span class="k">if</span> <span class="n">tooltip</span><span class="p">:</span>
</span><span id="Mimir.addLabeledCombobox-345"><a href="#Mimir.addLabeledCombobox-345"><span class="linenos">345</span></a>            <span class="bp">self</span><span class="o">.</span><span class="n">addTooltip</span><span class="p">(</span><span class="n">combo</span><span class="p">,</span> <span class="n">tooltip</span><span class="p">)</span>
</span><span id="Mimir.addLabeledCombobox-346"><a href="#Mimir.addLabeledCombobox-346"><span class="linenos">346</span></a>
</span><span id="Mimir.addLabeledCombobox-347"><a href="#Mimir.addLabeledCombobox-347"><span class="linenos">347</span></a>        <span class="k">return</span> <span class="n">combo</span>
</span></pre></div>


            <div class="docstring"><p>Add a labeled combobox dropdown field with options.</p>
</div>


                            </div>
                            <div id="Mimir.getNextAvailableRow" class="classattr">
                                        <input id="Mimir.getNextAvailableRow-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">getNextAvailableRow</span><span class="signature pdoc-code condensed">(<span class="param"><span class="bp">self</span>, </span><span class="param"><span class="n">frame</span><span class="p">:</span> <span class="n">tkinter</span><span class="o">.</span><span class="n">Widget</span></span><span class="return-annotation">) -> <span class="nb">int</span>:</span></span>

                <label class="view-source-button" for="Mimir.getNextAvailableRow-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#Mimir.getNextAvailableRow"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="Mimir.getNextAvailableRow-349"><a href="#Mimir.getNextAvailableRow-349"><span class="linenos">349</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">getNextAvailableRow</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">frame</span><span class="p">:</span> <span class="n">tk</span><span class="o">.</span><span class="n">Widget</span><span class="p">)</span> <span class="o">-&gt;</span> <span class="nb">int</span><span class="p">:</span>
</span><span id="Mimir.getNextAvailableRow-350"><a href="#Mimir.getNextAvailableRow-350"><span class="linenos">350</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="Mimir.getNextAvailableRow-351"><a href="#Mimir.getNextAvailableRow-351"><span class="linenos">351</span></a><span class="sd">        Return the next available row in a container by checking occupied grid rows.</span>
</span><span id="Mimir.getNextAvailableRow-352"><a href="#Mimir.getNextAvailableRow-352"><span class="linenos">352</span></a><span class="sd">        &quot;&quot;&quot;</span>        
</span><span id="Mimir.getNextAvailableRow-353"><a href="#Mimir.getNextAvailableRow-353"><span class="linenos">353</span></a>        <span class="n">used_rows</span> <span class="o">=</span> <span class="p">[</span>
</span><span id="Mimir.getNextAvailableRow-354"><a href="#Mimir.getNextAvailableRow-354"><span class="linenos">354</span></a>            <span class="n">child</span><span class="o">.</span><span class="n">grid_info</span><span class="p">()</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;row&quot;</span><span class="p">)</span>
</span><span id="Mimir.getNextAvailableRow-355"><a href="#Mimir.getNextAvailableRow-355"><span class="linenos">355</span></a>            <span class="k">for</span> <span class="n">child</span> <span class="ow">in</span> <span class="n">frame</span><span class="o">.</span><span class="n">winfo_children</span><span class="p">()</span>
</span><span id="Mimir.getNextAvailableRow-356"><a href="#Mimir.getNextAvailableRow-356"><span class="linenos">356</span></a>            <span class="k">if</span> <span class="s2">&quot;row&quot;</span> <span class="ow">in</span> <span class="n">child</span><span class="o">.</span><span class="n">grid_info</span><span class="p">()</span>
</span><span id="Mimir.getNextAvailableRow-357"><a href="#Mimir.getNextAvailableRow-357"><span class="linenos">357</span></a>        <span class="p">]</span>
</span><span id="Mimir.getNextAvailableRow-358"><a href="#Mimir.getNextAvailableRow-358"><span class="linenos">358</span></a>        <span class="k">return</span> <span class="nb">max</span><span class="p">(</span><span class="n">used_rows</span><span class="p">,</span> <span class="n">default</span><span class="o">=-</span><span class="mi">1</span><span class="p">)</span> <span class="o">+</span> <span class="mi">1</span>
</span></pre></div>


            <div class="docstring"><p>Return the next available row in a container by checking occupied grid rows.</p>
</div>


                            </div>
                </section>
                <section id="DataBlockWrapper">
                            <input id="DataBlockWrapper-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr class">
            
    <span class="def">class</span>
    <span class="name">DataBlockWrapper</span>:

                <label class="view-source-button" for="DataBlockWrapper-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#DataBlockWrapper"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="DataBlockWrapper-6"><a href="#DataBlockWrapper-6"><span class="linenos"> 6</span></a><span class="k">class</span><span class="w"> </span><span class="nc">DataBlockWrapper</span><span class="p">:</span>
</span><span id="DataBlockWrapper-7"><a href="#DataBlockWrapper-7"><span class="linenos"> 7</span></a>    <span class="k">def</span><span class="w"> </span><span class="fm">__init__</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">meta</span><span class="p">):</span>
</span><span id="DataBlockWrapper-8"><a href="#DataBlockWrapper-8"><span class="linenos"> 8</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span> <span class="o">=</span> <span class="n">meta</span>
</span><span id="DataBlockWrapper-9"><a href="#DataBlockWrapper-9"><span class="linenos"> 9</span></a>
</span><span id="DataBlockWrapper-10"><a href="#DataBlockWrapper-10"><span class="linenos">10</span></a>    <span class="nd">@property</span>
</span><span id="DataBlockWrapper-11"><a href="#DataBlockWrapper-11"><span class="linenos">11</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">meta</span><span class="p">(</span><span class="bp">self</span><span class="p">):</span>
</span><span id="DataBlockWrapper-12"><a href="#DataBlockWrapper-12"><span class="linenos">12</span></a>        <span class="k">return</span> <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span>
</span><span id="DataBlockWrapper-13"><a href="#DataBlockWrapper-13"><span class="linenos">13</span></a>
</span><span id="DataBlockWrapper-14"><a href="#DataBlockWrapper-14"><span class="linenos">14</span></a>    <span class="k">def</span><span class="w"> </span><span class="fm">__getattr__</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">key</span><span class="p">):</span>
</span><span id="DataBlockWrapper-15"><a href="#DataBlockWrapper-15"><span class="linenos">15</span></a>        <span class="n">fields</span> <span class="o">=</span> <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;fields_by_key&quot;</span><span class="p">,</span> <span class="p">{})</span>
</span><span id="DataBlockWrapper-16"><a href="#DataBlockWrapper-16"><span class="linenos">16</span></a>        <span class="k">if</span> <span class="n">key</span> <span class="ow">in</span> <span class="n">fields</span><span class="p">:</span>
</span><span id="DataBlockWrapper-17"><a href="#DataBlockWrapper-17"><span class="linenos">17</span></a>            <span class="k">return</span> <span class="n">fields</span><span class="p">[</span><span class="n">key</span><span class="p">]</span>
</span><span id="DataBlockWrapper-18"><a href="#DataBlockWrapper-18"><span class="linenos">18</span></a>        <span class="k">raise</span> <span class="ne">AttributeError</span><span class="p">(</span><span class="sa">f</span><span class="s2">&quot;&#39;</span><span class="si">{</span><span class="nb">type</span><span class="p">(</span><span class="bp">self</span><span class="p">)</span><span class="o">.</span><span class="vm">__name__</span><span class="si">}</span><span class="s2">&#39; object has no attribute &#39;</span><span class="si">{</span><span class="n">key</span><span class="si">}</span><span class="s2">&#39;&quot;</span><span class="p">)</span>
</span><span id="DataBlockWrapper-19"><a href="#DataBlockWrapper-19"><span class="linenos">19</span></a>
</span><span id="DataBlockWrapper-20"><a href="#DataBlockWrapper-20"><span class="linenos">20</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">get</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">key</span><span class="p">):</span>
</span><span id="DataBlockWrapper-21"><a href="#DataBlockWrapper-21"><span class="linenos">21</span></a>        <span class="k">return</span> <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;fields_by_key&quot;</span><span class="p">,</span> <span class="p">{})</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="n">key</span><span class="p">)</span>
</span><span id="DataBlockWrapper-22"><a href="#DataBlockWrapper-22"><span class="linenos">22</span></a>
</span><span id="DataBlockWrapper-23"><a href="#DataBlockWrapper-23"><span class="linenos">23</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">set</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">key</span><span class="p">,</span> <span class="n">value</span><span class="p">):</span>
</span><span id="DataBlockWrapper-24"><a href="#DataBlockWrapper-24"><span class="linenos">24</span></a>        <span class="k">try</span><span class="p">:</span>
</span><span id="DataBlockWrapper-25"><a href="#DataBlockWrapper-25"><span class="linenos">25</span></a>            <span class="n">variable</span> <span class="o">=</span> <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span><span class="p">[</span><span class="s2">&quot;fields_by_key&quot;</span><span class="p">][</span><span class="n">key</span><span class="p">]</span>
</span><span id="DataBlockWrapper-26"><a href="#DataBlockWrapper-26"><span class="linenos">26</span></a>            <span class="k">if</span> <span class="nb">hasattr</span><span class="p">(</span><span class="n">variable</span><span class="p">,</span> <span class="s2">&quot;set&quot;</span><span class="p">):</span>
</span><span id="DataBlockWrapper-27"><a href="#DataBlockWrapper-27"><span class="linenos">27</span></a>                <span class="n">variable</span><span class="o">.</span><span class="n">set</span><span class="p">(</span><span class="n">value</span><span class="p">)</span>
</span><span id="DataBlockWrapper-28"><a href="#DataBlockWrapper-28"><span class="linenos">28</span></a>            <span class="k">else</span><span class="p">:</span>
</span><span id="DataBlockWrapper-29"><a href="#DataBlockWrapper-29"><span class="linenos">29</span></a>                <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span><span class="p">[</span><span class="s2">&quot;fields_by_key&quot;</span><span class="p">][</span><span class="n">key</span><span class="p">]</span> <span class="o">=</span> <span class="n">value</span>
</span><span id="DataBlockWrapper-30"><a href="#DataBlockWrapper-30"><span class="linenos">30</span></a>        <span class="k">except</span> <span class="ne">KeyError</span><span class="p">:</span>
</span><span id="DataBlockWrapper-31"><a href="#DataBlockWrapper-31"><span class="linenos">31</span></a>            <span class="k">raise</span> <span class="ne">KeyError</span><span class="p">(</span><span class="sa">f</span><span class="s2">&quot;Key &#39;</span><span class="si">{</span><span class="n">key</span><span class="si">}</span><span class="s2">&#39; not found in DataBlockWrapper.&quot;</span><span class="p">)</span>
</span><span id="DataBlockWrapper-32"><a href="#DataBlockWrapper-32"><span class="linenos">32</span></a>
</span><span id="DataBlockWrapper-33"><a href="#DataBlockWrapper-33"><span class="linenos">33</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">items</span><span class="p">(</span><span class="bp">self</span><span class="p">):</span>
</span><span id="DataBlockWrapper-34"><a href="#DataBlockWrapper-34"><span class="linenos">34</span></a>        <span class="k">return</span> <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;fields_by_key&quot;</span><span class="p">,</span> <span class="p">{})</span><span class="o">.</span><span class="n">items</span><span class="p">()</span>
</span><span id="DataBlockWrapper-35"><a href="#DataBlockWrapper-35"><span class="linenos">35</span></a>
</span><span id="DataBlockWrapper-36"><a href="#DataBlockWrapper-36"><span class="linenos">36</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">values</span><span class="p">(</span><span class="bp">self</span><span class="p">):</span>
</span><span id="DataBlockWrapper-37"><a href="#DataBlockWrapper-37"><span class="linenos">37</span></a>        <span class="k">return</span> <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;fields_by_key&quot;</span><span class="p">,</span> <span class="p">{})</span><span class="o">.</span><span class="n">values</span><span class="p">()</span>
</span><span id="DataBlockWrapper-38"><a href="#DataBlockWrapper-38"><span class="linenos">38</span></a>
</span><span id="DataBlockWrapper-39"><a href="#DataBlockWrapper-39"><span class="linenos">39</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">keys</span><span class="p">(</span><span class="bp">self</span><span class="p">):</span>
</span><span id="DataBlockWrapper-40"><a href="#DataBlockWrapper-40"><span class="linenos">40</span></a>        <span class="k">return</span> <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;fields_by_key&quot;</span><span class="p">,</span> <span class="p">{})</span><span class="o">.</span><span class="n">keys</span><span class="p">()</span>
</span><span id="DataBlockWrapper-41"><a href="#DataBlockWrapper-41"><span class="linenos">41</span></a>
</span><span id="DataBlockWrapper-42"><a href="#DataBlockWrapper-42"><span class="linenos">42</span></a>    <span class="k">def</span><span class="w"> </span><span class="fm">__dir__</span><span class="p">(</span><span class="bp">self</span><span class="p">):</span>
</span><span id="DataBlockWrapper-43"><a href="#DataBlockWrapper-43"><span class="linenos">43</span></a>        <span class="k">return</span> <span class="nb">super</span><span class="p">()</span><span class="o">.</span><span class="fm">__dir__</span><span class="p">()</span> <span class="o">+</span> <span class="nb">list</span><span class="p">(</span><span class="bp">self</span><span class="o">.</span><span class="n">_meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;fields_by_key&quot;</span><span class="p">,</span> <span class="p">{})</span><span class="o">.</span><span class="n">keys</span><span class="p">())</span>
</span></pre></div>


    

                            <div id="DataBlockWrapper.__init__" class="classattr">
                                        <input id="DataBlockWrapper.__init__-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="name">DataBlockWrapper</span><span class="signature pdoc-code condensed">(<span class="param"><span class="n">meta</span></span>)</span>

                <label class="view-source-button" for="DataBlockWrapper.__init__-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#DataBlockWrapper.__init__"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="DataBlockWrapper.__init__-7"><a href="#DataBlockWrapper.__init__-7"><span class="linenos">7</span></a>    <span class="k">def</span><span class="w"> </span><span class="fm">__init__</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">meta</span><span class="p">):</span>
</span><span id="DataBlockWrapper.__init__-8"><a href="#DataBlockWrapper.__init__-8"><span class="linenos">8</span></a>        <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span> <span class="o">=</span> <span class="n">meta</span>
</span></pre></div>


    

                            </div>
                            <div id="DataBlockWrapper.meta" class="classattr">
                                        <input id="DataBlockWrapper.meta-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr variable">
            <span class="name">meta</span>

                <label class="view-source-button" for="DataBlockWrapper.meta-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#DataBlockWrapper.meta"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="DataBlockWrapper.meta-10"><a href="#DataBlockWrapper.meta-10"><span class="linenos">10</span></a>    <span class="nd">@property</span>
</span><span id="DataBlockWrapper.meta-11"><a href="#DataBlockWrapper.meta-11"><span class="linenos">11</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">meta</span><span class="p">(</span><span class="bp">self</span><span class="p">):</span>
</span><span id="DataBlockWrapper.meta-12"><a href="#DataBlockWrapper.meta-12"><span class="linenos">12</span></a>        <span class="k">return</span> <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span>
</span></pre></div>


    

                            </div>
                            <div id="DataBlockWrapper.get" class="classattr">
                                        <input id="DataBlockWrapper.get-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">get</span><span class="signature pdoc-code condensed">(<span class="param"><span class="bp">self</span>, </span><span class="param"><span class="n">key</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="DataBlockWrapper.get-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#DataBlockWrapper.get"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="DataBlockWrapper.get-20"><a href="#DataBlockWrapper.get-20"><span class="linenos">20</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">get</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">key</span><span class="p">):</span>
</span><span id="DataBlockWrapper.get-21"><a href="#DataBlockWrapper.get-21"><span class="linenos">21</span></a>        <span class="k">return</span> <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;fields_by_key&quot;</span><span class="p">,</span> <span class="p">{})</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="n">key</span><span class="p">)</span>
</span></pre></div>


    

                            </div>
                            <div id="DataBlockWrapper.set" class="classattr">
                                        <input id="DataBlockWrapper.set-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">set</span><span class="signature pdoc-code condensed">(<span class="param"><span class="bp">self</span>, </span><span class="param"><span class="n">key</span>, </span><span class="param"><span class="n">value</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="DataBlockWrapper.set-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#DataBlockWrapper.set"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="DataBlockWrapper.set-23"><a href="#DataBlockWrapper.set-23"><span class="linenos">23</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">set</span><span class="p">(</span><span class="bp">self</span><span class="p">,</span> <span class="n">key</span><span class="p">,</span> <span class="n">value</span><span class="p">):</span>
</span><span id="DataBlockWrapper.set-24"><a href="#DataBlockWrapper.set-24"><span class="linenos">24</span></a>        <span class="k">try</span><span class="p">:</span>
</span><span id="DataBlockWrapper.set-25"><a href="#DataBlockWrapper.set-25"><span class="linenos">25</span></a>            <span class="n">variable</span> <span class="o">=</span> <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span><span class="p">[</span><span class="s2">&quot;fields_by_key&quot;</span><span class="p">][</span><span class="n">key</span><span class="p">]</span>
</span><span id="DataBlockWrapper.set-26"><a href="#DataBlockWrapper.set-26"><span class="linenos">26</span></a>            <span class="k">if</span> <span class="nb">hasattr</span><span class="p">(</span><span class="n">variable</span><span class="p">,</span> <span class="s2">&quot;set&quot;</span><span class="p">):</span>
</span><span id="DataBlockWrapper.set-27"><a href="#DataBlockWrapper.set-27"><span class="linenos">27</span></a>                <span class="n">variable</span><span class="o">.</span><span class="n">set</span><span class="p">(</span><span class="n">value</span><span class="p">)</span>
</span><span id="DataBlockWrapper.set-28"><a href="#DataBlockWrapper.set-28"><span class="linenos">28</span></a>            <span class="k">else</span><span class="p">:</span>
</span><span id="DataBlockWrapper.set-29"><a href="#DataBlockWrapper.set-29"><span class="linenos">29</span></a>                <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span><span class="p">[</span><span class="s2">&quot;fields_by_key&quot;</span><span class="p">][</span><span class="n">key</span><span class="p">]</span> <span class="o">=</span> <span class="n">value</span>
</span><span id="DataBlockWrapper.set-30"><a href="#DataBlockWrapper.set-30"><span class="linenos">30</span></a>        <span class="k">except</span> <span class="ne">KeyError</span><span class="p">:</span>
</span><span id="DataBlockWrapper.set-31"><a href="#DataBlockWrapper.set-31"><span class="linenos">31</span></a>            <span class="k">raise</span> <span class="ne">KeyError</span><span class="p">(</span><span class="sa">f</span><span class="s2">&quot;Key &#39;</span><span class="si">{</span><span class="n">key</span><span class="si">}</span><span class="s2">&#39; not found in DataBlockWrapper.&quot;</span><span class="p">)</span>
</span></pre></div>


    

                            </div>
                            <div id="DataBlockWrapper.items" class="classattr">
                                        <input id="DataBlockWrapper.items-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">items</span><span class="signature pdoc-code condensed">(<span class="param"><span class="bp">self</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="DataBlockWrapper.items-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#DataBlockWrapper.items"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="DataBlockWrapper.items-33"><a href="#DataBlockWrapper.items-33"><span class="linenos">33</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">items</span><span class="p">(</span><span class="bp">self</span><span class="p">):</span>
</span><span id="DataBlockWrapper.items-34"><a href="#DataBlockWrapper.items-34"><span class="linenos">34</span></a>        <span class="k">return</span> <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;fields_by_key&quot;</span><span class="p">,</span> <span class="p">{})</span><span class="o">.</span><span class="n">items</span><span class="p">()</span>
</span></pre></div>


    

                            </div>
                            <div id="DataBlockWrapper.values" class="classattr">
                                        <input id="DataBlockWrapper.values-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">values</span><span class="signature pdoc-code condensed">(<span class="param"><span class="bp">self</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="DataBlockWrapper.values-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#DataBlockWrapper.values"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="DataBlockWrapper.values-36"><a href="#DataBlockWrapper.values-36"><span class="linenos">36</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">values</span><span class="p">(</span><span class="bp">self</span><span class="p">):</span>
</span><span id="DataBlockWrapper.values-37"><a href="#DataBlockWrapper.values-37"><span class="linenos">37</span></a>        <span class="k">return</span> <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;fields_by_key&quot;</span><span class="p">,</span> <span class="p">{})</span><span class="o">.</span><span class="n">values</span><span class="p">()</span>
</span></pre></div>


    

                            </div>
                            <div id="DataBlockWrapper.keys" class="classattr">
                                        <input id="DataBlockWrapper.keys-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
            
        <span class="def">def</span>
        <span class="name">keys</span><span class="signature pdoc-code condensed">(<span class="param"><span class="bp">self</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="DataBlockWrapper.keys-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#DataBlockWrapper.keys"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="DataBlockWrapper.keys-39"><a href="#DataBlockWrapper.keys-39"><span class="linenos">39</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">keys</span><span class="p">(</span><span class="bp">self</span><span class="p">):</span>
</span><span id="DataBlockWrapper.keys-40"><a href="#DataBlockWrapper.keys-40"><span class="linenos">40</span></a>        <span class="k">return</span> <span class="bp">self</span><span class="o">.</span><span class="n">_meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;fields_by_key&quot;</span><span class="p">,</span> <span class="p">{})</span><span class="o">.</span><span class="n">keys</span><span class="p">()</span>
</span></pre></div>


    

                            </div>
                </section>
                <section id="UtilityModule">
                            <input id="UtilityModule-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr class">
            
    <span class="def">class</span>
    <span class="name">UtilityModule</span>:

                <label class="view-source-button" for="UtilityModule-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#UtilityModule"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="UtilityModule-6"><a href="#UtilityModule-6"><span class="linenos"> 6</span></a><span class="k">class</span><span class="w"> </span><span class="nc">UtilityModule</span><span class="p">:</span>
</span><span id="UtilityModule-7"><a href="#UtilityModule-7"><span class="linenos"> 7</span></a>    <span class="nd">@staticmethod</span>
</span><span id="UtilityModule-8"><a href="#UtilityModule-8"><span class="linenos"> 8</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">buildBlockMeta</span><span class="p">(</span><span class="n">fields</span><span class="p">,</span> <span class="o">**</span><span class="n">extra</span><span class="p">):</span>
</span><span id="UtilityModule-9"><a href="#UtilityModule-9"><span class="linenos"> 9</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="UtilityModule-10"><a href="#UtilityModule-10"><span class="linenos">10</span></a><span class="sd">        Given a list of field definitions, return a structured metadata dictionary</span>
</span><span id="UtilityModule-11"><a href="#UtilityModule-11"><span class="linenos">11</span></a><span class="sd">        ready to be passed to DataBlockWrapper.</span>
</span><span id="UtilityModule-12"><a href="#UtilityModule-12"><span class="linenos">12</span></a>
</span><span id="UtilityModule-13"><a href="#UtilityModule-13"><span class="linenos">13</span></a><span class="sd">        Fields should be a list of dicts containing at least &#39;variable&#39; and optionally &#39;key&#39;.</span>
</span><span id="UtilityModule-14"><a href="#UtilityModule-14"><span class="linenos">14</span></a><span class="sd">        Additional metadata (e.g. label, index) can be passed via kwargs.</span>
</span><span id="UtilityModule-15"><a href="#UtilityModule-15"><span class="linenos">15</span></a><span class="sd">        &quot;&quot;&quot;</span>
</span><span id="UtilityModule-16"><a href="#UtilityModule-16"><span class="linenos">16</span></a>        <span class="k">return</span> <span class="p">{</span>
</span><span id="UtilityModule-17"><a href="#UtilityModule-17"><span class="linenos">17</span></a>            <span class="o">**</span><span class="n">extra</span><span class="p">,</span>
</span><span id="UtilityModule-18"><a href="#UtilityModule-18"><span class="linenos">18</span></a>            <span class="s2">&quot;fields&quot;</span><span class="p">:</span> <span class="n">fields</span><span class="p">,</span>
</span><span id="UtilityModule-19"><a href="#UtilityModule-19"><span class="linenos">19</span></a>            <span class="s2">&quot;fields_by_key&quot;</span><span class="p">:</span> <span class="p">{</span><span class="n">f</span><span class="p">[</span><span class="s2">&quot;key&quot;</span><span class="p">]:</span> <span class="n">f</span><span class="p">[</span><span class="s2">&quot;variable&quot;</span><span class="p">]</span> <span class="k">for</span> <span class="n">f</span> <span class="ow">in</span> <span class="n">fields</span> <span class="k">if</span> <span class="s2">&quot;key&quot;</span> <span class="ow">in</span> <span class="n">f</span><span class="p">},</span>
</span><span id="UtilityModule-20"><a href="#UtilityModule-20"><span class="linenos">20</span></a>        <span class="p">}</span>
</span><span id="UtilityModule-21"><a href="#UtilityModule-21"><span class="linenos">21</span></a>
</span><span id="UtilityModule-22"><a href="#UtilityModule-22"><span class="linenos">22</span></a>    <span class="nd">@staticmethod</span>
</span><span id="UtilityModule-23"><a href="#UtilityModule-23"><span class="linenos">23</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">getBlockValues</span><span class="p">(</span><span class="n">wrapper</span><span class="p">,</span> <span class="n">raw</span><span class="o">=</span><span class="kc">False</span><span class="p">):</span>
</span><span id="UtilityModule-24"><a href="#UtilityModule-24"><span class="linenos">24</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="UtilityModule-25"><a href="#UtilityModule-25"><span class="linenos">25</span></a><span class="sd">        Extract a dictionary of current values from a DataBlockWrapper.</span>
</span><span id="UtilityModule-26"><a href="#UtilityModule-26"><span class="linenos">26</span></a>
</span><span id="UtilityModule-27"><a href="#UtilityModule-27"><span class="linenos">27</span></a><span class="sd">        If raw=True, return variable objects. Otherwise, return .get() results.</span>
</span><span id="UtilityModule-28"><a href="#UtilityModule-28"><span class="linenos">28</span></a><span class="sd">        &quot;&quot;&quot;</span>
</span><span id="UtilityModule-29"><a href="#UtilityModule-29"><span class="linenos">29</span></a>        <span class="n">output</span> <span class="o">=</span> <span class="p">{}</span>
</span><span id="UtilityModule-30"><a href="#UtilityModule-30"><span class="linenos">30</span></a>        <span class="k">for</span> <span class="n">key</span><span class="p">,</span> <span class="n">var</span> <span class="ow">in</span> <span class="n">wrapper</span><span class="o">.</span><span class="n">items</span><span class="p">():</span>
</span><span id="UtilityModule-31"><a href="#UtilityModule-31"><span class="linenos">31</span></a>            <span class="k">if</span> <span class="nb">hasattr</span><span class="p">(</span><span class="n">var</span><span class="p">,</span> <span class="s2">&quot;get&quot;</span><span class="p">)</span> <span class="ow">and</span> <span class="ow">not</span> <span class="n">raw</span><span class="p">:</span>
</span><span id="UtilityModule-32"><a href="#UtilityModule-32"><span class="linenos">32</span></a>                <span class="n">output</span><span class="p">[</span><span class="n">key</span><span class="p">]</span> <span class="o">=</span> <span class="n">var</span><span class="o">.</span><span class="n">get</span><span class="p">()</span>
</span><span id="UtilityModule-33"><a href="#UtilityModule-33"><span class="linenos">33</span></a>            <span class="k">else</span><span class="p">:</span>
</span><span id="UtilityModule-34"><a href="#UtilityModule-34"><span class="linenos">34</span></a>                <span class="n">output</span><span class="p">[</span><span class="n">key</span><span class="p">]</span> <span class="o">=</span> <span class="n">var</span>
</span><span id="UtilityModule-35"><a href="#UtilityModule-35"><span class="linenos">35</span></a>        <span class="k">return</span> <span class="n">output</span>
</span><span id="UtilityModule-36"><a href="#UtilityModule-36"><span class="linenos">36</span></a>
</span><span id="UtilityModule-37"><a href="#UtilityModule-37"><span class="linenos">37</span></a>    <span class="nd">@staticmethod</span>
</span><span id="UtilityModule-38"><a href="#UtilityModule-38"><span class="linenos">38</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">updateBlockFromDict</span><span class="p">(</span><span class="n">wrapper</span><span class="p">,</span> <span class="n">data</span><span class="p">):</span>
</span><span id="UtilityModule-39"><a href="#UtilityModule-39"><span class="linenos">39</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="UtilityModule-40"><a href="#UtilityModule-40"><span class="linenos">40</span></a><span class="sd">        Given a dict of values, update the DataBlockWrapper by setting matching keys.</span>
</span><span id="UtilityModule-41"><a href="#UtilityModule-41"><span class="linenos">41</span></a><span class="sd">        &quot;&quot;&quot;</span>
</span><span id="UtilityModule-42"><a href="#UtilityModule-42"><span class="linenos">42</span></a>        <span class="k">for</span> <span class="n">key</span><span class="p">,</span> <span class="n">value</span> <span class="ow">in</span> <span class="n">data</span><span class="o">.</span><span class="n">items</span><span class="p">():</span>
</span><span id="UtilityModule-43"><a href="#UtilityModule-43"><span class="linenos">43</span></a>            <span class="k">try</span><span class="p">:</span>
</span><span id="UtilityModule-44"><a href="#UtilityModule-44"><span class="linenos">44</span></a>                <span class="n">wrapper</span><span class="o">.</span><span class="n">set</span><span class="p">(</span><span class="n">key</span><span class="p">,</span> <span class="n">value</span><span class="p">)</span>
</span><span id="UtilityModule-45"><a href="#UtilityModule-45"><span class="linenos">45</span></a>            <span class="k">except</span> <span class="ne">KeyError</span><span class="p">:</span>
</span><span id="UtilityModule-46"><a href="#UtilityModule-46"><span class="linenos">46</span></a>                <span class="k">continue</span>
</span><span id="UtilityModule-47"><a href="#UtilityModule-47"><span class="linenos">47</span></a>
</span><span id="UtilityModule-48"><a href="#UtilityModule-48"><span class="linenos">48</span></a>    <span class="nd">@staticmethod</span>
</span><span id="UtilityModule-49"><a href="#UtilityModule-49"><span class="linenos">49</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">isBlockValid</span><span class="p">(</span><span class="n">wrapper</span><span class="p">,</span> <span class="n">validatorResolver</span><span class="p">):</span>
</span><span id="UtilityModule-50"><a href="#UtilityModule-50"><span class="linenos">50</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="UtilityModule-51"><a href="#UtilityModule-51"><span class="linenos">51</span></a><span class="sd">        Check if all fields in a block pass their validator.</span>
</span><span id="UtilityModule-52"><a href="#UtilityModule-52"><span class="linenos">52</span></a><span class="sd">        The validatorResolver must take a string name and return a function.</span>
</span><span id="UtilityModule-53"><a href="#UtilityModule-53"><span class="linenos">53</span></a><span class="sd">        &quot;&quot;&quot;</span>
</span><span id="UtilityModule-54"><a href="#UtilityModule-54"><span class="linenos">54</span></a>        <span class="k">for</span> <span class="n">f</span> <span class="ow">in</span> <span class="n">wrapper</span><span class="o">.</span><span class="n">meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;fields&quot;</span><span class="p">,</span> <span class="p">[]):</span>
</span><span id="UtilityModule-55"><a href="#UtilityModule-55"><span class="linenos">55</span></a>            <span class="n">name</span> <span class="o">=</span> <span class="n">f</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;validation&quot;</span><span class="p">)</span>
</span><span id="UtilityModule-56"><a href="#UtilityModule-56"><span class="linenos">56</span></a>            <span class="n">var</span> <span class="o">=</span> <span class="n">f</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;variable&quot;</span><span class="p">)</span>
</span><span id="UtilityModule-57"><a href="#UtilityModule-57"><span class="linenos">57</span></a>            <span class="k">if</span> <span class="n">name</span> <span class="ow">and</span> <span class="n">var</span> <span class="ow">and</span> <span class="nb">callable</span><span class="p">(</span><span class="n">validatorResolver</span><span class="p">(</span><span class="n">name</span><span class="p">)):</span>
</span><span id="UtilityModule-58"><a href="#UtilityModule-58"><span class="linenos">58</span></a>                <span class="n">valid</span> <span class="o">=</span> <span class="n">validatorResolver</span><span class="p">(</span><span class="n">name</span><span class="p">)(</span><span class="n">var</span><span class="o">.</span><span class="n">get</span><span class="p">())</span>
</span><span id="UtilityModule-59"><a href="#UtilityModule-59"><span class="linenos">59</span></a>                <span class="k">if</span> <span class="ow">not</span> <span class="n">valid</span><span class="p">:</span>
</span><span id="UtilityModule-60"><a href="#UtilityModule-60"><span class="linenos">60</span></a>                    <span class="k">return</span> <span class="kc">False</span>
</span><span id="UtilityModule-61"><a href="#UtilityModule-61"><span class="linenos">61</span></a>        <span class="k">return</span> <span class="kc">True</span>
</span></pre></div>


    

                            <div id="UtilityModule.buildBlockMeta" class="classattr">
                                        <input id="UtilityModule.buildBlockMeta-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
                    <div class="decorator decorator-staticmethod">@staticmethod</div>

        <span class="def">def</span>
        <span class="name">buildBlockMeta</span><span class="signature pdoc-code condensed">(<span class="param"><span class="n">fields</span>, </span><span class="param"><span class="o">**</span><span class="n">extra</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="UtilityModule.buildBlockMeta-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#UtilityModule.buildBlockMeta"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="UtilityModule.buildBlockMeta-7"><a href="#UtilityModule.buildBlockMeta-7"><span class="linenos"> 7</span></a>    <span class="nd">@staticmethod</span>
</span><span id="UtilityModule.buildBlockMeta-8"><a href="#UtilityModule.buildBlockMeta-8"><span class="linenos"> 8</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">buildBlockMeta</span><span class="p">(</span><span class="n">fields</span><span class="p">,</span> <span class="o">**</span><span class="n">extra</span><span class="p">):</span>
</span><span id="UtilityModule.buildBlockMeta-9"><a href="#UtilityModule.buildBlockMeta-9"><span class="linenos"> 9</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="UtilityModule.buildBlockMeta-10"><a href="#UtilityModule.buildBlockMeta-10"><span class="linenos">10</span></a><span class="sd">        Given a list of field definitions, return a structured metadata dictionary</span>
</span><span id="UtilityModule.buildBlockMeta-11"><a href="#UtilityModule.buildBlockMeta-11"><span class="linenos">11</span></a><span class="sd">        ready to be passed to DataBlockWrapper.</span>
</span><span id="UtilityModule.buildBlockMeta-12"><a href="#UtilityModule.buildBlockMeta-12"><span class="linenos">12</span></a>
</span><span id="UtilityModule.buildBlockMeta-13"><a href="#UtilityModule.buildBlockMeta-13"><span class="linenos">13</span></a><span class="sd">        Fields should be a list of dicts containing at least &#39;variable&#39; and optionally &#39;key&#39;.</span>
</span><span id="UtilityModule.buildBlockMeta-14"><a href="#UtilityModule.buildBlockMeta-14"><span class="linenos">14</span></a><span class="sd">        Additional metadata (e.g. label, index) can be passed via kwargs.</span>
</span><span id="UtilityModule.buildBlockMeta-15"><a href="#UtilityModule.buildBlockMeta-15"><span class="linenos">15</span></a><span class="sd">        &quot;&quot;&quot;</span>
</span><span id="UtilityModule.buildBlockMeta-16"><a href="#UtilityModule.buildBlockMeta-16"><span class="linenos">16</span></a>        <span class="k">return</span> <span class="p">{</span>
</span><span id="UtilityModule.buildBlockMeta-17"><a href="#UtilityModule.buildBlockMeta-17"><span class="linenos">17</span></a>            <span class="o">**</span><span class="n">extra</span><span class="p">,</span>
</span><span id="UtilityModule.buildBlockMeta-18"><a href="#UtilityModule.buildBlockMeta-18"><span class="linenos">18</span></a>            <span class="s2">&quot;fields&quot;</span><span class="p">:</span> <span class="n">fields</span><span class="p">,</span>
</span><span id="UtilityModule.buildBlockMeta-19"><a href="#UtilityModule.buildBlockMeta-19"><span class="linenos">19</span></a>            <span class="s2">&quot;fields_by_key&quot;</span><span class="p">:</span> <span class="p">{</span><span class="n">f</span><span class="p">[</span><span class="s2">&quot;key&quot;</span><span class="p">]:</span> <span class="n">f</span><span class="p">[</span><span class="s2">&quot;variable&quot;</span><span class="p">]</span> <span class="k">for</span> <span class="n">f</span> <span class="ow">in</span> <span class="n">fields</span> <span class="k">if</span> <span class="s2">&quot;key&quot;</span> <span class="ow">in</span> <span class="n">f</span><span class="p">},</span>
</span><span id="UtilityModule.buildBlockMeta-20"><a href="#UtilityModule.buildBlockMeta-20"><span class="linenos">20</span></a>        <span class="p">}</span>
</span></pre></div>


            <div class="docstring"><p>Given a list of field definitions, return a structured metadata dictionary
ready to be passed to DataBlockWrapper.</p>

<p>Fields should be a list of dicts containing at least 'variable' and optionally 'key'.
Additional metadata (e.g. label, index) can be passed via kwargs.</p>
</div>


                            </div>
                            <div id="UtilityModule.getBlockValues" class="classattr">
                                        <input id="UtilityModule.getBlockValues-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
                    <div class="decorator decorator-staticmethod">@staticmethod</div>

        <span class="def">def</span>
        <span class="name">getBlockValues</span><span class="signature pdoc-code condensed">(<span class="param"><span class="n">wrapper</span>, </span><span class="param"><span class="n">raw</span><span class="o">=</span><span class="kc">False</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="UtilityModule.getBlockValues-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#UtilityModule.getBlockValues"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="UtilityModule.getBlockValues-22"><a href="#UtilityModule.getBlockValues-22"><span class="linenos">22</span></a>    <span class="nd">@staticmethod</span>
</span><span id="UtilityModule.getBlockValues-23"><a href="#UtilityModule.getBlockValues-23"><span class="linenos">23</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">getBlockValues</span><span class="p">(</span><span class="n">wrapper</span><span class="p">,</span> <span class="n">raw</span><span class="o">=</span><span class="kc">False</span><span class="p">):</span>
</span><span id="UtilityModule.getBlockValues-24"><a href="#UtilityModule.getBlockValues-24"><span class="linenos">24</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="UtilityModule.getBlockValues-25"><a href="#UtilityModule.getBlockValues-25"><span class="linenos">25</span></a><span class="sd">        Extract a dictionary of current values from a DataBlockWrapper.</span>
</span><span id="UtilityModule.getBlockValues-26"><a href="#UtilityModule.getBlockValues-26"><span class="linenos">26</span></a>
</span><span id="UtilityModule.getBlockValues-27"><a href="#UtilityModule.getBlockValues-27"><span class="linenos">27</span></a><span class="sd">        If raw=True, return variable objects. Otherwise, return .get() results.</span>
</span><span id="UtilityModule.getBlockValues-28"><a href="#UtilityModule.getBlockValues-28"><span class="linenos">28</span></a><span class="sd">        &quot;&quot;&quot;</span>
</span><span id="UtilityModule.getBlockValues-29"><a href="#UtilityModule.getBlockValues-29"><span class="linenos">29</span></a>        <span class="n">output</span> <span class="o">=</span> <span class="p">{}</span>
</span><span id="UtilityModule.getBlockValues-30"><a href="#UtilityModule.getBlockValues-30"><span class="linenos">30</span></a>        <span class="k">for</span> <span class="n">key</span><span class="p">,</span> <span class="n">var</span> <span class="ow">in</span> <span class="n">wrapper</span><span class="o">.</span><span class="n">items</span><span class="p">():</span>
</span><span id="UtilityModule.getBlockValues-31"><a href="#UtilityModule.getBlockValues-31"><span class="linenos">31</span></a>            <span class="k">if</span> <span class="nb">hasattr</span><span class="p">(</span><span class="n">var</span><span class="p">,</span> <span class="s2">&quot;get&quot;</span><span class="p">)</span> <span class="ow">and</span> <span class="ow">not</span> <span class="n">raw</span><span class="p">:</span>
</span><span id="UtilityModule.getBlockValues-32"><a href="#UtilityModule.getBlockValues-32"><span class="linenos">32</span></a>                <span class="n">output</span><span class="p">[</span><span class="n">key</span><span class="p">]</span> <span class="o">=</span> <span class="n">var</span><span class="o">.</span><span class="n">get</span><span class="p">()</span>
</span><span id="UtilityModule.getBlockValues-33"><a href="#UtilityModule.getBlockValues-33"><span class="linenos">33</span></a>            <span class="k">else</span><span class="p">:</span>
</span><span id="UtilityModule.getBlockValues-34"><a href="#UtilityModule.getBlockValues-34"><span class="linenos">34</span></a>                <span class="n">output</span><span class="p">[</span><span class="n">key</span><span class="p">]</span> <span class="o">=</span> <span class="n">var</span>
</span><span id="UtilityModule.getBlockValues-35"><a href="#UtilityModule.getBlockValues-35"><span class="linenos">35</span></a>        <span class="k">return</span> <span class="n">output</span>
</span></pre></div>


            <div class="docstring"><p>Extract a dictionary of current values from a DataBlockWrapper.</p>

<p>If raw=True, return variable objects. Otherwise, return .get() results.</p>
</div>


                            </div>
                            <div id="UtilityModule.updateBlockFromDict" class="classattr">
                                        <input id="UtilityModule.updateBlockFromDict-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
                    <div class="decorator decorator-staticmethod">@staticmethod</div>

        <span class="def">def</span>
        <span class="name">updateBlockFromDict</span><span class="signature pdoc-code condensed">(<span class="param"><span class="n">wrapper</span>, </span><span class="param"><span class="n">data</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="UtilityModule.updateBlockFromDict-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#UtilityModule.updateBlockFromDict"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="UtilityModule.updateBlockFromDict-37"><a href="#UtilityModule.updateBlockFromDict-37"><span class="linenos">37</span></a>    <span class="nd">@staticmethod</span>
</span><span id="UtilityModule.updateBlockFromDict-38"><a href="#UtilityModule.updateBlockFromDict-38"><span class="linenos">38</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">updateBlockFromDict</span><span class="p">(</span><span class="n">wrapper</span><span class="p">,</span> <span class="n">data</span><span class="p">):</span>
</span><span id="UtilityModule.updateBlockFromDict-39"><a href="#UtilityModule.updateBlockFromDict-39"><span class="linenos">39</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="UtilityModule.updateBlockFromDict-40"><a href="#UtilityModule.updateBlockFromDict-40"><span class="linenos">40</span></a><span class="sd">        Given a dict of values, update the DataBlockWrapper by setting matching keys.</span>
</span><span id="UtilityModule.updateBlockFromDict-41"><a href="#UtilityModule.updateBlockFromDict-41"><span class="linenos">41</span></a><span class="sd">        &quot;&quot;&quot;</span>
</span><span id="UtilityModule.updateBlockFromDict-42"><a href="#UtilityModule.updateBlockFromDict-42"><span class="linenos">42</span></a>        <span class="k">for</span> <span class="n">key</span><span class="p">,</span> <span class="n">value</span> <span class="ow">in</span> <span class="n">data</span><span class="o">.</span><span class="n">items</span><span class="p">():</span>
</span><span id="UtilityModule.updateBlockFromDict-43"><a href="#UtilityModule.updateBlockFromDict-43"><span class="linenos">43</span></a>            <span class="k">try</span><span class="p">:</span>
</span><span id="UtilityModule.updateBlockFromDict-44"><a href="#UtilityModule.updateBlockFromDict-44"><span class="linenos">44</span></a>                <span class="n">wrapper</span><span class="o">.</span><span class="n">set</span><span class="p">(</span><span class="n">key</span><span class="p">,</span> <span class="n">value</span><span class="p">)</span>
</span><span id="UtilityModule.updateBlockFromDict-45"><a href="#UtilityModule.updateBlockFromDict-45"><span class="linenos">45</span></a>            <span class="k">except</span> <span class="ne">KeyError</span><span class="p">:</span>
</span><span id="UtilityModule.updateBlockFromDict-46"><a href="#UtilityModule.updateBlockFromDict-46"><span class="linenos">46</span></a>                <span class="k">continue</span>
</span></pre></div>


            <div class="docstring"><p>Given a dict of values, update the DataBlockWrapper by setting matching keys.</p>
</div>


                            </div>
                            <div id="UtilityModule.isBlockValid" class="classattr">
                                        <input id="UtilityModule.isBlockValid-view-source" class="view-source-toggle-state" type="checkbox" aria-hidden="true" tabindex="-1">
<div class="attr function">
                    <div class="decorator decorator-staticmethod">@staticmethod</div>

        <span class="def">def</span>
        <span class="name">isBlockValid</span><span class="signature pdoc-code condensed">(<span class="param"><span class="n">wrapper</span>, </span><span class="param"><span class="n">validatorResolver</span></span><span class="return-annotation">):</span></span>

                <label class="view-source-button" for="UtilityModule.isBlockValid-view-source"><span>View Source</span></label>

    </div>
    <a class="headerlink" href="#UtilityModule.isBlockValid"></a>
            <div class="pdoc-code codehilite"><pre><span></span><span id="UtilityModule.isBlockValid-48"><a href="#UtilityModule.isBlockValid-48"><span class="linenos">48</span></a>    <span class="nd">@staticmethod</span>
</span><span id="UtilityModule.isBlockValid-49"><a href="#UtilityModule.isBlockValid-49"><span class="linenos">49</span></a>    <span class="k">def</span><span class="w"> </span><span class="nf">isBlockValid</span><span class="p">(</span><span class="n">wrapper</span><span class="p">,</span> <span class="n">validatorResolver</span><span class="p">):</span>
</span><span id="UtilityModule.isBlockValid-50"><a href="#UtilityModule.isBlockValid-50"><span class="linenos">50</span></a><span class="w">        </span><span class="sd">&quot;&quot;&quot;</span>
</span><span id="UtilityModule.isBlockValid-51"><a href="#UtilityModule.isBlockValid-51"><span class="linenos">51</span></a><span class="sd">        Check if all fields in a block pass their validator.</span>
</span><span id="UtilityModule.isBlockValid-52"><a href="#UtilityModule.isBlockValid-52"><span class="linenos">52</span></a><span class="sd">        The validatorResolver must take a string name and return a function.</span>
</span><span id="UtilityModule.isBlockValid-53"><a href="#UtilityModule.isBlockValid-53"><span class="linenos">53</span></a><span class="sd">        &quot;&quot;&quot;</span>
</span><span id="UtilityModule.isBlockValid-54"><a href="#UtilityModule.isBlockValid-54"><span class="linenos">54</span></a>        <span class="k">for</span> <span class="n">f</span> <span class="ow">in</span> <span class="n">wrapper</span><span class="o">.</span><span class="n">meta</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;fields&quot;</span><span class="p">,</span> <span class="p">[]):</span>
</span><span id="UtilityModule.isBlockValid-55"><a href="#UtilityModule.isBlockValid-55"><span class="linenos">55</span></a>            <span class="n">name</span> <span class="o">=</span> <span class="n">f</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;validation&quot;</span><span class="p">)</span>
</span><span id="UtilityModule.isBlockValid-56"><a href="#UtilityModule.isBlockValid-56"><span class="linenos">56</span></a>            <span class="n">var</span> <span class="o">=</span> <span class="n">f</span><span class="o">.</span><span class="n">get</span><span class="p">(</span><span class="s2">&quot;variable&quot;</span><span class="p">)</span>
</span><span id="UtilityModule.isBlockValid-57"><a href="#UtilityModule.isBlockValid-57"><span class="linenos">57</span></a>            <span class="k">if</span> <span class="n">name</span> <span class="ow">and</span> <span class="n">var</span> <span class="ow">and</span> <span class="nb">callable</span><span class="p">(</span><span class="n">validatorResolver</span><span class="p">(</span><span class="n">name</span><span class="p">)):</span>
</span><span id="UtilityModule.isBlockValid-58"><a href="#UtilityModule.isBlockValid-58"><span class="linenos">58</span></a>                <span class="n">valid</span> <span class="o">=</span> <span class="n">validatorResolver</span><span class="p">(</span><span class="n">name</span><span class="p">)(</span><span class="n">var</span><span class="o">.</span><span class="n">get</span><span class="p">())</span>
</span><span id="UtilityModule.isBlockValid-59"><a href="#UtilityModule.isBlockValid-59"><span class="linenos">59</span></a>                <span class="k">if</span> <span class="ow">not</span> <span class="n">valid</span><span class="p">:</span>
</span><span id="UtilityModule.isBlockValid-60"><a href="#UtilityModule.isBlockValid-60"><span class="linenos">60</span></a>                    <span class="k">return</span> <span class="kc">False</span>
</span><span id="UtilityModule.isBlockValid-61"><a href="#UtilityModule.isBlockValid-61"><span class="linenos">61</span></a>        <span class="k">return</span> <span class="kc">True</span>
</span></pre></div>


            <div class="docstring"><p>Check if all fields in a block pass their validator.
The validatorResolver must take a string name and return a function.</p>
</div>


                            </div>
                </section>
    </main>
<script>
    function escapeHTML(html) {
        return document.createElement('div').appendChild(document.createTextNode(html)).parentNode.innerHTML;
    }

    const originalContent = document.querySelector("main.pdoc");
    let currentContent = originalContent;

    function setContent(innerHTML) {
        let elem;
        if (innerHTML) {
            elem = document.createElement("main");
            elem.classList.add("pdoc");
            elem.innerHTML = innerHTML;
        } else {
            elem = originalContent;
        }
        if (currentContent !== elem) {
            currentContent.replaceWith(elem);
            currentContent = elem;
        }
    }

    function getSearchTerm() {
        return (new URL(window.location)).searchParams.get("search");
    }

    const searchBox = document.querySelector(".pdoc input[type=search]");
    searchBox.addEventListener("input", function () {
        let url = new URL(window.location);
        if (searchBox.value.trim()) {
            url.hash = "";
            url.searchParams.set("search", searchBox.value);
        } else {
            url.searchParams.delete("search");
        }
        history.replaceState("", "", url.toString());
        onInput();
    });
    window.addEventListener("popstate", onInput);


    let search, searchErr;

    async function initialize() {
        try {
            search = await new Promise((resolve, reject) => {
                const script = document.createElement("script");
                script.type = "text/javascript";
                script.async = true;
                script.onload = () => resolve(window.pdocSearch);
                script.onerror = (e) => reject(e);
                script.src = "search.js";
                document.getElementsByTagName("head")[0].appendChild(script);
            });
        } catch (e) {
            console.error("Cannot fetch pdoc search index");
            searchErr = "Cannot fetch search index.";
        }
        onInput();

        document.querySelector("nav.pdoc").addEventListener("click", e => {
            if (e.target.hash) {
                searchBox.value = "";
                searchBox.dispatchEvent(new Event("input"));
            }
        });
    }

    function onInput() {
        setContent((() => {
            const term = getSearchTerm();
            if (!term) {
                return null
            }
            if (searchErr) {
                return `<h3>Error: ${searchErr}</h3>`
            }
            if (!search) {
                return "<h3>Searching...</h3>"
            }

            window.scrollTo({top: 0, left: 0, behavior: 'auto'});

            const results = search(term);

            let html;
            if (results.length === 0) {
                html = `No search results for '${escapeHTML(term)}'.`
            } else {
                html = `<h4>${results.length} search result${results.length > 1 ? "s" : ""} for '${escapeHTML(term)}'.</h4>`;
            }
            for (let result of results.slice(0, 10)) {
                let doc = result.doc;
                let url = `${doc.modulename.replaceAll(".", "/")}.html`;
                if (doc.qualname) {
                    url += `#${doc.qualname}`;
                }

                let heading;
                switch (result.doc.kind) {
                    case "function":
                        if (doc.fullname.endsWith(".__init__")) {
                            heading = `<span class="name">${doc.fullname.replace(/\.__init__$/, "")}</span>${doc.signature}`;
                        } else {
                            heading = `<span class="def">${doc.funcdef}</span> <span class="name">${doc.fullname}</span>${doc.signature}`;
                        }
                        break;
                    case "class":
                        heading = `<span class="def">class</span> <span class="name">${doc.fullname}</span>`;
                        if (doc.bases)
                            heading += `<wbr>(<span class="base">${doc.bases}</span>)`;
                        heading += `:`;
                        break;
                    case "variable":
                        heading = `<span class="name">${doc.fullname}</span>`;
                        if (doc.annotation)
                            heading += `<span class="annotation">${doc.annotation}</span>`;
                        if (doc.default_value)
                            heading += `<span class="default_value"> = ${doc.default_value}</span>`;
                        break;
                    default:
                        heading = `<span class="name">${doc.fullname}</span>`;
                        break;
                }
                html += `
                        <section class="search-result">
                        <a href="${url}" class="attr ${doc.kind}">${heading}</a>
                        <div class="docstring">${doc.doc}</div>
                        </section>
                    `;

            }
            return html;
        })());
    }

    if (getSearchTerm()) {
        initialize();
        searchBox.value = getSearchTerm();
        onInput();
    } else {
        searchBox.addEventListener("focus", initialize, {once: true});
    }

    searchBox.addEventListener("keydown", e => {
        if (["ArrowDown", "ArrowUp", "Enter"].includes(e.key)) {
            let focused = currentContent.querySelector(".search-result.focused");
            if (!focused) {
                currentContent.querySelector(".search-result").classList.add("focused");
            } else if (
                e.key === "ArrowDown"
                && focused.nextElementSibling
                && focused.nextElementSibling.classList.contains("search-result")
            ) {
                focused.classList.remove("focused");
                focused.nextElementSibling.classList.add("focused");
                focused.nextElementSibling.scrollIntoView({
                    behavior: "smooth",
                    block: "nearest",
                    inline: "nearest"
                });
            } else if (
                e.key === "ArrowUp"
                && focused.previousElementSibling
                && focused.previousElementSibling.classList.contains("search-result")
            ) {
                focused.classList.remove("focused");
                focused.previousElementSibling.classList.add("focused");
                focused.previousElementSibling.scrollIntoView({
                    behavior: "smooth",
                    block: "nearest",
                    inline: "nearest"
                });
            } else if (
                e.key === "Enter"
            ) {
                focused.querySelector("a").click();
            }
        }
    });
</script></body>
</html>

</details>
